@@ -198,6 +198,14 @@ typedef	uint32_t	OSType;
 #define OSTYPE_DECLARED
 #endif
 
+enum _NSDirectoryEnumerationOptions
+  {
+    NSDirectoryEnumerationSkipsSubdirectoryDescendants = 1L << 0,
+    NSDirectoryEnumerationSkipsPackageDescendants = 1L << 1,
+    NSDirectoryEnumerationSkipsHiddenFiles = 1L << 2
+  };
+typedef NSUInteger NSDirectoryEnumerationOptions; 
+  
 @interface NSFileManager : NSObject
 {
 #if	GS_EXPOSE(NSFileManager)
@@ -299,9 +307,26 @@ typedef	uint32_t	OSType;
 - (BOOL) contentsEqualAtPath: (NSString*)path1
 		     andPath: (NSString*)path2;
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+/**
+ * Returns an array of NSURL of the contents of the specified directory. <br>
+ * The listing is shallow and does not recurse into subdirectories.
+ * The special files '.' and '..' are excluded but it can return
+ * hidden files.<br>
+ * The only mask option supported is
+ * NSDirectoryEnumerationSkipsHiddenFiles.<br>
+ * The current implementation handles only files and property keys are ignored.
+ */
+- (NSArray*) contentsOfDirectoryAtURL: (NSURL*)url
+           includingPropertiesForKeys: (NSArray*)keys
+                              options: (NSDirectoryEnumerationOptions)mask
+                                error: (NSError **)error;
+#endif
+  
 #if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
 /**
- * Returns an array of the contents of the specified directory.<br />
+ * Returns an array of NSStrings of the contents of the
+ * specified directory.<br />
  * The listing does <strong>not</strong> recursively list subdirectories.<br />
  * The special files '.' and '..' are not listed.<br />
  * Indicates an error by returning nil (eg. if path is not a directory or
