@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>GSObjCRuntime function and macro reference</title>
-   $Date: 2014-07-04 12:10:08 +0100 (Fri, 04 Jul 2014) $ $Revision: 37978 $
+   $Date: 2015-10-08 10:13:32 +0100 (Thu, 08 Oct 2015) $ $Revision: 39042 $
    */
 
 #import "common.h"
@@ -1090,23 +1090,23 @@ GSObjCGetVal(NSObject *self, const char *key, SEL sel,
 	    }
 	    break;
 
-#if     defined(_C_BOOL)
+#if __GNUC__ > 2 && defined(_C_BOOL)
           case _C_BOOL:
             {
-              bool      v;
+              _Bool     v;
  
               if (sel == 0)
                 {
-                  v = *(bool *)((char *)self + offset);
+                  v = *(_Bool *)((char *)self + offset);
                 }
               else
                 {
-                  bool  (*imp)(id, SEL) =
-                    (bool (*)(id, SEL))[self methodForSelector: sel];
+                  _Bool (*imp)(id, SEL) =
+                    (_Bool (*)(id, SEL))[self methodForSelector: sel];
  
                   v = (*imp)(self, sel);
                 }
-              val = [NSNumber numberWithBool: v];
+              val = [NSNumber numberWithBool: (BOOL)v];
             }
             break;
 #endif
@@ -1555,21 +1555,21 @@ GSObjCSetVal(NSObject *self, const char *key, id val, SEL sel,
 	    }
 	    break;
 
-#if     defined(_C_BOOL)
+#if __GNUC__ > 2 && defined(_C_BOOL)
           case _C_BOOL:
             {
-              bool      v = [val boolValue];
+              _Bool     v = (_Bool)[val boolValue];
  
               if (sel == 0)
                 {
-                  bool *ptr = (bool*)((char *)self + offset);
+                  _Bool *ptr = (_Bool*)((char *)self + offset);
  
                   *ptr = v;
                 }
               else
                 {
-                  void  (*imp)(id, SEL, bool) =
-                    (void (*)(id, SEL, bool))[self methodForSelector: sel];
+                  void  (*imp)(id, SEL, _Bool) =
+                    (void (*)(id, SEL, _Bool))[self methodForSelector: sel];
  
                   (*imp)(self, sel, v);
                 }
