@@ -31,16 +31,23 @@
  */
 @implementation NSCalendarDate (GNUstepBase)
 
-/**
- * The ISO standard week of the year is based on the first week of the
- * year being that week (starting on monday) for which the thursday
- * is on or after the first of january.<br />
- * This has the effect that, if january first is a friday, saturday or
- * sunday, the days of that week (up to and including the sunday) are
- * considered to be in week 53 of the preceding year. Similarly if the
- * last day of the year is a monday tuesday or wednesday, these days are
- * part of week 1 of the next year.
- */
+- (NSUInteger) isoYear
+{
+  NSUInteger year = [self yearOfCommonEra];
+  NSUInteger week = [self weekOfYear];
+  NSUInteger month = [self monthOfYear];
+
+  if (week == 1 && month == 12)
+    {
+      year++;
+    }
+  else if (week >= 52 && month == 1)
+    {
+      year--;
+    }
+  return year;
+}
+
 - (NSInteger) weekOfYear
 {
   NSInteger	dayOfWeek = [self dayOfWeek];
