@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2013-07-03 07:46:41 +0100 (Wed, 03 Jul 2013) $ $Revision: 36810 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
    */
 
 #import "common.h"
@@ -84,8 +84,8 @@ static Class	GSInlineArrayClass;
   NSString     *reason;
 
   info = [NSDictionary dictionaryWithObjectsAndKeys:
-    [NSNumber numberWithUnsignedInt: index], @"Index",
-    [NSNumber numberWithUnsignedInt: _count], @"Count",
+    [NSNumber numberWithUnsignedInteger: index], @"Index",
+    [NSNumber numberWithUnsignedInteger: _count], @"Count",
     self, @"Array", nil, nil];
 
   reason = [NSString stringWithFormat:
@@ -325,6 +325,11 @@ static Class	GSInlineArrayClass;
   return nil;
 }
 
+- (BOOL) makeImmutable
+{
+  return YES;
+}
+
 - (id) objectAtIndex: (NSUInteger)index
 {
   if (index >= _count)
@@ -387,6 +392,7 @@ static Class	GSInlineArrayClass;
   state->state += count;
   return count;
 }
+
 @end
 
 #if	!GS_WITH_GC
@@ -591,7 +597,7 @@ static Class	GSInlineArrayClass;
       NSDictionary *info;
 
       info = [NSDictionary dictionaryWithObjectsAndKeys:
-	[NSNumber numberWithUnsignedInt: index], @"Index",
+	[NSNumber numberWithUnsignedInteger: index], @"Index",
 	self, @"Array", nil, nil];
 
       exception = [NSException exceptionWithName: NSInvalidArgumentException
@@ -631,12 +637,50 @@ static Class	GSInlineArrayClass;
   _version++;
 }
 
+- (BOOL) makeImmutable
+{
+  GSClassSwizzle(self, [GSArray class]);
+  return YES;
+}
+
 - (id) makeImmutableCopyOnFail: (BOOL)force
 {
   GSClassSwizzle(self, [GSArray class]);
   return self;
 }
 
+- (void) removeAllObjects
+{
+  NSUInteger    pos;
+
+  if ((pos = _count) > 0)
+    {
+#if	GS_WITH_GC == 0
+      IMP       rel = 0;
+      Class    last = Nil;
+#endif
+
+      _version++;
+      _count = 0;
+      while (pos-- > 0)
+        {
+#if	GS_WITH_GC == 0
+          id    o = _contents_array[pos];
+          Class c = object_getClass(o);
+
+          if (c != last)
+            {
+              last = c;
+              rel = [o methodForSelector: @selector(release)];
+            }
+          (*rel)(o, @selector(release));
+#endif
+          _contents_array[pos] = nil;
+        }
+      _version++;
+    }
+}
+
 - (void) removeLastObject
 {
   _version++;
@@ -757,6 +801,60 @@ static Class	GSInlineArrayClass;
   _version++;
 }
 
+- (void) removeObjectsInRange: (NSRange)aRange
+{
+  GS_RANGE_CHECK(aRange, _count);
+
+  if (aRange.length > 0)
+    {
+      NSUInteger        index;
+      NSUInteger        tail;
+      NSUInteger        end;
+#if	GS_WITH_GC == 0
+      IMP       rel = 0;
+      Class    last = Nil;
+#endif
+
+      _version++;
+      index = aRange.location;
+
+#if	GS_WITH_GC == 0
+      /* Release all the objects we are removing.
+       */
+      end = NSMaxRange(aRange);
+      while (end-- > index)
+        {
+          id    o = _contents_array[end];
+          Class c = object_getClass(o);
+
+          if (c != last)
+            {
+              last = c;
+              rel = [o methodForSelector: @selector(release)];
+            }
+          (*rel)(o, @selector(release));
+          _contents_array[end] = nil;
+        }
+#endif
+      /* Move any trailing objects to fill the hole we made.
+       */
+      end = NSMaxRange(aRange);
+      tail = _count - end;
+      if (tail > 0)
+        {
+          memmove(_contents_array + index, _contents_array + end,
+            tail * sizeof(id));
+          index += tail;
+        }
+      _count = index;
+
+      /* Clear emptied part of buffer.
+       */
+      memset(_contents_array + _count, 0, aRange.length * sizeof(id));
+      _version++;
+    }
+}
+
 - (void) replaceObjectAtIndex: (NSUInteger)index withObject: (id)anObject
 {
   id	obj;
@@ -772,7 +870,7 @@ static Class	GSInlineArrayClass;
       NSDictionary *info;
 
       info = [NSDictionary dictionaryWithObjectsAndKeys:
-	[NSNumber numberWithUnsignedInt: index], @"Index",
+	[NSNumber numberWithUnsignedInteger: index], @"Index",
         _contents_array[index], @"OldObject",
 	self, @"Array", nil, nil];
 
@@ -884,6 +982,23 @@ static Class	GSInlineArrayClass;
   state->itemsPtr = stackbuf;
   return count;
 }
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = GSPrivateMemorySize(self, exclude);
+
+  if (size > 0)
+    {
+      NSUInteger	count = _count;
+
+      size += _capacity*sizeof(void*);
+      while (count-- > 0)
+	{
+	  size += [_contents_array[count] sizeInBytesExcluding: exclude];
+	}
+    }
+  return size;
+}
 @end
 
 
@@ -1050,6 +1165,7 @@ static Class	GSInlineArrayClass;
     }
   return index;
 }
+
 @end
 
 @implementation	GSPlaceholderArray
