@@ -34,6 +34,8 @@
 
 #import "GNUstepBase/GSObjCRuntime.h"
 
+#import "GSPrivate.h"
+
 /*
  *	The 'Fastmap' stuff provides an inline implementation of a mapping
  *	table - for maximum performance.
@@ -336,6 +338,11 @@ static SEL	objSel;
     NSDefaultMallocZone()] initWithDictionary: self]);
 }
 
+- (BOOL) makeImmutable
+{
+  return YES;
+}
+
 - (NSEnumerator*) objectEnumerator
 {
   return AUTORELEASE([[GSDictionaryObjectEnumerator allocWithZone:
@@ -364,6 +371,28 @@ static SEL	objSel;
   return GSIMapCountByEnumeratingWithStateObjectsCount
     (&map, state, stackbuf, len);
 }
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = GSPrivateMemorySize(self, exclude);
+
+  if (size > 0)
+    {
+      GSIMapEnumerator_t	enumerator = GSIMapEnumeratorForMap(&map);
+      GSIMapNode 		node = GSIMapEnumeratorNextNode(&enumerator);
+
+      size += GSIMapSize(&map) - sizeof(map);
+      while (node != 0)
+        {
+          size += [node->key.obj sizeInBytesExcluding: exclude];
+          size += [node->value.obj sizeInBytesExcluding: exclude];
+          node = GSIMapEnumeratorNextNode(&enumerator);
+        }
+      GSIMapEndEnumerator(&enumerator);
+    }
+  return size;
+}
+
 @end
 
 @implementation GSMutableDictionary
@@ -395,6 +424,12 @@ static SEL	objSel;
   return self;
 }
 
+- (BOOL) makeImmutable
+{
+  GSClassSwizzle(self, [GSDictionary class]);
+  return YES;
+}
+
 - (id) makeImmutableCopyOnFail: (BOOL)force
 {
   GSClassSwizzle(self, [GSDictionary class]);
