@@ -22,7 +22,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2012-03-01 09:14:08 +0000 (Thu, 01 Mar 2012) $ $Revision: 34852 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
 */
 
 #import "common.h"
@@ -69,7 +69,7 @@ typedef struct{
   unsigned	maxWatchers;
   NSTimer	*housekeeper;	/** Housekeeping timer for loop.	*/
 @private
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   NSMapTable    *handleMap;     
   NSMapTable	*winMsgMap;
 #else
