@@ -48,7 +48,7 @@ static GC_descr	nodeDesc;	// Type descriptor for map node.
 #include "GNUstepBase/GSIMap.h"
 
 static SEL	memberSel;
-
+static SEL      privateCountOfSel;
 @interface GSSet : NSSet
 {
 @public
@@ -128,6 +128,7 @@ static Class	mutableSetClass;
       setClass = [GSSet class];
       mutableSetClass = [GSMutableSet class];
       memberSel = @selector(member:);
+      privateCountOfSel = @selector(_countForObject:);
     }
 }
 
@@ -424,8 +425,10 @@ static Class	mutableSetClass;
 	      GSIMapEnumerator_t	enumerator;
 	      GSIMapNode 		node;
 	      IMP			imp;
+              IMP                       countImp;
 
 	      imp = [other methodForSelector: memberSel];
+              countImp = [other methodForSelector: privateCountOfSel];
 	      enumerator = GSIMapEnumeratorForMap(&map);
 	      node = GSIMapEnumeratorNextNode(&enumerator);
 
@@ -436,6 +439,16 @@ static Class	mutableSetClass;
 		      GSIMapEndEnumerator(&enumerator);
 		      return NO;
 		    }
+                  else
+                    {
+                      NSUInteger c = (NSUInteger)
+                       countImp(other,privateCountOfSel,node->key.obj);
+                      // GSSet does not have duplicate entries
+                      if (c != 1)
+                        {
+                          return NO;
+                        }
+                    }
 		  node = GSIMapEnumeratorNextNode(&enumerator);
 		}
 	      GSIMapEndEnumerator(&enumerator);
@@ -445,6 +458,11 @@ static Class	mutableSetClass;
     }
 }
 
+- (BOOL) makeImmutable
+{
+  return YES;
+}
+
 - (void) makeObjectsPerform: (SEL)aSelector
 {
   GSIMapEnumerator_t	enumerator = GSIMapEnumeratorForMap(&map);
@@ -524,6 +542,27 @@ static Class	mutableSetClass;
   return GSIMapCountByEnumeratingWithStateObjectsCount
     (&map, state, stackbuf, len);
 }
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = GSPrivateMemorySize(self, exclude);
+
+  if (size > 0)
+    {
+      GSIMapEnumerator_t	enumerator = GSIMapEnumeratorForMap(&map);
+      GSIMapNode 		node = GSIMapEnumeratorNextNode(&enumerator);
+
+      size += GSIMapSize(&map) - sizeof(map);
+      while (node != 0)
+        {
+          size += [node->key.obj sizeInBytesExcluding: exclude];
+          node = GSIMapEnumeratorNextNode(&enumerator);
+        }
+      GSIMapEndEnumerator(&enumerator);
+    }
+  return size;
+}
+
 @end
 
 @implementation GSMutableSet
@@ -701,6 +740,12 @@ static Class	mutableSetClass;
     }
 }
 
+- (BOOL) makeImmutable
+{
+  GSClassSwizzle(self, [GSSet class]);
+  return YES;
+}
+
 - (id) makeImmutableCopyOnFail: (BOOL)force
 {
   GSClassSwizzle(self, [GSSet class]);
