@@ -68,7 +68,7 @@ GSPrivateSockaddrLength(struct sockaddr *addr)
 #ifdef	AF_INET6
     case AF_INET6:      return sizeof(struct sockaddr_in6);
 #endif
-#ifndef	__MINGW__
+#ifndef	_WIN32
     case AF_LOCAL:       return sizeof(struct sockaddr_un);
 #endif
     default:            return 0;
@@ -240,7 +240,7 @@ GSPrivateSockaddrSetup(NSString *machine, uint16_t port,
       ((struct sockaddr_in*)(void*)sin)->sin_port = GSSwapHostI16ToBig(port);
     }
 #else
-  ((struct sockaddr_ind*)sin)->sin6_port = GSSwapHostI16ToBig(port);
+  ((struct sockaddr_in*)sin)->sin_port = GSSwapHostI16ToBig(port);
 #endif
   return YES;
 }
@@ -354,6 +354,17 @@ GSPrivateSockaddrSetup(NSString *machine, uint16_t port,
 @public
   GSTLSSession  *session;
 }
+
+/**
+ * Populates the dictionary 'dict', copying in all the properties
+ * of the supplied streams. If a property is set for both then
+ * the output stream's one has precedence.
+ */
++ (void) populateProperties: (NSMutableDictionary**)dict
+            withTLSPriority: (NSString*)pri
+            fromInputStream: (NSStream*)i
+             orOutputStream: (NSStream*)o;
+
 @end
 
 /* Callback to allow the TLS code to pull data from the remote system.
@@ -443,6 +454,37 @@ static NSArray  *keys = nil;
     }
 }
 
++ (void) populateProperties: (NSMutableDictionary**)dict
+	    withTLSPriority: (NSString*)pri
+	    fromInputStream: (NSStream*)i
+	     orOutputStream: (NSStream*)o
+{
+  NSString              *str;
+  NSMutableDictionary   *opts = *dict;
+  NSUInteger            count;
+  
+  if (NULL != dict)
+    {
+      if (nil != pri)
+	{
+	  [opts setObject: pri forKey: GSTLSPriority];
+	}
+      count = [keys count];
+      while (count-- > 0)
+	{
+	  NSString  *key = [keys objectAtIndex: count];
+
+	  str = [o propertyForKey: key];
+	  if (nil == str) str = [i propertyForKey: key];
+	  if (nil != str) [opts setObject: str forKey: key];
+	}
+    }
+  else
+    {
+      NSWarnLog(@"%@ requires not nil 'dict'", NSStringFromSelector(_cmd));
+    }
+}
+
 + (void) tryInput: (GSSocketInputStream*)i output: (GSSocketOutputStream*)o
 {
   NSString      *tls;
@@ -537,10 +579,10 @@ static NSArray  *keys = nil;
 {
   NSString              *str;
   NSMutableDictionary   *opts;
-  NSUInteger            count;
   BOOL		        server;
 
-  server = [[o propertyForKey: @"IsServer"] boolValue];
+  // Check whether the input stream has been accepted by a listening socket
+  server = [[i propertyForKey: @"IsServer"] boolValue];
 
   str = [o propertyForKey: NSStreamSocketSecurityLevelKey];
   if (nil == str) str = [i propertyForKey: NSStreamSocketSecurityLevelKey];
@@ -580,16 +622,10 @@ static NSArray  *keys = nil;
    * properties.  GSTLSPriority overrides NSStreamSocketSecurityLevelKey.
    */
   opts = [NSMutableDictionary new];
-  if (nil != str) [opts setObject: str forKey: GSTLSPriority];
-  count = [keys count];
-  while (count-- > 0)
-    {
-      NSString  *key = [keys objectAtIndex: count];
-
-      str = [o propertyForKey: key];
-      if (nil == str) str = [i propertyForKey: key];
-      if (nil != str) [opts setObject: str forKey: key];
-    }
+  [[self class] populateProperties: &opts
+		   withTLSPriority: str
+		   fromInputStream: i
+		    orOutputStream: o];
   
   session = [[GSTLSSession alloc] initWithOptions: opts
                                         direction: (server ? NO : YES)
@@ -693,6 +729,59 @@ static NSArray  *keys = nil;
 @interface      GSTLSHandler : GSStreamHandler
 @end
 @implementation GSTLSHandler
+
+static NSArray  *keys = nil;
+
++ (void) initialize
+{
+  if (nil == keys)
+    {
+      keys = [[NSArray alloc] initWithObjects:
+        GSTLSCAFile,
+        GSTLSCertificateFile,
+        GSTLSCertificateKeyFile,
+        GSTLSCertificateKeyPassword,
+        GSTLSDebug,
+        GSTLSPriority,
+        GSTLSRemoteHosts,
+        GSTLSRevokeFile,
+        GSTLSVerify,
+        nil];
+      [[NSObject leakAt: &keys] release];
+    }
+}
+
++ (void) populateProperties: (NSMutableDictionary**)dict
+	    withTLSPriority: (NSString*)pri
+	    fromInputStream: (NSStream*)i
+	     orOutputStream: (NSStream*)o
+{
+  NSString              *str;
+  NSMutableDictionary   *opts = *dict;
+  NSUInteger            count;
+  
+  if (NULL != dict)
+    {
+      if (nil != pri)
+	{
+	  [opts setObject: pri forKey: GSTLSPriority];
+	}
+      count = [keys count];
+      while (count-- > 0)
+	{
+	  NSString  *key = [keys objectAtIndex: count];
+
+	  str = [o propertyForKey: key];
+	  if (nil == str) str = [i propertyForKey: key];
+	  if (nil != str) [opts setObject: str forKey: key];
+	}
+    }
+  else
+    {
+      NSWarnLog(@"%@ requires not nil 'dict'", NSStringFromSelector(_cmd));
+    }
+}
+
 + (void) tryInput: (GSSocketInputStream*)i output: (GSSocketOutputStream*)o
 {
   NSString	*tls;
@@ -1316,7 +1405,7 @@ static NSString * const GSSOCKSAckConn = @"GSSOCKSAckConn";
 static inline BOOL
 socketError(int result)
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   return (result == SOCKET_ERROR) ? YES : NO;
 #else
   return (result < 0) ? YES : NO;
@@ -1333,7 +1422,7 @@ socketWouldBlock()
 static void
 setNonBlocking(SOCKET fd)
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   unsigned long dummy = 1;
 
   if (ioctlsocket(fd, FIONBIO, &dummy) == SOCKET_ERROR)
@@ -1373,7 +1462,7 @@ setNonBlocking(SOCKET fd)
       _sibling = nil;
       _closing = NO;
       _passive = NO;
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
       _loopID = WSA_INVALID_EVENT;
 #else
       _loopID = (void*)(intptr_t)-1;
@@ -1518,7 +1607,7 @@ setNonBlocking(SOCKET fd)
         }
 #endif
 
-#ifndef	__MINGW__
+#ifndef	_WIN32
       case AF_LOCAL:
 	{
 	  struct sockaddr_un	peer;
@@ -1552,7 +1641,7 @@ setNonBlocking(SOCKET fd)
 
 - (void) _setLoopID: (void *)ref
 {
-#if	!defined(__MINGW__)
+#if	!defined(_WIN32)
   _sock = (SOCKET)(intptr_t)ref;        // On gnu/linux _sock is _loopID
 #endif
   _loopID = ref;
@@ -1583,7 +1672,7 @@ setNonBlocking(SOCKET fd)
    * monitored, and on mswindows systems we create an event object to be
    * monitored (the socket events are assoociated with this object later).
    */
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   _loopID = CreateEvent(NULL, NO, NO, NULL);
 #else
   _loopID = (void*)(intptr_t)sock;      // On gnu/linux _sock is _loopID
@@ -1679,7 +1768,7 @@ setNonBlocking(SOCKET fd)
            * indication of opened
            */
           [self _setStatus: NSStreamStatusOpening];
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
           WSAEventSelect(_sock, _loopID, FD_ALL_EVENTS);
 #endif
 	  if (NSCountMapTable(_loops) > 0)
@@ -1713,7 +1802,7 @@ setNonBlocking(SOCKET fd)
     }
 
  open_ok:
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   WSAEventSelect(_sock, _loopID, FD_ALL_EVENTS);
 #endif
   [super open];
@@ -1734,7 +1823,8 @@ setNonBlocking(SOCKET fd)
       return;
     }
   [_handler bye];
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
+  [super close];
   if (_sibling && [_sibling streamStatus] != NSStreamStatusClosed)
     {
       /*
@@ -1754,15 +1844,14 @@ setNonBlocking(SOCKET fd)
       closesocket(_sock);
     }
   WSACloseEvent(_loopID);
-  [super close];
   _loopID = WSA_INVALID_EVENT;
 #else
+  [super close];
   // read shutdown is ignored, because the other side may shutdown first.
   if (!_sibling || [_sibling streamStatus] == NSStreamStatusClosed)
     close((intptr_t)_loopID);
   else
     shutdown((intptr_t)_loopID, SHUT_RD);
-  [super close];
   _loopID = (void*)(intptr_t)-1;
 #endif
   _sock = INVALID_SOCKET;
@@ -1803,7 +1892,7 @@ setNonBlocking(SOCKET fd)
     }
   else
     {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
       readLen = recv([self _sock], (char*) buffer, (socklen_t) len, 0);
 #else
       readLen = read([self _sock], buffer, len);
@@ -1857,7 +1946,7 @@ setNonBlocking(SOCKET fd)
 
 - (void) _dispatch
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   AUTORELEASE(RETAIN(self));
   /*
    * Windows only permits a single event to be associated with a socket
@@ -2012,7 +2101,7 @@ setNonBlocking(SOCKET fd)
 #endif
 }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 - (BOOL) runLoopShouldBlock: (BOOL*)trigger
 {
   *trigger = YES;
@@ -2050,7 +2139,7 @@ setNonBlocking(SOCKET fd)
       return 0;
     }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   writeLen = send([self _sock], (char*) buffer, (socklen_t) len, 0);
 #else
   writeLen = write([self _sock], buffer, (socklen_t) len);
@@ -2151,7 +2240,7 @@ setNonBlocking(SOCKET fd)
            * indication of opened
            */
           [self _setStatus: NSStreamStatusOpening];
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
           WSAEventSelect(_sock, _loopID, FD_ALL_EVENTS);
 #endif
 	  if (NSCountMapTable(_loops) > 0)
@@ -2185,7 +2274,7 @@ setNonBlocking(SOCKET fd)
     }
 
  open_ok:
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   WSAEventSelect(_sock, _loopID, FD_ALL_EVENTS);
 #endif
   [super open];
@@ -2208,7 +2297,7 @@ setNonBlocking(SOCKET fd)
       return;
     }
   [_handler bye];
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   if (_sibling && [_sibling streamStatus] != NSStreamStatusClosed)
     {
       /*
@@ -2244,16 +2333,32 @@ setNonBlocking(SOCKET fd)
 
 - (NSInteger) write: (const uint8_t *)buffer maxLength: (NSUInteger)len
 {
+  if (len == 0)
+    {
+      /*
+       *  The method allows the 'len' equal to 0. In this case the 'buffer'
+       *  is ignored. This can be useful if there is a necessity to postpone
+       *  actual writing (for no data are ready for example) without leaving
+       *  the stream in the state of unhandled NSStreamEventHasSpaceAvailable
+       *  (to keep receiving of that event from a runloop).
+       *  The delegate's -[stream:handleEvent:] would keep calling of
+       *  -[write: NULL maxLength: 0] until the delegate's state allows it
+       *  to write actual bytes.
+       *  The downside of that is that it produces a busy wait ... with the
+       *  run loop immediately notifying the stream that it has space to
+       *  write, so care should be taken to ensure that the delegate has a
+       *  near constant supply of data to write, or has some mechanism to
+       *  detect that no more data is arriving, and shut down.
+       */ 
+      _events &= ~NSStreamEventHasSpaceAvailable;
+      return 0;
+    }
+
   if (buffer == 0)
     {
       [NSException raise: NSInvalidArgumentException
 		  format: @"null pointer for buffer"];
     }
-  if (len == 0)
-    {
-      [NSException raise: NSInvalidArgumentException
-		  format: @"zero byte length write requested"];
-    }
 
   if (_handler == nil)
     return [self _write: buffer maxLength: len];
@@ -2263,7 +2368,7 @@ setNonBlocking(SOCKET fd)
 
 - (void) _dispatch
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   AUTORELEASE(RETAIN(self));
   /*
    * Windows only permits a single event to be associated with a socket
@@ -2416,7 +2521,7 @@ setNonBlocking(SOCKET fd)
 #endif
 }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 - (BOOL) runLoopShouldBlock: (BOOL*)trigger
 {
   *trigger = YES;
@@ -2517,7 +2622,7 @@ setNonBlocking(SOCKET fd)
       [self _sendEvent: NSStreamEventErrorOccurred];
       return;
     }
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   WSAEventSelect(_sock, _loopID, FD_ALL_EVENTS);
 #endif
   [super open];
@@ -2525,7 +2630,7 @@ setNonBlocking(SOCKET fd)
 
 - (void) close
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   if (_loopID != WSA_INVALID_EVENT)
     {
       WSACloseEvent(_loopID);
@@ -2550,6 +2655,11 @@ setNonBlocking(SOCKET fd)
 - (void) acceptWithInputStream: (NSInputStream **)inputStream
                   outputStream: (NSOutputStream **)outputStream
 {
+  NSArray *keys;
+  NSUInteger count;
+  NSMutableDictionary *opts;
+  NSString *str;
+
   GSSocketStream *ins = AUTORELEASE([[self _inputStreamClass] new]);
   GSSocketStream *outs = AUTORELEASE([[self _outputStreamClass] new]);
   /* Align on a 2 byte boundary for a 16bit port number in the sockaddr
@@ -2577,18 +2687,51 @@ setNonBlocking(SOCKET fd)
       // no need to connect again
       [ins _setPassive: YES];
       [outs _setPassive: YES];
+
       // copy the addr to outs
       [ins _setAddress: addr];
       [outs _setAddress: addr];
       [ins _setSock: acceptReturn];
       [outs _setSock: acceptReturn];
-      [ins _setStatus: NSStreamStatusOpen];
-      [outs _setStatus: NSStreamStatusOpen];
+
       /* Set property to indicate that the input stream was accepted by
        * a listening socket (server) rather than produced by an outgoing
        * connection (client).
        */
       [ins setProperty: @"YES" forKey: @"IsServer"];
+
+      /* At this point, we can insert the handler to deal with TLS
+       */
+      str = [self propertyForKey: NSStreamSocketSecurityLevelKey];
+      if (nil != str)
+	{
+	  opts = [NSMutableDictionary new];
+	  [opts setObject: str forKey: NSStreamSocketSecurityLevelKey];
+	  // copy the properties in the 'opts'
+	  [GSTLSHandler populateProperties: &opts
+			   withTLSPriority: str
+			   fromInputStream: self
+			    orOutputStream: nil];
+	  // and set the input/output streams's properties from the 'opts'
+	  keys = [opts allKeys];
+	  count = [keys count];
+	  while(count-- > 0)
+	    {
+	      NSString *key = [keys objectAtIndex: count];
+	      str = [opts objectForKey: key];
+	      [ins setProperty: str forKey: key];
+	      [outs setProperty: str forKey: key];
+	    }
+
+          /* Set the streams to be 'open' in order to have the TLS
+           * handshake done.  On completion the state will be reset.
+           */
+          [ins _setStatus: NSStreamStatusOpen];
+          [outs _setStatus: NSStreamStatusOpen];
+	  [GSTLSHandler tryInput: (GSSocketInputStream *)ins
+			  output: (GSSocketOutputStream *)outs];
+	  DESTROY(opts);
+	}
     }
   if (inputStream)
     {
@@ -2600,11 +2743,13 @@ setNonBlocking(SOCKET fd)
       [outs _setSibling: ins];
       *outputStream = (NSOutputStream*)outs;
     }
+  /* Now the streams are redy to be opened.
+   */
 }
 
 - (void) _dispatch
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   WSANETWORKEVENTS events;
 
   if (WSAEnumNetworkEvents(_sock, _loopID, &events) == SOCKET_ERROR)
