@@ -88,7 +88,7 @@ NSString * const NSStreamSOCKSProxyVersionKey
  */
 static RunLoopEventType typeForStream(NSStream *aStream)
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   if ([aStream _loopID] == (void*)aStream)
     {
       return ET_TRIGGER;
@@ -125,10 +125,14 @@ static RunLoopEventType typeForStream(NSStream *aStream)
 
 - (void) removeStream: (NSStream*)aStream mode: (NSString*)mode
 {
+  /* We may have added the stream more than once (eg if the stream -open
+   * method was called more than once, so we need to remove all event
+   * registrations.
+   */
   [self removeEvent: [aStream _loopID]
 	       type: typeForStream(aStream)
 	    forMode: mode
-		all: NO];
+		all: YES];
 }
 @end
 
