@@ -42,6 +42,35 @@
 
 #import "GSPrivate.h"
 
+@interface	NSString(gnutlsFileSystemRepresentation)
+- (const char*) gnutlsFileSystemRepresentation;
+@end
+
+@implementation	NSString(gnutlsFileSystemRepresentation)
+- (const char*) gnutlsFileSystemRepresentation
+{
+#if	defined(_WIN32)
+  const unichar	*buf = (const unichar*)[self fileSystemRepresentation];
+  int		len = 0;
+  NSString	*str;
+  const char	*result;
+
+  while (buf[len] > 0)
+    {
+      len++;
+    }
+  str = [[NSString alloc] initWithBytes: buf 
+				 length: len * 2
+			       encoding: NSUnicodeStringEncoding];
+  result = [str UTF8String];
+  RELEASE(str);
+  return result;   
+#else
+  return [self fileSystemRepresentation];
+#endif
+}
+@end
+
 /* Constants to control TLS/SSL (options).
  */
 NSString * const GSTLSCAFile = @"GSTLSCAFile";
@@ -54,6 +83,8 @@ NSString * const GSTLSRemoteHosts = @"GSTLSRemoteHosts";
 NSString * const GSTLSRevokeFile = @"GSTLSRevokeFile";
 NSString * const GSTLSVerify = @"GSTLSVerify";
 
+#if     defined(HAVE_GNUTLS)
+
 static NSString *
 standardizedPath(NSString *path)
 {
@@ -69,8 +100,6 @@ standardizedPath(NSString *path)
   return [path stringByStandardizingPath];
 }
 
-#if     defined(HAVE_GNUTLS)
-
 #if GNUTLS_VERSION_NUMBER <= 0x020b00
 /* Set up locking callbacks for gcrypt so that it will be thread-safe.
  */
@@ -174,63 +203,94 @@ static NSMutableDictionary      *fileMap = nil;
 
 + (void) _defaultsChanged: (NSNotification*)n
 {
-  NSString      *str;
+  NSBundle              *bundle;
+  NSUserDefaults        *defs;
+  NSDictionary          *env;
+  NSString              *str;
 
-  str = [[NSUserDefaults standardUserDefaults] stringForKey: @"GSCipherList"];
+  bundle = [NSBundle bundleForClass: [NSObject class]];
+  defs = [NSUserDefaults standardUserDefaults];
+  env = [[NSProcessInfo processInfo] environment];
+
+  str = [defs stringForKey: @"GSCipherList"];
   if (nil != str)
     {
       GSOnceMLog(@"GSCipherList is no longer used, please try GSTLSPriority");
     }
 
-  str = [[NSUserDefaults standardUserDefaults] stringForKey: GSTLSPriority];
-  if (nil != str)
+  str = [defs stringForKey: GSTLSPriority];
+  if (0 == [str length])
     {
-      ASSIGN(priority, str);
+      str = nil;        // nil or empty string resets to default
     }
+  ASSIGN(priority, str);
+
 
   /* The GSTLSCAFile user default overrides the builtin value or the
    * GS_TLS_CA_FILE environment variable.
    */
-  str = [[NSUserDefaults standardUserDefaults] stringForKey: GSTLSCAFile];
-  str = standardizedPath(str);
-  if (nil != str)
+  str = [defs stringForKey: GSTLSCAFile];
+  if (nil == str)
     {
-      ASSIGN(caFile, str);
+      /* Let the GS_TLS_CA_FILE environment variable override the
+       * default certificate authority location.
+       */
+      str = [env objectForKey: @"GS_TLS_CA_FILE"];
+      if (nil == str)
+        {
+          str = [bundle pathForResource: @"ca-certificates"
+                                 ofType: @"crt"
+                            inDirectory: @"GSTLS"];
+        }
     }
+  str = standardizedPath(str);
+  ASSIGN(caFile, str);
 
   /* The GSTLSRevokeFile user default overrides the builtin value or the
    * GS_TLS_REVOKE environment variable.
    */
-  str = [[NSUserDefaults standardUserDefaults] stringForKey: GSTLSRevokeFile];
+  str = [defs stringForKey: GSTLSRevokeFile];
+  if (nil == str)
+    {
+      /* Let the GS_TLS_REVOKE environment variable override the
+       * default revocation list location.
+       */
+      str = [env objectForKey: @"GS_TLS_REVOKE"];
+      if (nil == str)
+        {
+          str = [bundle pathForResource: @"revoke"
+                                 ofType: @"crl"
+                            inDirectory: @"GSTLS"];
+        }
+    }
   str = standardizedPath(str);
-  if (nil != str)
-    {
-      ASSIGN(revokeFile, str);
-    }
+  ASSIGN(revokeFile, str);
 
-  str = [[NSUserDefaults standardUserDefaults]
-    stringForKey: @"GSTLSVerifyClient"];
-  if (nil != str)
+  str = [defs stringForKey: @"GSTLSVerifyClient"];
+  if (nil == str)
     {
-      verifyClient = [str boolValue];
+      str = [env objectForKey: @"GS_TLS_VERIFY_C"];
     }
+  verifyClient = [str boolValue];
 
-  str = [[NSUserDefaults standardUserDefaults]
-    stringForKey: @"GSTLSVerifyServer"];
-  if (nil != str)
-    {
-      verifyServer = [str boolValue];
+  str = [defs stringForKey: @"GSTLSVerifyServer"];
+  if (nil == str)
+    { 
+      str = [env objectForKey: @"GS_TLS_VERIFY_S"];
     }
+  verifyServer = [str boolValue];
 
-  str = [[NSUserDefaults standardUserDefaults] stringForKey: GSTLSDebug];
-  if (nil != str)
+  str = [defs stringForKey: GSTLSDebug];
+  if (nil == str)
     {
-      globalDebug = [str intValue];
+      str = [env objectForKey: @"GS_TLS_DEBUG"];
     }
+  globalDebug = [str intValue];
   if (globalDebug < 0)
     {
       globalDebug = 0;
     }
+
   gnutls_global_set_log_level(globalDebug);
 }
 
@@ -261,69 +321,11 @@ static NSMutableDictionary      *fileMap = nil;
 
       if (beenHere == NO)
         {
-          NSProcessInfo         *pi;
-          NSBundle              *bundle;
-          NSString              *str;
-
           beenHere = YES;
 
-          bundle = [NSBundle bundleForClass: [NSObject class]];
-
           fileLock = [NSLock new];
           fileMap = [NSMutableDictionary new];
 
-          /* Let the GS_TLS_CA_FILE environment variable override the
-           * default certificate authority location.
-           */
-          pi = [NSProcessInfo processInfo];
-          str = [[pi environment] objectForKey: @"GS_TLS_CA_FILE"];
-          if (nil == str)
-            {
-              str = [bundle pathForResource: @"ca-certificates"
-                                     ofType: @"crt"
-                                inDirectory: @"GSTLS"];
-            }
-          else
-            {
-              str = standardizedPath(str);
-            }
-          ASSIGN(caFile, str);
-
-          /* Let the GS_TLS_REVOKE environment variable override the
-           * default revocation list location.
-           */
-          pi = [NSProcessInfo processInfo];
-          str = [[pi environment] objectForKey: @"GS_TLS_REVOKE"];
-          if (nil == str)
-            {
-              str = [bundle pathForResource: @"revoke"
-                                     ofType: @"crl"
-                                inDirectory: @"GSTLS"];
-            }
-          else
-            {
-              str = standardizedPath(str);
-            }
-          ASSIGN(revokeFile, str);
-
-          str = [[pi environment] objectForKey: @"GS_TLS_VERIFY_C"];
-          if (nil != str)
-            {
-              verifyClient = [str boolValue];
-            }
-
-          str = [[pi environment] objectForKey: @"GS_TLS_VERIFY_S"];
-          if (nil != str)
-            {
-              verifyServer = [str boolValue];
-            }
-
-          str = [[pi environment] objectForKey: @"GS_TLS_DEBUG"];
-          if (nil != str)
-            {
-              globalDebug = [str intValue];
-            }
-
           [[NSNotificationCenter defaultCenter]
             addObserver: self
                selector: @selector(_defaultsChanged:)
@@ -355,6 +357,7 @@ static NSMutableDictionary      *fileMap = nil;
 
 + (void) setData: (NSData*)data forTLSFile: (NSString*)fileName
 {
+  fileName = standardizedPath(fileName);
   if (nil != data && NO == [data isKindOfClass: [NSData class]])
     {
       [NSException raise: NSInvalidArgumentException
@@ -1071,7 +1074,7 @@ static NSMutableDictionary      *credentialsCache = nil;
           const char    *path;
           int           ret;
 
-          path = [dca fileSystemRepresentation];
+          path = [dca gnutlsFileSystemRepresentation];
           ret = gnutls_certificate_set_x509_trust_file(c->certcred,
             path, GNUTLS_X509_FMT_PEM);
           if (ret < 0)
@@ -1100,7 +1103,7 @@ static NSMutableDictionary      *credentialsCache = nil;
           const char    *path;
           int           ret;
 
-          path = [dca fileSystemRepresentation];
+          path = [dca gnutlsFileSystemRepresentation];
           ret = gnutls_certificate_set_x509_trust_file(c->certcred,
             path, GNUTLS_X509_FMT_PEM);
           if (ret < 0)
@@ -1132,7 +1135,7 @@ static NSMutableDictionary      *credentialsCache = nil;
           const char    *path;
           int           ret;
 
-          path = [drv fileSystemRepresentation];
+          path = [drv gnutlsFileSystemRepresentation];
           ret = gnutls_certificate_set_x509_crl_file(c->certcred,
             path, GNUTLS_X509_FMT_PEM);
           if (ret < 0)
@@ -1156,7 +1159,7 @@ static NSMutableDictionary      *credentialsCache = nil;
           const char    *path;
           int           ret;
 
-          path = [rv fileSystemRepresentation];
+          path = [rv gnutlsFileSystemRepresentation];
           ret = gnutls_certificate_set_x509_crl_file(c->certcred,
             path, GNUTLS_X509_FMT_PEM);
           if (ret < 0)
@@ -1494,8 +1497,6 @@ static NSMutableDictionary      *credentialsCache = nil;
             }
           else if ([pri isEqual: NSStreamSocketSecurityLevelSSLv3] == YES)
             {
-              GSOnceMLog(@"NSStreamSocketSecurityLevelSSLv3 is insecure ..."
-                @" please change your code to stop using it");
 #if GNUTLS_VERSION_NUMBER < 0x020C00
               const int proto_prio[2] = {
                 GNUTLS_SSL3,
@@ -1505,6 +1506,8 @@ static NSMutableDictionary      *credentialsCache = nil;
               gnutls_priority_set_direct(session,
                 "NORMAL:-VERS-TLS-ALL:+VERS-SSL3.0", NULL);
 #endif
+              GSOnceMLog(@"NSStreamSocketSecurityLevelSSLv3 is insecure ..."
+                @" please change your code to stop using it");
             }
           else if ([pri isEqual: NSStreamSocketSecurityLevelTLSv1] == YES)
             {
