@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArchiver class reference</title>
-   $Date: 2013-04-14 07:25:22 +0100 (Sun, 14 Apr 2013) $ $Revision: 36518 $
+   $Date: 2015-10-08 10:13:32 +0100 (Thu, 08 Oct 2015) $ $Revision: 39042 $
    */
 
 #import "common.h"
@@ -310,6 +310,9 @@ static Class	NSMutableDataMallocClass;
       case _C_ULNG_LNG:	info = _GSC_ULNG_LNG | _GSC_S_LNG_LNG;	break;
       case _C_FLT:	info = _GSC_FLT;	break;
       case _C_DBL:	info = _GSC_DBL;	break;
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:	info = _GSC_BOOL;	break;
+#endif
       default:		info = _GSC_NONE;	break;
     }
 
@@ -681,6 +684,13 @@ static Class	NSMutableDataMallocClass;
 	(*_serImp)(_dst, serSel, (void*)buf, @encode(double), nil);
 	return;
 
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:
+	(*_tagImp)(_dst, tagSel, _GSC_BOOL);
+	(*_serImp)(_dst, serSel, (void*)buf, @encode(_Bool), nil);
+	return;
+#endif
+
       case _C_VOID:
 	[NSException raise: NSInvalidArgumentException
 		    format: @"can't encode void item"];
