@@ -1,5 +1,5 @@
 /** NSArray - Array object to hold other objects.
-   Copyright (C) 1995, 1996, 1998 Free Software Foundation, Inc.
+   Copyright (C) 1995-2015 Free Software Foundation, Inc.
 
    Written by:  Andrew Kachites McCallum <mccallum@gnu.ai.mit.edu>
    From skeleton by:  Adam Fedor <fedor@boulder.colorado.edu>
@@ -27,7 +27,7 @@
    Boston, MA 02111 USA.
 
    <title>NSArray class reference</title>
-   $Date: 2014-01-09 09:55:36 +0000 (Thu, 09 Jan 2014) $ $Revision: 37564 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
    */
 
 #import "common.h"
@@ -66,9 +66,9 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 @interface NSArrayEnumerator : NSEnumerator
 {
   NSArray	*array;
-  unsigned	pos;
+  NSUInteger	pos;
   IMP		get;
-  unsigned	(*cnt)(NSArray*, SEL);
+  NSUInteger	(*cnt)(NSArray*, SEL);
 }
 - (id) initWithArray: (NSArray*)anArray;
 @end
@@ -285,7 +285,7 @@ static SEL	rlSel;
 - (NSArray*) arrayByAddingObject: (id)anObject
 {
   id na;
-  unsigned	c = [self count];
+  NSUInteger	c = [self count];
 
   if (anObject == nil)
     [NSException raise: NSInvalidArgumentException
@@ -316,9 +316,9 @@ static SEL	rlSel;
 - (NSArray*) arrayByAddingObjectsFromArray: (NSArray*)anotherArray
 {
   id		na;
-  unsigned	c;
-  unsigned	l;
-  unsigned	e;
+  NSUInteger	c;
+  NSUInteger	l;
+  NSUInteger	e;
 
   c = [self count];
   l = [anotherArray count];
@@ -330,8 +330,8 @@ static SEL	rlSel;
     [self getObjects: objects];
     if ([anotherArray isProxy])
       {
-	unsigned	i = c;
-	unsigned	j = 0;
+	NSUInteger	i = c;
+	NSUInteger	j = 0;
 
 	while (i < e)
 	  {
@@ -435,7 +435,7 @@ static SEL	rlSel;
  */
 - (void) encodeWithCoder: (NSCoder*)aCoder
 {
-  unsigned	count = [self count];
+  NSUInteger	count = [self count];
 
   if ([aCoder allowsKeyedCoding])
     {
@@ -449,13 +449,13 @@ static SEL	rlSel;
 	}
       else
 	{
-	  unsigned	i;
+	  NSUInteger	i;
 
 	  for (i = 0; i < count; i++)
 	    {
 	      NSString	*key;
 
-	      key = [NSString stringWithFormat: @"NS.object.%u", i];
+	      key = [NSString stringWithFormat: @"NS.object.%lu", (unsigned long)i];
 	      [(NSKeyedArchiver*)aCoder encodeObject: [self objectAtIndex: i]
 					      forKey: key];
 	    }
@@ -463,9 +463,10 @@ static SEL	rlSel;
     }
   else
     {
-      [aCoder encodeValueOfObjCType: @encode(unsigned)
-				 at: &count];
+      unsigned  items = (unsigned)count;
 
+      [aCoder encodeValueOfObjCType: @encode(unsigned)
+				 at: &items];
       if (count > 0)
 	{
 	  GS_BEGINIDBUF(a, count);
@@ -485,7 +486,7 @@ static SEL	rlSel;
  */
 - (void) getObjects: (__unsafe_unretained id[])aBuffer
 {
-  unsigned i, c = [self count];
+  NSUInteger i, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
 
   for (i = 0; i < c; i++)
@@ -498,7 +499,7 @@ static SEL	rlSel;
  */
 - (void) getObjects: (__unsafe_unretained id[])aBuffer range: (NSRange)aRange
 {
-  unsigned i, j = 0, c = [self count], e = aRange.location + aRange.length;
+  NSUInteger i, j = 0, c = [self count], e = aRange.location + aRange.length;
   IMP	get = [self methodForSelector: oaiSel];
 
   GS_RANGE_CHECK(aRange, c);
@@ -521,12 +522,12 @@ static SEL	rlSel;
  */
 - (NSUInteger) indexOfObjectIdenticalTo: (id)anObject
 {
-  unsigned c = [self count];
+  NSUInteger c = [self count];
 
   if (c > 0)
     {
       IMP	get = [self methodForSelector: oaiSel];
-      unsigned	i;
+      NSUInteger	i;
 
       for (i = 0; i < c; i++)
 	if (anObject == (*get)(self, oaiSel, i))
@@ -541,7 +542,7 @@ static SEL	rlSel;
  */
 - (NSUInteger) indexOfObjectIdenticalTo: anObject inRange: (NSRange)aRange
 {
-  unsigned i, e = aRange.location + aRange.length, c = [self count];
+  NSUInteger i, e = aRange.location + aRange.length, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
 
   GS_RANGE_CHECK(aRange, c);
@@ -559,11 +560,11 @@ static SEL	rlSel;
  */
 - (NSUInteger) indexOfObject: (id)anObject
 {
-  unsigned	c = [self count];
+  NSUInteger	c = [self count];
 
   if (c > 0 && anObject != nil)
     {
-      unsigned	i;
+      NSUInteger	i;
       IMP	get = [self methodForSelector: oaiSel];
       BOOL	(*eq)(id, SEL, id)
 	= (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
@@ -582,7 +583,7 @@ static SEL	rlSel;
  */
 - (NSUInteger) indexOfObject: (id)anObject inRange: (NSRange)aRange
 {
-  unsigned i, e = aRange.location + aRange.length, c = [self count];
+  NSUInteger i, e = aRange.location + aRange.length, c = [self count];
   IMP	get = [self methodForSelector: oaiSel];
   BOOL	(*eq)(id, SEL, id)
     = (BOOL (*)(id, SEL, id))[anObject methodForSelector: eqSel];
@@ -636,12 +637,12 @@ static SEL	rlSel;
  */
 - (id) initWithArray: (NSArray*)array copyItems: (BOOL)shouldCopy
 {
-  unsigned	c = [array count];
+  NSUInteger	c = [array count];
   GS_BEGINIDBUF(objects, c);
 
   if ([array isProxy])
     {
-      unsigned	i;
+      NSUInteger	i;
 
       for (i = 0; i < c; i++)
 	{
@@ -654,7 +655,7 @@ static SEL	rlSel;
     }
   if (shouldCopy == YES)
     {
-      unsigned	i;
+      NSUInteger	i;
 
       for (i = 0; i < c; i++)
 	{
@@ -683,12 +684,12 @@ static SEL	rlSel;
  */
 - (id) initWithArray: (NSArray*)array
 {
-  unsigned	c = [array count];
+  NSUInteger	c = [array count];
   GS_BEGINIDBUF(objects, c);
 
   if ([array isProxy])
     {
-      unsigned	i;
+      NSUInteger	i;
 
       for (i = 0; i < c; i++)
 	{
@@ -718,19 +719,19 @@ static SEL	rlSel;
 						@"NS.objects"];
       if (array == nil)
 	{
-	  unsigned	i = 0;
+	  NSUInteger	i = 0;
 	  NSString	*key;
 	  id		val;
 
 	  array = [NSMutableArray arrayWithCapacity: 2];
-	  key = [NSString stringWithFormat: @"NS.object.%u", i];
+	  key = [NSString stringWithFormat: @"NS.object.%lu", (unsigned long)i];
 	  val = [(NSKeyedUnarchiver*)aCoder decodeObjectForKey: key];
 
 	  while (val != nil)
 	    {
 	      [array addObject: val];
 	      i++;
-	      key = [NSString stringWithFormat: @"NS.object.%u", i];
+	      key = [NSString stringWithFormat: @"NS.object.%lu", (unsigned long)i];
 	      val = [(NSKeyedUnarchiver*)aCoder decodeObjectForKey: key];
 	    }
 	}
@@ -739,22 +740,22 @@ static SEL	rlSel;
     }
   else
     {
-      unsigned    count;
+      unsigned    items;
 
       [aCoder decodeValueOfObjCType: @encode(unsigned)
-	                         at: &count];
-      if (count > 0)
+	                         at: &items];
+      if (items > 0)
         {
-	  GS_BEGINIDBUF(contents, count);
+	  GS_BEGINIDBUF(contents, items);
 
 	  [aCoder decodeArrayOfObjCType: @encode(id)
-		  count: count
+		  count: items
 		  at: contents];
-	  self = [self initWithObjects: contents count: count];
+	  self = [self initWithObjects: contents count: items];
 #if GS_WITH_GC == 0
-	  while (count-- > 0)
+	  while (items-- > 0)
 	    {
-	      [contents[count] release];
+	      [contents[items] release];
 	    }
 #endif
 	  GS_ENDIDBUF();
@@ -913,9 +914,9 @@ static SEL	rlSel;
   return nil;
 }
 
-- (id) objectAtIndexedSubscript: (size_t)anIndex
+- (id) objectAtIndexedSubscript: (NSUInteger)anIndex
 {
-  return [self objectAtIndex: (NSUInteger)anIndex];
+  return [self objectAtIndex: anIndex];
 }
 
 - (NSArray *) objectsAtIndexes: (NSIndexSet *)indexes
@@ -930,7 +931,7 @@ static SEL	rlSel;
       i = [indexes indexGreaterThanIndex: i];
     }
 
-  return [group makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(group);
 }
 
 - (BOOL) isEqual: (id)anObject
@@ -948,7 +949,7 @@ static SEL	rlSel;
  */
 - (BOOL) isEqualToArray: (NSArray*)otherArray
 {
-  unsigned i, c;
+  NSUInteger i, c;
 
   if (self == (id)otherArray)
     return YES;
@@ -972,24 +973,35 @@ static SEL	rlSel;
  */
 - (id) lastObject
 {
-  unsigned count = [self count];
+  NSUInteger count = [self count];
   if (count == 0)
     return nil;
   return [self objectAtIndex: count-1];
 }
 
+/**
+ * Returns the first object in the receiver, or nil if the receiver is empty.
+ */
+- (id) firstObject
+{
+  NSUInteger count = [self count];
+  if (count == 0)
+    return nil;
+  return [self objectAtIndex: 0];
+}
+
 /**
  * Makes each object in the array perform aSelector.<br />
  * This is done sequentially from the first to the last object.
  */
 - (void) makeObjectsPerformSelector: (SEL)aSelector
 {
-  unsigned	c = [self count];
+  NSUInteger	c = [self count];
 
   if (c > 0)
     {
-      IMP	get = [self methodForSelector: oaiSel];
-      unsigned	i = 0;
+      IMP	        get = [self methodForSelector: oaiSel];
+      NSUInteger	i = 0;
 
       while (i < c)
 	{
@@ -1012,12 +1024,12 @@ static SEL	rlSel;
  */
 - (void) makeObjectsPerformSelector: (SEL)aSelector withObject: (id)arg
 {
-  unsigned	c = [self count];
+  NSUInteger    c = [self count];
 
   if (c > 0)
     {
-      IMP	get = [self methodForSelector: oaiSel];
-      unsigned	i = 0;
+      IMP	        get = [self methodForSelector: oaiSel];
+      NSUInteger	i = 0;
 
       while (i < c)
 	{
@@ -1101,11 +1113,11 @@ compare(id elem1, id elem2, void* context)
 {
   NSMutableArray	*sortedArray;
 
-  sortedArray = [[[NSMutableArrayClass allocWithZone:
-    NSDefaultMallocZone()] initWithArray: self copyItems: NO] autorelease];
+  sortedArray = AUTORELEASE([[NSMutableArrayClass allocWithZone:
+    NSDefaultMallocZone()] initWithArray: self copyItems: NO]);
   [sortedArray sortUsingFunction: comparator context: context];
 
-  return [sortedArray makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(sortedArray);
 }
 
 
@@ -1114,11 +1126,11 @@ compare(id elem1, id elem2, void* context)
 {
   NSMutableArray	*sortedArray;
 
-  sortedArray = [[[NSMutableArrayClass allocWithZone:
-    NSDefaultMallocZone()] initWithArray: self copyItems: NO] autorelease];
+  sortedArray = AUTORELEASE([[NSMutableArrayClass allocWithZone:
+    NSDefaultMallocZone()] initWithArray: self copyItems: NO]);
   [sortedArray sortWithOptions: options usingComparator: comparator];
 
-  return [sortedArray makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(sortedArray);
 }
 
 - (NSArray*) sortedArrayUsingComparator: (NSComparator)comparator
@@ -1214,14 +1226,14 @@ compare(id elem1, id elem2, void* context)
  */
 - (NSString*) componentsJoinedByString: (NSString*)separator
 {
-  unsigned int		c = [self count];
+  NSUInteger		c = [self count];
   NSMutableString	*s;
 
-  s = [[[NSMutableString alloc] initWithCapacity: c] autorelease];
+  s = [NSMutableString stringWithCapacity: c];
   if (c > 0)
     {
-      unsigned	l = [separator length];
-      unsigned	i;
+      NSUInteger	l = [separator length];
+      NSUInteger	i;
 
       [s appendString: [[self objectAtIndex: 0] description]];
       for (i = 1; i < c; i++)
@@ -1233,7 +1245,7 @@ compare(id elem1, id elem2, void* context)
 	  [s appendString: [[self objectAtIndex: i] description]];
 	}
     }
-  return [s makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(s);
 }
 
 /**
@@ -1243,8 +1255,8 @@ compare(id elem1, id elem2, void* context)
  */
 - (NSArray*) pathsMatchingExtensions: (NSArray*)extensions
 {
-  unsigned i, c = [self count];
-  NSMutableArray *a = [[[NSMutableArray alloc] initWithCapacity: 1] autorelease];
+  NSUInteger i, c = [self count];
+  NSMutableArray *a = AUTORELEASE([[NSMutableArray alloc] initWithCapacity: 1]);
   Class	cls = [NSString class];
   IMP	get = [self methodForSelector: oaiSel];
   IMP	add = [a methodForSelector: addSel];
@@ -1261,7 +1273,7 @@ compare(id elem1, id elem2, void* context)
 	    }
 	}
     }
-  return [a makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(a);
 }
 
 /**
@@ -1271,7 +1283,7 @@ compare(id elem1, id elem2, void* context)
  */
 - (id) firstObjectCommonWithArray: (NSArray*)otherArray
 {
-  unsigned i, c = [self count];
+  NSUInteger i, c = [self count];
   id o;
 
   for (i = 0; i < c; i++)
@@ -1291,7 +1303,7 @@ compare(id elem1, id elem2, void* context)
 - (NSArray*) subarrayWithRange: (NSRange)aRange
 {
   id na;
-  unsigned c = [self count];
+  NSUInteger c = [self count];
 
   GS_RANGE_CHECK(aRange, c);
 
@@ -1463,20 +1475,20 @@ compare(id elem1, id elem2, void* context)
 
   if ([key isEqualToString: @"@count"] == YES)
     {
-      result = [NSNumber numberWithUnsignedInt: [self count]];
+      result = [NSNumber numberWithUnsignedInteger: [self count]];
     }
   else if ([key isEqualToString: @"count"] == YES)
     {
       GSOnceMLog(
 @"[NSArray-valueForKey:] called wth 'count' is deprecated .. use '@count'");
-      result = [NSNumber numberWithUnsignedInt: [self count]];
+      result = [NSNumber numberWithUnsignedInteger: [self count]];
     }
   else
     {
       NSMutableArray	*results = nil;
       static NSNull	*null = nil;
-      unsigned		i;
-      unsigned		count = [self count];
+      NSUInteger	i;
+      NSUInteger	count = [self count];
       volatile id	object = nil;
 
       results = [NSMutableArray arrayWithCapacity: count];
@@ -1517,7 +1529,7 @@ compare(id elem1, id elem2, void* context)
         {
           if ([path isEqualToString: @"@count"] == YES)
             {
-              result = [NSNumber numberWithUnsignedInt: [self count]];
+              result = [NSNumber numberWithUnsignedInteger: [self count]];
             }
           else
             {
@@ -1528,11 +1540,11 @@ compare(id elem1, id elem2, void* context)
         {
           NSString      *op = [path substringToIndex: r.location];
           NSString      *rem = [path substringFromIndex: NSMaxRange(r)];
-          unsigned      count = [self count];
+          NSUInteger    count = [self count];
 
           if ([op isEqualToString: @"@count"] == YES)
             {
-              result = [NSNumber numberWithUnsignedInt: count];
+              result = [NSNumber numberWithUnsignedInteger: count];
             }
           else if ([op isEqualToString: @"@avg"] == YES)
             {
@@ -1676,7 +1688,7 @@ compare(id elem1, id elem2, void* context)
                       o = [o valueForKeyPath: rem];
                       [result addObjectsFromArray: o];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -1696,7 +1708,7 @@ compare(id elem1, id elem2, void* context)
                       o = [o valueForKeyPath: rem];
                       [result addObject: o];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -1716,7 +1728,7 @@ compare(id elem1, id elem2, void* context)
                       o = [o valueForKeyPath: rem];
                       [result addObjectsFromArray: [o allObjects]];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -1743,8 +1755,8 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) setValue: (id)value forKey: (NSString*)key
 {
-  unsigned	i;
-  unsigned	count = [self count];
+  NSUInteger    i;
+  NSUInteger	count = [self count];
   volatile id	object = nil;
 
   for (i = 0; i < count; i++)
@@ -1759,6 +1771,7 @@ compare(id elem1, id elem2, void* context)
 {
   [self enumerateObjectsWithOptions: 0 usingBlock: aBlock];
 }
+
 - (void) enumerateObjectsWithOptions: (NSEnumerationOptions)opts
 			  usingBlock: (GSEnumeratorBlock)aBlock
 {
@@ -1780,18 +1793,18 @@ compare(id elem1, id elem2, void* context)
   FOR_IN (id, obj, enumerator)
     GS_DISPATCH_SUBMIT_BLOCK(enumQueueGroup, enumQueue, if (YES == shouldStop) {return;}, return, aBlock, obj, count, &shouldStop);
       if (isReverse)
-      {
-        count--;
-      }
+        {
+          count--;
+        }
       else
-      {
-        count++;
-      }
+        {
+          count++;
+        }
 
       if (shouldStop)
-      {
-        break;
-      }
+        {
+          break;
+        }
     END_FOR_IN(enumerator)
     GS_DISPATCH_TEARDOWN_QUEUE_AND_GROUP_FOR_ENUMERATION(enumQueue, opts)
   }
@@ -1809,13 +1822,11 @@ compare(id elem1, id elem2, void* context)
 				 passingTest: (GSPredicateBlock)predicate
 {
   /* TODO: Concurrency. */
-  NSMutableIndexSet *set = [NSMutableIndexSet indexSet];
-  BLOCK_SCOPE BOOL shouldStop = NO;
+  NSMutableIndexSet     *set = [NSMutableIndexSet indexSet];
+  BLOCK_SCOPE BOOL      shouldStop = NO;
   id<NSFastEnumeration> enumerator = self;
-  NSUInteger count = 0;
-  BLOCK_SCOPE NSLock *setLock = nil;
-
-
+  NSUInteger            count = 0;
+  BLOCK_SCOPE NSLock    *setLock = nil;
 
   /* If we are enumerating in reverse, use the reverse enumerator for fast
    * enumeration. */
@@ -1824,9 +1835,9 @@ compare(id elem1, id elem2, void* context)
       enumerator = [self reverseObjectEnumerator];
     }
   if (opts & NSEnumerationConcurrent)
-  {
-    setLock = [NSLock new];
-  }
+    {
+      setLock = [NSLock new];
+    }
   {
     GS_DISPATCH_CREATE_QUEUE_AND_GROUP_FOR_ENUMERATION(enumQueue, opts)
     FOR_IN (id, obj, enumerator)
@@ -1860,7 +1871,7 @@ compare(id elem1, id elem2, void* context)
     END_FOR_IN(enumerator)
     GS_DISPATCH_TEARDOWN_QUEUE_AND_GROUP_FOR_ENUMERATION(enumQueue, opts);
   }
-  [setLock release];
+  RELEASE(setLock);
   return set;
 }
 
@@ -1878,15 +1889,16 @@ compare(id elem1, id elem2, void* context)
     passingTest: predicate];
 }
 
-- (NSUInteger)indexOfObjectWithOptions: (NSEnumerationOptions)opts
-			   passingTest: (GSPredicateBlock)predicate
+- (NSUInteger) indexOfObjectWithOptions: (NSEnumerationOptions)opts
+			    passingTest: (GSPredicateBlock)predicate
 {
   /* TODO: Concurrency. */
   id<NSFastEnumeration> enumerator = self;
-  BLOCK_SCOPE BOOL shouldStop = NO;
-  NSUInteger count = 0;
+  BLOCK_SCOPE BOOL      shouldStop = NO;
+  NSUInteger            count = 0;
   BLOCK_SCOPE NSUInteger index = NSNotFound;
-  BLOCK_SCOPE NSLock *indexLock = nil;
+  BLOCK_SCOPE NSLock    *indexLock = nil;
+
   /* If we are enumerating in reverse, use the reverse enumerator for fast
    * enumeration. */
   if (opts & NSEnumerationReverse)
@@ -1895,9 +1907,9 @@ compare(id elem1, id elem2, void* context)
     }
 
   if (opts & NSEnumerationConcurrent)
-  {
-    indexLock = [NSLock new];
-  }
+    {
+      indexLock = [NSLock new];
+    }
   {
     GS_DISPATCH_CREATE_QUEUE_AND_GROUP_FOR_ENUMERATION(enumQueue, opts)
     FOR_IN (id, obj, enumerator)
@@ -1934,7 +1946,7 @@ compare(id elem1, id elem2, void* context)
     END_FOR_IN(enumerator)
     GS_DISPATCH_TEARDOWN_QUEUE_AND_GROUP_FOR_ENUMERATION(enumQueue, opts);
   }
-  [indexLock release];
+  RELEASE(indexLock);
   return index;
 }
 
@@ -1943,13 +1955,33 @@ compare(id elem1, id elem2, void* context)
   return [self indexOfObjectWithOptions: 0 passingTest: predicate];
 }
 
-- (NSUInteger)indexOfObjectAtIndexes: (NSIndexSet*)indexSet
-			     options: (NSEnumerationOptions)opts
-			 passingTest: (GSPredicateBlock)predicate
+- (NSUInteger) indexOfObjectAtIndexes: (NSIndexSet*)indexSet
+			      options: (NSEnumerationOptions)opts
+			  passingTest: (GSPredicateBlock)predicate
 {
   return [[self objectsAtIndexes: indexSet]
-    indexOfObjectWithOptions: 0
-    passingTest: predicate];
+        indexOfObjectWithOptions: 0
+                     passingTest: predicate];
+}
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = [super sizeInBytesExcluding: exclude];
+
+  if (size > 0)
+    {
+      NSUInteger	count = [self count];
+      GS_BEGINIDBUF(objects, count);
+
+      size += count*sizeof(void*);
+      [self getObjects: objects];
+      while (count-- > 0)
+	{
+	  size += [objects[count] sizeInBytesExcluding: exclude];
+	}
+      GS_ENDIDBUF();
+    }
+  return size;
 }
 @end
 
@@ -2012,7 +2044,7 @@ compare(id elem1, id elem2, void* context)
 {
   id	tmp = [self objectAtIndex: i1];
 
-  IF_NO_GC([tmp retain];)
+  RETAIN(tmp);
   [self replaceObjectAtIndex: i1 withObject: [self objectAtIndex: i2]];
   [self replaceObjectAtIndex: i2 withObject: tmp];
   RELEASE(tmp);
@@ -2023,9 +2055,16 @@ compare(id elem1, id elem2, void* context)
   [self subclassResponsibility: _cmd];
 }
 
-- (void) setObject: (id)anObject atIndexedSubscript: (size_t)anIndex
+- (void) setObject: (id)anObject atIndexedSubscript: (NSUInteger)anIndex
 {
-  [self replaceObjectAtIndex: (NSUInteger)anIndex withObject: anObject];
+  if ([self count] == anIndex)
+    {
+      [self addObject: anObject];
+    }
+  else
+    {
+      [self replaceObjectAtIndex: anIndex withObject: anObject];
+    }
 }
 
 /** Replaces the values in the receiver at the locations given by the
@@ -2123,7 +2162,7 @@ compare(id elem1, id elem2, void* context)
   self = [self initWithCapacity: count];
   if (count > 0)
     {
-      unsigned	i;
+      NSUInteger	i;
       IMP	add = [self methodForSelector: addSel];
 
       for (i = 0; i < count; i++)
@@ -2138,7 +2177,7 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeLastObject
 {
-  unsigned	count = [self count];
+  NSUInteger	count = [self count];
 
   if (count == 0)
     [NSException raise: NSRangeException
@@ -2152,7 +2191,7 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObjectIdenticalTo: (id)anObject
 {
-  unsigned	i;
+  NSUInteger	i;
 
   if (anObject == nil)
     {
@@ -2187,9 +2226,9 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObject: (id)anObject inRange: (NSRange)aRange
 {
-  unsigned	c;
-  unsigned	s;
-  unsigned	i;
+  NSUInteger	c;
+  NSUInteger	s;
+  NSUInteger	i;
 
   if (anObject == nil)
     {
@@ -2224,7 +2263,7 @@ compare(id elem1, id elem2, void* context)
 		   * first equal object we don't get left with a bad object
 		   * pointer for later comparisons.
 		   */
-		  IF_NO_GC([anObject retain];)
+		  RETAIN(anObject);
 		}
 	      (*rem)(self, remSel, i);
 	    }
@@ -2244,9 +2283,9 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObjectIdenticalTo: (id)anObject inRange: (NSRange)aRange
 {
-  unsigned	c;
-  unsigned	s;
-  unsigned	i;
+  NSUInteger	c;
+  NSUInteger	s;
+  NSUInteger	i;
 
   if (anObject == nil)
     {
@@ -2287,7 +2326,7 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObject: (id)anObject
 {
-  unsigned	i;
+  NSUInteger	i;
 
   if (anObject == nil)
     {
@@ -2316,7 +2355,7 @@ compare(id elem1, id elem2, void* context)
 		   * first equal object we don't get left with a bad object
 		   * pointer for later comparisons.
 		   */
-		  IF_NO_GC([anObject retain];)
+		  RETAIN(anObject);
 		}
 	      (*rem)(self, remSel, i);
 	    }
@@ -2335,7 +2374,7 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeAllObjects
 {
-  unsigned	c = [self count];
+  NSUInteger	c = [self count];
 
   if (c > 0)
     {
@@ -2353,11 +2392,11 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) addObjectsFromArray: (NSArray*)otherArray
 {
-  unsigned c = [otherArray count];
+  NSUInteger c = [otherArray count];
 
   if (c > 0)
     {
-      unsigned	i;
+      NSUInteger	i;
       IMP	get = [otherArray methodForSelector: oaiSel];
       IMP	add = [self methodForSelector: addSel];
 
@@ -2403,14 +2442,14 @@ compare(id elem1, id elem2, void* context)
 {
   if (count > 0)
     {
-      unsigned	to = 0;
-      unsigned	from = 0;
-      unsigned	i;
+      NSUInteger	to = 0;
+      NSUInteger	from = 0;
+      NSUInteger	i;
       GS_BEGINITEMBUF(sorted, count, NSUInteger);
 
       while (from < count)
 	{
-	  unsigned	val = indices[from++];
+	  NSUInteger	val = indices[from++];
 
 	  i = to;
 	  while (i > 0 && sorted[i-1] > val)
@@ -2423,7 +2462,7 @@ compare(id elem1, id elem2, void* context)
 	    }
 	  else if (sorted[i] != val)
 	    {
-	      unsigned	j = to++;
+	      NSUInteger	j = to++;
 
 	      if (sorted[i] < val)
 		{
@@ -2457,11 +2496,11 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObjectsInArray: (NSArray*)otherArray
 {
-  unsigned	c = [otherArray count];
+  NSUInteger	c = [otherArray count];
 
   if (c > 0)
     {
-      unsigned	i;
+      NSUInteger	i;
       IMP	get = [otherArray methodForSelector: oaiSel];
       IMP	rem = [self methodForSelector: @selector(removeObject:)];
 
@@ -2475,9 +2514,9 @@ compare(id elem1, id elem2, void* context)
  */
 - (void) removeObjectsInRange: (NSRange)aRange
 {
-  unsigned	i;
-  unsigned	s = aRange.location;
-  unsigned	c = [self count];
+  NSUInteger	i;
+  NSUInteger	s = aRange.location;
+  NSUInteger	c = [self count];
 
   i = aRange.location + aRange.length;
 
@@ -2589,7 +2628,7 @@ compare(id elem1, id elem2, void* context)
       IF_NO_GC(RETAIN(array));
       pos = 0;
       get = [array methodForSelector: oaiSel];
-      cnt = (unsigned (*)(NSArray*, SEL))[array methodForSelector: countSel];
+      cnt = (NSUInteger (*)(NSArray*, SEL))[array methodForSelector: countSel];
     }
   return self;
 }
