@@ -28,7 +28,7 @@
 
 
    <title>NSBundle class reference</title>
-   $Date: 2014-01-14 14:13:31 +0000 (Tue, 14 Jan 2014) $ $Revision: 37602 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
 */
 
 #define	EXPOSE_NSBundle_IVARS	1
@@ -173,8 +173,6 @@ static NSLock *pathCacheLock = nil;
 static NSMutableDictionary *pathCache = nil;
 
 @interface NSObject (PrivateFrameworks)
-+ (NSString*) frameworkEnv;
-+ (NSString*) frameworkPath;
 + (NSString*) frameworkVersion;
 + (NSString**) frameworkClasses;
 @end
@@ -272,7 +270,7 @@ AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 	{
 	  pathlist = [env objectForKey:@"Path"];
 	}
-#if defined(__MINGW__)
+#if defined(_WIN32)
       patharr = [pathlist componentsSeparatedByString:@";"];
 #else
       patharr = [pathlist componentsSeparatedByString:@":"];
@@ -303,7 +301,7 @@ AbsolutePathOfExecutable(NSString *path, BOOL atLaunch)
 	      result = [prefix stringByStandardizingPath];
 	      break;
 	    }
-#if defined(__WIN32__)
+#if defined(_WIN32)
 	  else
 	    {
 	      NSString	*extension = [path pathExtension];
@@ -464,7 +462,7 @@ bundle_object_name(NSString *path, NSString* executable)
     return path1;
   else if ([mgr isReadableFileAtPath: path0] == YES)
     return path0;
-#if defined(__MINGW__)
+#if defined(_WIN32)
   /* If we couldn't find the binary, and we are on windows, and the name
    * has no path extension, then let's try looking for a dll.
    */
@@ -790,7 +788,7 @@ _find_main_bundle_for_tool(NSString *toolName)
 	    {
 	      bundlePath = [bundlePath stringByDeletingLastPathComponent];
 	    }
-#if defined(__MINGW__)
+#if defined(_WIN32)
 	  /* On windows, the library (dll) is in the Tools area rather than
 	   * in the framework, so we can adjust the path here.
 	   */
@@ -825,7 +823,7 @@ _find_main_bundle_for_tool(NSString *toolName)
 		  if (bundlePath)
 		    bundle = [[self alloc] initWithPath: bundlePath];
 		}
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
 	    }
 #endif
 
@@ -1350,7 +1348,7 @@ _bundle_load_callback(Class theClass, struct objc_category *theCategory)
          the executable name here - just in case it turns out it's a
          tool.  */
       NSString *toolName = [GSPrivateExecutablePath() lastPathComponent];
-#if defined(__WIN32__) || defined(__CYGWIN__)
+#if defined(_WIN32) || defined(__CYGWIN__)
       toolName = [toolName stringByDeletingPathExtension];
 #endif
 
@@ -1679,7 +1677,7 @@ IF_NO_GC(
       NSWarnMLog(@"NSBundle -initWithPath: requires absolute path names, "
 	@"given '%@'", path);
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
       if ([path length] > 0 &&
 	([path characterAtIndex: 0]=='/' || [path characterAtIndex: 0]=='\\'))
 	{
@@ -1865,7 +1863,7 @@ IF_NO_GC(
 		    {
 		      [pathCache removeObjectForKey: path];
 		    }
-#if defined(__MINGW__)
+#if defined(_WIN32)
 		  else if ('\\' == c)
 		    {
 		      [pathCache removeObjectForKey: path];
@@ -2260,7 +2258,7 @@ IF_NO_GC(
 {
   NSString *rootPath;
 
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
   if (_frameworkVersion)
     rootPath = [NSString stringWithFormat:@"%@/Versions/%@", [self bundlePath],
       _frameworkVersion];
@@ -2469,7 +2467,7 @@ IF_NO_GC(
   /* I guess this is arbitrary if we can't find a match? */
   if ([array count] == 0 && [localizationsArray count] > 0)
     [array addObject: [localizationsArray objectAtIndex: 0]];
-  return [array makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(array);
 }
 
 - (NSDictionary*) localizedInfoDictionary
@@ -2533,7 +2531,7 @@ IF_NO_GC(
       locale = [[locale lastPathComponent] stringByDeletingPathExtension];
       [array addObject: locale];
     }
-  return [array makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(array);
 }
 
 - (NSArray *) preferredLocalizations
@@ -2739,7 +2737,7 @@ IF_NO_GC(
       mangledName = [mangledName stringByReplacingString: @"+"
 				 withString: @"_1"];
 
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
       path = [_path stringByAppendingPathComponent:@"Versions/Current"];
 #else
       path = _path;
@@ -2772,7 +2770,7 @@ IF_NO_GC(
 
   if (_bundleType == NSBUNDLE_FRAMEWORK)
     {
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
       return [_path stringByAppendingPathComponent:
                       [NSString stringWithFormat:@"Versions/%@/%@",
                       version, executableName]];
@@ -2801,12 +2799,12 @@ IF_NO_GC(
 
   if (_bundleType == NSBUNDLE_FRAMEWORK)
     {
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
       return [_path stringByAppendingPathComponent:
 		      [NSString stringWithFormat:@"Versions/%@/Resources",
 				version]];
 #else
-      /* No Versions (that require symlinks) on MINGW */
+      /* No Versions (that require symlinks) on mswindows */
       return [_path stringByAppendingPathComponent: @"Resources"];
 #endif
     }
@@ -2858,7 +2856,7 @@ IF_NO_GC(
 
   if (_bundleType == NSBUNDLE_FRAMEWORK)
     {
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
       return [_path stringByAppendingPathComponent:
                       [NSString stringWithFormat:@"Versions/%@/PlugIns",
                       version]];
@@ -2886,7 +2884,7 @@ IF_NO_GC(
 
   if (_bundleType == NSBUNDLE_FRAMEWORK)
     {
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
       return [_path stringByAppendingPathComponent:
                       [NSString stringWithFormat:@"Versions/%@/PrivateFrameworks",
                       version]];
@@ -2956,7 +2954,7 @@ IF_NO_GC(
    */
   libraryName = [libraryName lastPathComponent];
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
   /* A dll is usually of the form 'xxx-maj_min.dll'
    * so we can extract the version info and use it.
    */
