@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSCalendarDate class reference</title>
-   $Date: 2014-02-13 10:41:55 +0000 (Thu, 13 Feb 2014) $ $Revision: 37683 $
+   $Date: 2016-03-15 19:04:51 +0000 (Tue, 15 Mar 2016) $ $Revision: 39549 $
    */
 
 #import "common.h"
@@ -63,6 +63,9 @@
 @end
 
 
+#define DISTANT_FUTURE	63113990400.0
+#define DISTANT_PAST	-63113817600.0
+
 static NSString	*cformat = @"%Y-%m-%d %H:%M:%S %z";
 
 static NSTimeZone	*localTZ = nil;
@@ -187,10 +190,10 @@ absoluteGregorianDay(NSUInteger day, NSUInteger month, NSUInteger year)
      + year/400);   // ...plus prior years divisible by 400
 }
 
-static inline int
+static inline NSInteger
 dayOfCommonEra(NSTimeInterval when)
 {
-  int r;
+  NSInteger r;
 
   // Get reference date in terms of days
   when /= 86400.0;
@@ -201,22 +204,27 @@ dayOfCommonEra(NSTimeInterval when)
 }
 
 static void
-gregorianDateFromAbsolute(NSInteger abs, int *day, int *month, int *year)
+gregorianDateFromAbsolute(NSInteger abs,
+  NSInteger *day, NSInteger *month, NSInteger *year)
 {
+  NSInteger     y;
+  NSInteger     m;
+
   // Search forward year by year from approximate year
-  *year = abs/366;
-  while (abs >= absoluteGregorianDay(1, 1, (*year)+1))
+  y = abs/366;
+  while (abs >= absoluteGregorianDay(1, 1, y+1))
     {
-      (*year)++;
+      y++;
     }
   // Search forward month by month from January
-  (*month) = 1;
-  while (abs > absoluteGregorianDay(lastDayOfGregorianMonth(*month, *year),
-    *month, *year))
+  m = 1;
+  while (abs > absoluteGregorianDay(lastDayOfGregorianMonth(m, y), m, y))
     {
-      (*month)++;
+      m++;
     }
-  *day = abs - absoluteGregorianDay(1, *month, *year) + 1;
+  *year = y;
+  *month = m;
+  *day = abs - absoluteGregorianDay(1, m, y) + 1;
 }
 
 /**
@@ -246,10 +254,11 @@ GSTime(unsigned day, unsigned month, unsigned year, unsigned hour, unsigned minu
  * External - so NSTimeZone  can use it ... but should really be static.
  */
 void
-GSBreakTime(NSTimeInterval when, int *year, int *month, int *day,
-  int *hour, int *minute, int *second, int *mil)
+GSBreakTime(NSTimeInterval when,
+  NSInteger *year, NSInteger *month, NSInteger *day,
+  NSInteger *hour, NSInteger *minute, NSInteger *second, NSInteger *mil)
 {
-  int h, m, dayOfEra;
+  NSInteger h, m, dayOfEra;
   double a, b, c, d;
 
   /* The 0.1 constant was experimentally derived to cause our behavior
@@ -292,7 +301,7 @@ NSTimeInterval
 GSPrivateTimeNow(void)
 {
   NSTimeInterval t;
-#if !defined(__MINGW__)
+#if !defined(_WIN32)
   struct timeval tp;
 
   gettimeofday (&tp, NULL);
@@ -340,7 +349,7 @@ GSPrivateTimeNow(void)
   GetSystemTime(&sys_time);
   t = GSTime(sys_time.wDay, sys_time.wMonth, sys_time.wYear, sys_time.wHour,
     sys_time.wMinute, sys_time.wSecond, sys_time.wMilliseconds);
-#endif /* __MINGW__ */
+#endif /* _WIN32 */
 
   return t;
 }
@@ -1614,6 +1623,16 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
 	          format: @"[%@-%@] interval is not a number",
 	NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
     }
+#if	GS_SIZEOF_VOIDP == 4
+  if (seconds <= DISTANT_PAST)
+    {
+      seconds = DISTANT_PAST;
+    }
+  else if (seconds >= DISTANT_FUTURE)
+    {
+      seconds = DISTANT_FUTURE;
+    }
+#endif
   _seconds_since_ref = seconds;
   if (_calendar_format == nil)
     {
@@ -1643,7 +1662,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) dayOfMonth
 {
-  int m, d, y;
+  NSInteger m, d, y;
   NSTimeInterval	when;
 
   when = _seconds_since_ref + offset(_time_zone, self);
@@ -1666,7 +1685,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) dayOfWeek
 {
-  int	d;
+  NSInteger	        d;
   NSTimeInterval	when;
 
   when = _seconds_since_ref + offset(_time_zone, self);
@@ -1687,7 +1706,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) dayOfYear
 {
-  int m, d, y, days, i;
+  NSInteger m, d, y, days, i;
   NSTimeInterval	when;
 
   when = _seconds_since_ref + offset(_time_zone, self);
@@ -1704,7 +1723,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) hourOfDay
 {
-  int h;
+  NSInteger h;
   double a, d;
   NSTimeInterval	when;
 
@@ -1712,7 +1731,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
   d = dayOfCommonEra(when);
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (_seconds_since_ref + offset(_time_zone, self)));
+  a = fabs(d - (_seconds_since_ref + offset(_time_zone, self)));
   a = a / 3600;
   h = (NSInteger)a;
 
@@ -1729,7 +1748,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) minuteOfHour
 {
-  int h, m;
+  NSInteger h, m;
   double a, b, d;
   NSTimeInterval	when;
 
@@ -1737,7 +1756,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
   d = dayOfCommonEra(when);
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (_seconds_since_ref + offset(_time_zone, self)));
+  a = fabs(d - (_seconds_since_ref + offset(_time_zone, self)));
   b = a / 3600;
   h = (NSInteger)b;
   h = h * 3600;
@@ -1753,7 +1772,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) monthOfYear
 {
-  int m, d, y;
+  NSInteger m, d, y;
   NSTimeInterval	when;
 
   when = _seconds_since_ref + offset(_time_zone, self);
@@ -1767,7 +1786,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) secondOfMinute
 {
-  int h, m, s;
+  NSInteger h, m, s;
   double a, b, c, d;
   NSTimeInterval	when;
 
@@ -1775,7 +1794,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
   d = dayOfCommonEra(when);
   d -= GREGORIAN_REFERENCE;
   d *= 86400;
-  a = abs(d - (_seconds_since_ref + offset(_time_zone, self)));
+  a = fabs(d - (_seconds_since_ref + offset(_time_zone, self)));
   b = a / 3600;
   h = (NSInteger)b;
   h = h * 3600;
@@ -1795,7 +1814,7 @@ static inline int getDigits(const char *from, char *to, int limit, BOOL *error)
  */
 - (NSInteger) yearOfCommonEra
 {
-  int m, d, y;
+  NSInteger m, d, y;
   NSTimeInterval	when;
 
   when = _seconds_since_ref + offset(_time_zone, self);
@@ -1850,13 +1869,13 @@ typedef struct {
   unichar	*t;
   unsigned	length;
   unsigned	offset;
-  int		yd;
-  int		md;
-  int		dom;
-  int		hd;
-  int		mnd;
-  int		sd;
-  int		mil;
+  NSInteger	yd;
+  NSInteger	md;
+  NSInteger	dom;
+  NSInteger	hd;
+  NSInteger	mnd;
+  NSInteger	sd;
+  NSInteger	mil;
 } DescriptionInfo;
 
 static void Grow(DescriptionInfo *info, unsigned size)
@@ -2576,12 +2595,7 @@ static void outputValueWithFormat(int v, char *fldfmt, DescriptionInfo *info)
 			     month: (NSInteger *)month
 			      year: (NSInteger *)year
 {
-  int   dd, mm, yy;
-
-  gregorianDateFromAbsolute(d, &dd, &mm, &yy);
-  *day = dd;
-  *month = mm;
-  *year = yy;
+  gregorianDateFromAbsolute(d, day, month, year);
 }
 
 @end
@@ -2603,7 +2617,7 @@ static void outputValueWithFormat(int v, char *fldfmt, DescriptionInfo *info)
   NSTimeInterval	s;
   NSTimeInterval	oldOffset;
   NSTimeInterval	newOffset;
-  int			i, year, month, day, hour, minute, second, mil;
+  NSInteger		i, year, month, day, hour, minute, second, mil;
 
   /* Apply timezone offset to _seconds_since_ref from GMT to local time,
    * then break into components in local time zone.
@@ -2763,9 +2777,9 @@ static void outputValueWithFormat(int v, char *fldfmt, DescriptionInfo *info)
   int			diff;
   int			extra;
   int			sign;
-  int			mil;
-  int			syear, smonth, sday, shour, sminute, ssecond;
-  int			eyear, emonth, eday, ehour, eminute, esecond;
+  NSInteger		mil;
+  NSInteger		syear, smonth, sday, shour, sminute, ssecond;
+  NSInteger		eyear, emonth, eday, ehour, eminute, esecond;
 
   /* FIXME What if the two dates are in different time zones?
     How about daylight savings time?
