@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSCountedSet class reference</title>
-   $Date: 2013-08-22 16:44:54 +0100 (Thu, 22 Aug 2013) $ $Revision: 37003 $
+   $Date: 2015-05-02 09:03:05 +0100 (Sat, 02 May 2015) $ $Revision: 38470 $
    */
 
 #import "common.h"
@@ -91,6 +91,11 @@ static Class NSCountedSet_concrete_class;
     }
 }
 
+- (NSUInteger) _countForObject: (id)anObject
+{
+  return [self countForObject: anObject];
+}
+
 /**
  * Returns the number of times that an object that is equal to the
  * specified object (as determined by the [-isEqual:] method) has
