@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDate class reference</title>
-   $Date: 2013-10-14 23:30:33 +0100 (Mon, 14 Oct 2013) $ $Revision: 37230 $
+   $Date: 2016-03-15 19:04:51 +0000 (Tue, 15 Mar 2016) $ $Revision: 39549 $
    */
 
 #import "common.h"
@@ -49,6 +49,11 @@
 #define DISTANT_FUTURE	63113990400.0
 #define DISTANT_PAST	-63113817600.0
 
+/* On older Solaris we don't have NAN nor nan() */
+#if defined(__sun) && defined(__SVR4) && !defined(NAN)
+#define NAN 0x7fffffffffffffff
+#endif
+
 const NSTimeInterval NSTimeIntervalSince1970 = 978307200.0;
 
 
@@ -1407,6 +1412,18 @@ otherTime(NSDate* other)
 	          format: @"[%@-%@] interval is not a number",
 	NSStringFromClass([self class]), NSStringFromSelector(_cmd)];
     }
+
+#if	GS_SIZEOF_VOIDP == 4
+  if (secs <= DISTANT_PAST)
+    {
+      secs = DISTANT_PAST;
+    }
+  else if (secs >= DISTANT_FUTURE)
+    {
+      secs = DISTANT_FUTURE;
+    }
+#endif
+
   _seconds_since_ref = secs;
   return self;
 }
