@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDateFormatter class reference</title>
-   $Date: 2014-02-18 21:14:11 +0000 (Tue, 18 Feb 2014) $ $Revision: 37695 $
+   $Date: 2016-03-02 15:52:53 +0000 (Wed, 02 Mar 2016) $ $Revision: 39444 $
    */
 
 #define	GS_NSDateFormatter_IVARS \
@@ -63,8 +63,8 @@
 
 @interface NSDateFormatter (PrivateMethods)
 - (void) _resetUDateFormat;
-- (void) _setSymbols: (NSArray *) array : (NSInteger) symbol;
-- (NSArray *) _getSymbols: (NSInteger) symbol;
+- (void) _setSymbols: (NSArray *)array : (NSInteger)symbol;
+- (NSArray *) _getSymbols: (NSInteger)symbol;
 @end
 
 static inline NSInteger
@@ -115,7 +115,8 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   [self _resetUDateFormat];
   
 /* According to Apple docs, default behavior is NSDateFormatterBehavior10_4 on
-   10.5 and later. Yeah, go figure. */
+ * 10.5 and later. Yeah, go figure.
+ */
 #if GS_USE_ICU == 1
   {
     int length;
@@ -296,7 +297,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return _defaultBehavior;
 }
 
-+ (void) setDefaultFormatterBehavior: (NSDateFormatterBehavior) behavior
++ (void) setDefaultFormatterBehavior: (NSDateFormatterBehavior)behavior
 {
   _defaultBehavior = behavior;
 }
@@ -306,7 +307,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return internal->_behavior;
 }
 
-- (void) setFormatterBehavior: (NSDateFormatterBehavior) behavior
+- (void) setFormatterBehavior: (NSDateFormatterBehavior)behavior
 {
   internal->_behavior = behavior;
 }
@@ -316,7 +317,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return NO; // FIXME
 }
 
-- (void) setGeneratesCalendarDates: (BOOL) flag
+- (void) setGeneratesCalendarDates: (BOOL)flag
 {
   return; // FIXME
 }
@@ -330,7 +331,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setLenient: (BOOL) flag
+- (void) setLenient: (BOOL)flag
 {
 #if GS_USE_ICU == 1
   udat_setLenient (internal->_formatter, flag);
@@ -351,7 +352,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   int32_t pPos = 0;
   
   textLength = [string length];
-  text = NSZoneMalloc ([self zone], sizeof(UChar) * textLength);
+  text = malloc(sizeof(UChar) * textLength);
   if (text == NULL)
     return nil;
   
@@ -362,7 +363,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
     result =
       [NSDate dateWithTimeIntervalSince1970: (NSTimeInterval)(date / 1000.0)];
   
-  NSZoneFree ([self zone], text);
+  free(text);
   return result;
 #else
   return nil;
@@ -375,17 +376,16 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   NSString *result;
   int32_t length;
   unichar *string;
-  NSZone *z = [self zone];
   UDate udate = [date timeIntervalSince1970] * 1000.0;
   UErrorCode err = U_ZERO_ERROR;
   
   length = udat_format (internal->_formatter, udate, NULL, 0, NULL, &err);
-  string = NSZoneMalloc (z, sizeof(UChar) * (length + 1));
+  string = malloc(sizeof(UChar) * (length + 1));
   err = U_ZERO_ERROR;
   udat_format (internal->_formatter, udate, string, length, NULL, &err);
   if (U_SUCCESS(err))
     {
-      result = AUTORELEASE([[NSString allocWithZone: z]
+      result = AUTORELEASE([[NSString allocWithZone: NSDefaultMallocZone()]
         initWithBytesNoCopy: string
         length: length * sizeof(UChar)
         encoding: NSUnicodeStringEncoding
@@ -393,7 +393,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
       return result;
     }
   
-  NSZoneFree (z, string);
+  free(string);
   return nil;
 #else
   return nil;
@@ -408,19 +408,19 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return NO; // FIXME
 }
 
-- (void) setDateFormat: (NSString *) string
+- (void) setDateFormat: (NSString *)string
 {
 #if GS_USE_ICU == 1
   UChar *pattern;
   int32_t patternLength;
   
   patternLength = [string length];
-  pattern = NSZoneMalloc ([self zone], sizeof(UChar) * patternLength);
+  pattern = malloc(sizeof(UChar) * patternLength);
   [string getCharacters: pattern range: NSMakeRange(0, patternLength)];
   
   udat_applyPattern (internal->_formatter, 0, pattern, patternLength);
   
-  NSZoneFree ([self zone], pattern);
+  free(pattern);
 #endif
   ASSIGNCOPY(_dateFormat, string);
 }
@@ -430,7 +430,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return internal->_dateStyle;
 }
 
-- (void) setDateStyle: (NSDateFormatterStyle) style
+- (void) setDateStyle: (NSDateFormatterStyle)style
 {
   internal->_dateStyle = style;
   [self _resetUDateFormat];
@@ -441,7 +441,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return internal->_timeStyle;
 }
 
-- (void) setTimeStyle: (NSDateFormatterStyle) style
+- (void) setTimeStyle: (NSDateFormatterStyle)style
 {
   internal->_timeStyle = style;
   [self _resetUDateFormat];
@@ -452,7 +452,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return [internal->_locale objectForKey: NSLocaleCalendar];
 }
 
-- (void) setCalendar: (NSCalendar *) calendar
+- (void) setCalendar: (NSCalendar *)calendar
 {
   NSMutableDictionary *dict;
   NSLocale *locale;
@@ -474,7 +474,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return nil;  // FIXME
 }
 
-- (void) setDefaultDate: (NSDate *) date
+- (void) setDefaultDate: (NSDate *)date
 {
   return; // FIXME
 }
@@ -484,7 +484,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return internal->_locale;
 }
 
-- (void) setLocale: (NSLocale *) locale
+- (void) setLocale: (NSLocale *)locale
 {
   if (locale == internal->_locale)
     return;
@@ -499,7 +499,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return internal->_tz;
 }
 
-- (void) setTimeZone: (NSTimeZone *) tz
+- (void) setTimeZone: (NSTimeZone *)tz
 {
   if (tz == internal->_tz)
     return;
@@ -520,7 +520,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setTwoDigitStartDate: (NSDate *) date
+- (void) setTwoDigitStartDate: (NSDate *)date
 {
 #if GS_USE_ICU == 1
   UErrorCode err = U_ZERO_ERROR;
@@ -560,7 +560,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setPMSymbol: (NSString *) string
+- (void) setPMSymbol: (NSString *)string
 {
   return;
 }
@@ -574,7 +574,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setWeekdaySymbols: (NSArray *) array
+- (void) setWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_WEEKDAYS];
@@ -592,10 +592,10 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortWeekdaySymbols: (NSArray *) array
+- (void) setShortWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
-  [self _getSymbols: UDAT_SHORT_WEEKDAYS];
+  [self _setSymbols: array : UDAT_SHORT_WEEKDAYS];
 #else
   return;
 #endif
@@ -610,10 +610,10 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setMonthSymbols: (NSArray *) array
+- (void) setMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
-  [self _getSymbols: UDAT_MONTHS];
+  [self _setSymbols: array : UDAT_MONTHS];
 #else
   return;
 #endif
@@ -628,10 +628,10 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortMonthSymbols: (NSArray *) array
+- (void) setShortMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
-  [self _getSymbols: UDAT_SHORT_MONTHS];
+  [self _setSymbols: array : UDAT_SHORT_MONTHS];
 #else
   return;
 #endif
@@ -646,7 +646,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setEraSymbols: (NSArray *) array
+- (void) setEraSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_ERAS];
@@ -660,7 +660,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return nil;
 }
 
-- (void) setGregorianStartDate: (NSDate *) date
+- (void) setGregorianStartDate: (NSDate *)date
 {
   return;
 }
@@ -674,7 +674,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setLongEraSymbols: (NSArray *) array
+- (void) setLongEraSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_ERA_NAMES];
@@ -693,7 +693,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setQuarterSymbols: (NSArray *) array
+- (void) setQuarterSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_QUARTERS];
@@ -711,7 +711,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortQuarterSymbols: (NSArray *) array
+- (void) setShortQuarterSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_SHORT_QUARTERS];
@@ -729,7 +729,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setStandaloneQuarterSymbols: (NSArray *) array
+- (void) setStandaloneQuarterSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_QUARTERS];
@@ -747,7 +747,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortStandaloneQuarterSymbols: (NSArray *) array
+- (void) setShortStandaloneQuarterSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_SHORT_QUARTERS];
@@ -765,7 +765,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortStandaloneMonthSymbols: (NSArray *) array
+- (void) setShortStandaloneMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_SHORT_MONTHS];
@@ -783,7 +783,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setStandaloneMonthSymbols: (NSArray *) array
+- (void) setStandaloneMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_MONTHS];
@@ -801,7 +801,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setVeryShortMonthSymbols: (NSArray *) array
+- (void) setVeryShortMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_NARROW_MONTHS];
@@ -819,7 +819,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setVeryShortStandaloneMonthSymbols: (NSArray *) array
+- (void) setVeryShortStandaloneMonthSymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_NARROW_MONTHS];
@@ -837,7 +837,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setShortStandaloneWeekdaySymbols: (NSArray *) array
+- (void) setShortStandaloneWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_SHORT_WEEKDAYS];
@@ -855,7 +855,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setStandaloneWeekdaySymbols: (NSArray *) array
+- (void) setStandaloneWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_WEEKDAYS];
@@ -873,7 +873,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setVeryShortWeekdaySymbols: (NSArray *) array
+- (void) setVeryShortWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_SHORT_WEEKDAYS];
@@ -891,7 +891,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 #endif
 }
 
-- (void) setVeryShortStandaloneWeekdaySymbols: (NSArray *) array
+- (void) setVeryShortStandaloneWeekdaySymbols: (NSArray *)array
 {
 #if GS_USE_ICU == 1
   [self _setSymbols: array : UDAT_STANDALONE_NARROW_WEEKDAYS];
@@ -955,7 +955,7 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
   return (internal->_dateStyle & FormatterDoesRelativeDateFormatting) ? YES : NO;
 }
 
-- (void) setDoesRelativeDateFormatting: (BOOL) flag
+- (void) setDoesRelativeDateFormatting: (BOOL)flag
 {
   internal->_dateStyle |= FormatterDoesRelativeDateFormatting;
 }
@@ -965,7 +965,9 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
 - (void) _resetUDateFormat
 {
 #if GS_USE_ICU == 1
+  UChar *pat;
   UChar *tzID;
+  int32_t patLength;
   int32_t tzIDLength;
   UErrorCode err = U_ZERO_ERROR;
   
@@ -973,86 +975,121 @@ static NSDateFormatterBehavior _defaultBehavior = 0;
     udat_close (internal->_formatter);
   
   tzIDLength = [[internal->_tz name] length];
-  tzID = NSZoneMalloc ([self zone], sizeof(UChar) * tzIDLength);
-  [[internal->_tz name] getCharacters: tzID range: NSMakeRange (0, tzIDLength)];
+  tzID = malloc(sizeof(UChar) * tzIDLength);
+  [[internal->_tz name] getCharacters: tzID];
   
+  if (nil == self->_dateFormat)
+    {
+      patLength = 0;
+      pat = 0;
+    }
+  else
+    {
+      patLength = [self->_dateFormat length];
+      pat = malloc(sizeof(UChar) * patLength);
+      [self->_dateFormat getCharacters: pat];
+    }
   internal->_formatter = udat_open (NSToUDateFormatStyle(internal->_timeStyle),
                           NSToUDateFormatStyle(internal->_dateStyle),
                           [[internal->_locale localeIdentifier] UTF8String],
                           tzID,
                           tzIDLength,
-                          NULL,
-                          0,
+                          pat,
+                          patLength,
                           &err);
   if (U_FAILURE(err))
     internal->_formatter = NULL;
-  
-  NSZoneFree ([self zone], tzID);
+  if (0 != pat) free(pat);
+  free(tzID);
 #else
   return;
 #endif
 }
 
-- (void) _setSymbols: (NSArray *) array : (NSInteger) symbol
-{
 #if GS_USE_ICU == 1
-  int idx = 0;
-  int count = udat_countSymbols (internal->_formatter, symbol);
-  
-  if ([array count] != count)
-    return;
-  
-  while (idx < count)
+static inline void
+symbolRange(NSInteger symbol, int *from)
+{
+  switch (symbol)
     {
-      int length;
-      UChar *value;
-      UErrorCode err = U_ZERO_ERROR;
-      NSString *string = [array objectAtIndex: idx];
-      
-      length = [string length];
-      value = NSZoneMalloc ([self zone], sizeof(unichar) * length);
-      [string getCharacters: value range: NSMakeRange(0, length)];
-      
-      udat_setSymbols (internal->_formatter, symbol, idx, value, length, &err);
-      
-      NSZoneFree ([self zone], value);
-      
-      ++idx;
+      case UDAT_SHORT_WEEKDAYS:
+      case UDAT_STANDALONE_NARROW_WEEKDAYS:
+      case UDAT_STANDALONE_SHORT_WEEKDAYS:
+      case UDAT_STANDALONE_WEEKDAYS:
+      case UDAT_WEEKDAYS:
+        /* In ICU days of the week number from 1 rather than zero.
+         */
+        *from = 1;
+        break;
+
+      default:
+        *from = 0;
+        break;
     }
-#else
-  return;
-#endif
 }
+#endif
 
-- (NSArray *) _getSymbols: (NSInteger) symbol
+- (void) _setSymbols: (NSArray*)array : (NSInteger)symbol
 {
 #if GS_USE_ICU == 1
-  NSMutableArray *mArray;
-  int idx = 0;
+  int idx;
   int count = udat_countSymbols (internal->_formatter, symbol);
   
-  mArray = [NSMutableArray arrayWithCapacity: count];
+  symbolRange(symbol, &idx);
+  if ([array count] == count - idx)
+    {
+      while (idx < count)
+        {
+          int           length;
+          UChar         *value;
+          UErrorCode    err = U_ZERO_ERROR;
+          NSString      *string = [array objectAtIndex: idx];
+          
+          length = [string length];
+          value = malloc(sizeof(unichar) * length);
+          [string getCharacters: value range: NSMakeRange(0, length)];
+          udat_setSymbols(internal->_formatter, symbol, idx,
+            value, length, &err);
+          free(value);
+          ++idx;
+        }
+    }
+#endif
+  return;
+}
+
+- (NSArray *) _getSymbols: (NSInteger)symbol
+{
+#if GS_USE_ICU == 1
+  NSMutableArray        *mArray;
+  int                   idx;
+  int                   count;
+  
+  count = udat_countSymbols(internal->_formatter, symbol);
+  symbolRange(symbol, &idx);
+  mArray = [NSMutableArray arrayWithCapacity: count - idx];
   while (idx < count)
     {
-      int length;
-      unichar *value;
-      NSString *str;
-      NSZone *z = [self zone];
-      UErrorCode err = U_ERROR_LIMIT;
+      int               length;
+      unichar           *value;
+      NSString          *str;
+      NSZone            *z = [self zone];
+      UErrorCode        err = U_ERROR_LIMIT;
       
       length
-	= udat_getSymbols (internal->_formatter, symbol, idx, NULL, 0, &err);
-      value = NSZoneMalloc (z, sizeof(unichar) * (length + 1));
+        = udat_getSymbols(internal->_formatter, symbol, idx, NULL, 0, &err);
+      value = NSZoneMalloc(z, sizeof(unichar) * (length + 1));
       err = U_ZERO_ERROR;
-      udat_getSymbols (internal->_formatter, symbol, idx, value, length, &err);
+      udat_getSymbols(internal->_formatter, symbol, idx, value, length, &err);
       if (U_SUCCESS(err))
         {
-          str = AUTORELEASE([[NSString allocWithZone: z]
+          str = [[NSString allocWithZone: z]
             initWithBytesNoCopy: value
             length: length * sizeof(unichar)
             encoding: NSUnicodeStringEncoding
-            freeWhenDone: YES]);
+            freeWhenDone: YES];
           [mArray addObject: str];
+          RELEASE(str);
         }
       else
         {
