@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDebug utilities reference</title>
-   $Date: 2014-05-09 09:04:58 +0100 (Fri, 09 May 2014) $ $Revision: 37873 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
    */
 
 #import "common.h"
@@ -40,6 +40,8 @@
 #import "Foundation/NSThread.h"
 #import "Foundation/NSValue.h"
 
+#import "GSSorting.h"
+
 #if     HAVE_EXECINFO_H
 #include        <execinfo.h>
 #endif
@@ -59,6 +61,19 @@ typedef struct {
   unsigned int   stack_size;
 } table_entry;
 
+typedef struct {
+  const char    *name;
+  int           count;
+} list_entry;
+
+static NSInteger itemComp(id v0, id v1, void *ctxt)
+{
+  int i = strcmp(((list_entry*)v0)->name, ((list_entry *)v1)->name);
+  if (i < 0) return NSOrderedAscending;
+  if (i > 0) return NSOrderedDescending;
+  return NSOrderedSame;
+}
+
 static	unsigned int	num_classes = 0;
 static	unsigned int	table_size = 0;
 
@@ -66,10 +81,14 @@ static table_entry*	the_table = 0;
 
 static BOOL	debug_allocation = NO;
 
-static GSLazyRecursiveLock	*uniqueLock = nil;
+static NSRecursiveLock	*uniqueLock = nil;
+static SEL              doLockSel = 0;
+static SEL              unLockSel = 0;
+static IMP              doLockImp = 0;
+static IMP              unLockImp = 0;
 
-static const char*	_GSDebugAllocationList(BOOL difference);
-static const char*	_GSDebugAllocationListAll(void);
+static void     _GSDebugAllocationFetch(list_entry *items, BOOL difference);
+static void     _GSDebugAllocationFetchAll(list_entry *items);
 
 static void _GSDebugAllocationAdd(Class c, id o);
 static void _GSDebugAllocationRemove(Class c, id o);
@@ -79,6 +98,9 @@ static void (*_GSDebugAllocationAddFunc)(Class c, id o)
 static void (*_GSDebugAllocationRemoveFunc)(Class c, id o)
   = _GSDebugAllocationRemove;
 
+#define doLock() (*doLockImp)(uniqueLock, doLockSel)
+#define unLock() (*unLockImp)(uniqueLock, unLockSel)
+
 @interface GSDebugAlloc : NSObject
 + (void) initialize;
 @end
@@ -86,7 +108,11 @@ static void (*_GSDebugAllocationRemoveFunc)(Class c, id o)
 @implementation GSDebugAlloc
 + (void) initialize
 {
-  uniqueLock = [GSLazyRecursiveLock new];
+  uniqueLock = [NSRecursiveLock new];
+  doLockSel = @selector(lock);
+  unLockSel = @selector(unlock);
+  doLockImp = [uniqueLock methodForSelector: doLockSel];
+  unLockImp = [uniqueLock methodForSelector: unLockSel];
   [[NSObject leakAt: &uniqueLock] release];
 }
 @end
@@ -95,7 +121,7 @@ void
 GSSetDebugAllocationFunctions(void (*newAddObjectFunc)(Class c, id o),
   void (*newRemoveObjectFunc)(Class c, id o))
 {
-  [uniqueLock lock];
+  doLock();
 
   if (newAddObjectFunc && newRemoveObjectFunc)
     {	   	
@@ -109,7 +135,7 @@ GSSetDebugAllocationFunctions(void (*newAddObjectFunc)(Class c, id o),
       _GSDebugAllocationRemoveFunc = _GSDebugAllocationRemove;
     }
 
-  [uniqueLock unlock];
+  unLock();
 }
 
 BOOL
@@ -137,7 +163,7 @@ GSDebugAllocationRecordObjects(Class c, BOOL newState)
     {
       if (the_table[i].class == c)
 	{
-	  [uniqueLock lock];
+	  doLock();
           oldState = (YES == the_table[i].is_recording) ? YES : NO;
           if (newState)
             {
@@ -156,13 +182,13 @@ GSDebugAllocationRecordObjects(Class c, BOOL newState)
                   the_table[i].recorded_tags[j] = nil;
                 }
             }
-	  [uniqueLock unlock];
+	  unLock();
 	  return oldState;
 	}
     }
   if (YES == newState)
     {
-      [uniqueLock lock];
+      doLock();
       if (num_classes >= table_size)
         {
           int		more = table_size + 128;
@@ -172,7 +198,7 @@ GSDebugAllocationRecordObjects(Class c, BOOL newState)
 
           if (tmp == 0)
             {
-              [uniqueLock unlock];
+              unLock();
               return NO;
             }
           if (the_table)
@@ -194,7 +220,7 @@ GSDebugAllocationRecordObjects(Class c, BOOL newState)
       the_table[num_classes].num_recorded_objects = 0;
       the_table[num_classes].stack_size = 0;
       num_classes++;
-      [uniqueLock unlock];
+      unLock();
     }
   return oldState;
 }
@@ -222,7 +248,7 @@ _GSDebugAllocationAdd(Class c, id o)
 	{
 	  if (the_table[i].class == c)
 	    {
-	      [uniqueLock lock];
+	      doLock();
 	      the_table[i].count++;
 	      the_table[i].total++;
 	      if (the_table[i].count > the_table[i].peak)
@@ -242,7 +268,7 @@ _GSDebugAllocationAdd(Class c, id o)
 					 more * sizeof(id));
 		      if (tmp == 0)
 			{
-			  [uniqueLock unlock];
+			  unLock();
 			  return;
 			}
 
@@ -251,7 +277,7 @@ _GSDebugAllocationAdd(Class c, id o)
 		      if (tmp1 == 0)
 			{
 			  NSZoneFree(NSDefaultMallocZone(),  tmp);
-			  [uniqueLock unlock];
+			  unLock();
 			  return;
 			}
 
@@ -280,11 +306,11 @@ _GSDebugAllocationAdd(Class c, id o)
 		    [the_table[i].num_recorded_objects] = nil;
 		  the_table[i].num_recorded_objects++;
 		}
-	      [uniqueLock unlock];
+	      unLock();
 	      return;
 	    }
 	}
-      [uniqueLock lock];
+      doLock();
       if (num_classes >= table_size)
 	{
 	  unsigned int	more = table_size + 128;
@@ -294,7 +320,7 @@ _GSDebugAllocationAdd(Class c, id o)
 	
 	  if (tmp == 0)
 	    {
-	      [uniqueLock unlock];
+	      unLock();
 	      return;		/* Argh	*/
 	    }
 	  if (the_table)
@@ -316,7 +342,7 @@ _GSDebugAllocationAdd(Class c, id o)
       the_table[num_classes].num_recorded_objects = 0;
       the_table[num_classes].stack_size = 0;
       num_classes++;
-      [uniqueLock unlock];
+      unLock();
     }
 }
 
@@ -372,7 +398,7 @@ GSDebugAllocationClassList()
   size_t siz;
   unsigned int	i;
 
-  [uniqueLock lock];
+  doLock();
 
   siz = sizeof(Class) * (num_classes + 1);
   ans = NSZoneMalloc(NSDefaultMallocZone(), siz);
@@ -383,7 +409,7 @@ GSDebugAllocationClassList()
     }
   ans[num_classes] = NULL;
 
-  [uniqueLock unlock];
+  unLock();
 
   return ans;
 }
@@ -391,169 +417,164 @@ GSDebugAllocationClassList()
 const char*
 GSDebugAllocationList(BOOL changeFlag)
 {
-  const char	*ans;
-  NSData	*d;
+  list_entry    *items;
+  unsigned      size;
 
   if (debug_allocation == NO)
     {
       return "Debug allocation system is not active!\n";
     }
-  [uniqueLock lock];
-  ans = _GSDebugAllocationList(changeFlag);
-  d = [NSData dataWithBytes: ans length: strlen(ans) + 1];
-  [uniqueLock unlock];
-  return (const char*)[d bytes];
-}
 
-static const char*
-_GSDebugAllocationList(BOOL difference)
-{
-  unsigned int	pos = 0;
-  unsigned int	i;
-  static unsigned int	siz = 0;
-  static char	*buf = 0;
-
-  for (i = 0; i < num_classes; i++)
+  doLock();
+  size = num_classes;
+  if (size > 0)
     {
-      int	val = the_table[i].count;
-
-      if (difference)
-	{
-	  val -= the_table[i].lastc;
-	}
-      if (val != 0)
-	{
-	  pos += 22 + strlen(class_getName(the_table[i].class));
-	}
+      items = malloc(sizeof(list_entry) * size);
+      _GSDebugAllocationFetch(items, changeFlag);
     }
-  if (pos == 0)
+  else
     {
-      if (difference)
-	{
-	  return "There are NO newly allocated or deallocated object!\n";
-	}
+      items = 0;
+    }
+  unLock();
+
+  while (size > 0 && 0 == items[size - 1].name)
+    {
+      size--;
+    }
+  if (0 == size)
+    {
+      if (items != 0)
+        {
+          free(items);
+        }
+      if (changeFlag)
+        {
+          return "There are NO newly allocated or deallocated object!\n";
+        }
       else
-	{
-	  return "I can find NO allocated object!\n";
-	}
+        {
+          return "I can find NO allocated object!\n";
+        }
     }
-
-  pos++;
-
-  if (pos > siz)
+  else
     {
-      if (pos & 0xff)
-	{
-	  pos = ((pos >> 8) + 1) << 8;
-	}
-      siz = pos;
-      if (buf)
-	{
-	  NSZoneFree(NSDefaultMallocZone(), buf);
-	}
-      buf = NSZoneMalloc(NSDefaultMallocZone(), siz);
+      NSMutableString   *result;
+      id                order[size];
+      unsigned          index;
+
+      for (index = 0; index < size; index++)
+        {
+          order[index] = (id)&items[index];
+        }
+      GSSortUnstable(order, NSMakeRange(0,size), (id)itemComp,
+        GSComparisonTypeFunction, 0);
+
+      result = [NSMutableString stringWithCapacity: 1000];
+      for (index = 0; index < size; index++)
+        {
+          list_entry    *item = (list_entry*)order[index];
+
+          [result appendFormat: @"%d\t%s\n", item->count, item->name];
+        }
+      free(items);
+      return [result UTF8String];
     }
-
-  if (0 == buf)
-    {
-      return "";
-    }
-
-  pos = 0;
-  for (i = 0; i < num_classes; i++)
-    {
-      int	val = the_table[i].count;
-
-      if (difference)
-	{
-	  val -= the_table[i].lastc;
-	}
-      the_table[i].lastc = the_table[i].count;
-
-      if (val != 0)
-	{
-	  snprintf(&buf[pos], siz - pos, "%d\t%s\n",
-	    val, class_getName(the_table[i].class));
-	  pos += strlen(&buf[pos]);
-	}
-    }
-
-  return buf;
 }
 
 const char*
 GSDebugAllocationListAll()
 {
-  const char	*ans;
-  NSData	*d;
+  list_entry    *items;
+  unsigned      size;
 
   if (debug_allocation == NO)
     {
       return "Debug allocation system is not active!\n";
     }
-  [uniqueLock lock];
-  ans = _GSDebugAllocationListAll();
-  d = [NSData dataWithBytes: ans length: strlen(ans)+1];
-  [uniqueLock unlock];
-  return (const char*)[d bytes];
-}
 
-static const char*
-_GSDebugAllocationListAll(void)
-{
-  unsigned int	pos = 0;
-  unsigned int	i;
-  static unsigned int	siz = 0;
-  static char	*buf = 0;
-
-  for (i = 0; i < num_classes; i++)
+  doLock();
+  size = num_classes;
+  if (size > 0)
     {
-      int	val = the_table[i].total;
-
-      if (val != 0)
-	{
-	  pos += 22 + strlen(class_getName(the_table[i].class));
-	}
+      items = malloc(sizeof(list_entry) * size);
+      _GSDebugAllocationFetchAll(items);
     }
-  if (pos == 0)
+  else
+    {
+      items = 0;
+    }
+  unLock();
+
+  if (0 == items)
     {
       return "I can find NO allocated object!\n";
     }
-  pos++;
-
-  if (pos > siz)
+  else
     {
-      if (pos & 0xff)
-	{
-	  pos = ((pos >> 8) + 1) << 8;
-	}
-      siz = pos;
-      if (buf)
-	{
-	  NSZoneFree(NSDefaultMallocZone(), buf);
-	}
-      buf = NSZoneMalloc(NSDefaultMallocZone(), siz);
-    }
+      NSMutableString   *result;
+      id                order[size];
+      unsigned          index;
 
-  if (0 == buf)
+      for (index = 0; index < size; index++)
+        {
+          order[index] = (id)&items[index];
+        }
+      GSSortUnstable(order, NSMakeRange(0,size), (id)itemComp,
+        GSComparisonTypeFunction, 0);
+
+      result = [NSMutableString stringWithCapacity: 1000];
+      for (index = 0; index < size; index++)
+        {
+          list_entry    *item = (list_entry*)order[index];
+
+          [result appendFormat: @"%d\t%s\n", item->count, item->name];
+        }
+      free(items);
+      return [result UTF8String];
+    }
+}
+
+static void
+_GSDebugAllocationFetch(list_entry *items, BOOL difference)
+{
+  unsigned      i;
+  unsigned      pos;
+
+  for (i = pos = 0; i < num_classes; i++)
     {
-      return "";
+      int	val = the_table[i].count;
+
+      if (difference)
+	{
+	  val -= the_table[i].lastc;
+          the_table[i].lastc = the_table[i].count;
+	}
+      if (val)
+        {
+          items[pos].name = class_getName(the_table[i].class);
+          items[pos].count = val;
+          pos++;
+        }
     }
+  while (pos < num_classes)
+    {
+      items[pos].name = 0;
+      items[pos].count = 0;
+      pos++;
+    }
+}
+
+static void
+_GSDebugAllocationFetchAll(list_entry *items)
+{
+  unsigned      i;
 
-  pos = 0;
   for (i = 0; i < num_classes; i++)
     {
-      int	val = the_table[i].total;
-
-      if (val != 0)
-	{
-	  snprintf(&buf[pos], siz - pos, "%d\t%s\n",
-	    val, class_getName(the_table[i].class));
-	  pos += strlen(&buf[pos]);
-	}
+      items[i].name = class_getName(the_table[i].class);
+      items[i].count = the_table[i].total;
     }
-
-  return buf;
 }
 
 void
@@ -575,7 +596,7 @@ _GSDebugAllocationRemove(Class c, id o)
 	    {
 	      id	tag = nil;
 
-	      [uniqueLock lock];
+	      doLock();
 	      the_table[i].count--;
 	      if (the_table[i].is_recording)
 		{
@@ -610,7 +631,7 @@ _GSDebugAllocationRemove(Class c, id o)
 		      ;
 		    }
 		}
-	      [uniqueLock unlock];
+	      unLock();
 	      [tag release];
 	      return;
 	    }
@@ -630,7 +651,7 @@ GSDebugAllocationTagRecordedObject(id object, id tag)
     {
       return nil;
     }
-  [uniqueLock lock];
+  doLock();
 
   for (i = 0; i < num_classes; i++)
     {
@@ -644,7 +665,7 @@ GSDebugAllocationTagRecordedObject(id object, id tag)
     || the_table[i].is_recording == NO
     || the_table[i].num_recorded_objects == 0)
     {
-      [uniqueLock unlock];
+      unLock();
       return nil;
     }
 
@@ -658,7 +679,7 @@ GSDebugAllocationTagRecordedObject(id object, id tag)
 	}
     }
 
-  [uniqueLock unlock];
+  unLock();
   return AUTORELEASE(o);
 }
 
@@ -674,7 +695,7 @@ GSDebugAllocationListRecordedObjects(Class c)
       return nil;
     }
 
-  [uniqueLock lock];
+  doLock();
 
   for (i = 0; i < num_classes; i++)
     {
@@ -686,19 +707,19 @@ GSDebugAllocationListRecordedObjects(Class c)
 
   if (i == num_classes)
     {
-      [uniqueLock unlock];
+      unLock();
       return nil;
     }
 
   if (the_table[i].is_recording == NO)
     {
-      [uniqueLock unlock];
+      unLock();
       return nil;
     }
 
   if (the_table[i].num_recorded_objects == 0)
     {
-      [uniqueLock unlock];
+      unLock();
       return [NSArray array];
     }
 
@@ -706,7 +727,7 @@ GSDebugAllocationListRecordedObjects(Class c)
     the_table[i].num_recorded_objects * sizeof(id));
   if (tmp == 0)
     {
-      [uniqueLock unlock];
+      unLock();
       return nil;
     }
 
@@ -724,7 +745,7 @@ GSDebugAllocationListRecordedObjects(Class c)
 #endif
 
   /* Then, we bravely unlock the lock */
-  [uniqueLock unlock];
+  unLock();
 
   /* Only then we create an array with them - this is now safe as we
    * have copied the objects out, unlocked, and retained them. */
@@ -775,7 +796,7 @@ __builtin_extract_return_address(__builtin_return_address(a + 1)) : 0); break;
 
 #include <setjmp.h>
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 #ifndef SIGBUS
 #define SIGBUS  SIGILL
 #endif
