@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDictionary class reference</title>
-   $Date: 2013-12-05 13:16:36 +0000 (Thu, 05 Dec 2013) $ $Revision: 37431 $
+   $Date: 2016-02-22 21:04:18 +0000 (Mon, 22 Feb 2016) $ $Revision: 39406 $
    */
 
 #import "common.h"
@@ -863,7 +863,7 @@ static SEL	appSel;
 }
 
 - (void)getObjects: (__unsafe_unretained id[])objects
-           andKeys: (__unsafe_unretained id[])keys
+           andKeys: (__unsafe_unretained id<NSCopying>[])keys
 {
   int i=0;
   FOR_IN(id, key, self)
@@ -1229,6 +1229,32 @@ compareIt(id o1, id o2, void* context)
   [self subclassResponsibility: _cmd];
   return 0;
 }
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger	size = [super sizeInBytesExcluding: exclude];
+
+  if (size > 0)
+    {
+      NSUInteger	count = [self count];
+
+      size += 3 * sizeof(void*) * count;
+      if (count > 0)
+        {
+	  NSEnumerator  *enumerator = [self keyEnumerator];
+	  NSObject<NSCopying>	*k = nil;
+
+	  while ((k = [enumerator nextObject]) != nil)
+	    {
+	      NSObject	*o = [self objectForKey: k];
+
+	      size += [k sizeInBytesExcluding: exclude];
+	      size += [o sizeInBytesExcluding: exclude];
+	    }
+	}
+    }
+  return size;
+}
 @end
 
 
