@@ -23,7 +23,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistantObject class reference</title>
-   $Date: 2013-04-13 11:40:15 +0100 (Sat, 13 Apr 2013) $ $Revision: 36517 $
+   $Date: 2016-03-17 08:12:45 +0000 (Thu, 17 Mar 2016) $ $Revision: 39554 $
    */
 
 #import "common.h"
@@ -380,13 +380,16 @@ GS_ROOT_CLASS @interface	GSDistantObjectPlaceHolder
 @end
 
 @interface NSDistantObject (Debug)
-+ (void) setDebug: (int)val;
++ (int) setDebug: (int)val;
 @end
 
 @implementation NSDistantObject (Debug)
-+ (void) setDebug: (int)val
++ (int) setDebug: (int)val
 {
+  int   old = debug_proxy;
+
   debug_proxy = val;
+  return old;
 }
 @end
 
@@ -603,6 +606,11 @@ GS_ROOT_CLASS @interface	GSDistantObjectPlaceHolder
       @"You should request NSConnectionDidDieNotification's and\n"
       @"release all references to the proxy's of invalid Connections."];
 
+  /* We could be released while the connection is forwarding, so we need
+   * to retain self.  But, the remote end couild raise an exception, so
+   * we can't rely on being able to release again; use autorelease.
+   */
+  AUTORELEASE(RETAIN(self));
   [_connection forwardInvocation: anInvocation forProxy: self];
 }
 
