@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSDistributedNotificationCenter class reference</title>
-   $Date: 2011-06-19 03:26:03 -0600 (Sun, 19 Jun 2011) $ $Revision: 33344 $
+   $Date: 2013-08-22 16:44:54 +0100 (Thu, 22 Aug 2013) $ $Revision: 37003 $
    */
 
 #import	"common.h"
@@ -38,7 +38,6 @@
 #import	"Foundation/NSRunLoop.h"
 #import	"Foundation/NSTask.h"
 #import	"GNUstepBase/NSTask+GNUstepBase.h"
-#import	"GNUstepBase/NSObject+GNUstepBase.h"
 #import	"Foundation/NSDistributedNotificationCenter.h"
 #import	"Foundation/NSUserDefaults.h"
 #import	"Foundation/NSHost.h"
@@ -118,7 +117,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
  * equivalent to calling +notificationCenterForType: with
  * <code>NSLocalNotificationCenterType</code> as its argument.
  */
-+ (NSNotificationCenter*) defaultCenter
++ (id) defaultCenter
 {
   return [self notificationCenterForType: NSLocalNotificationCenterType];
 }
@@ -135,7 +134,7 @@ static NSDistributedNotificationCenter	*netCenter = nil;
  * a notification center used by processes on the local network.<br />
  * MacOS-X supports only <code>NSLocalNotificationCenterType</code>.
  */
-+ (NSNotificationCenter*) notificationCenterForType: (NSString*)type
++ (NSDistributedNotificationCenter*) notificationCenterForType: (NSString*)type
 {
   if ([type isEqual: NSLocalNotificationCenterType] == YES)
     {
