@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileHandle class reference</title>
-   $Date: 2013-11-30 08:57:20 +0000 (Sat, 30 Nov 2013) $ $Revision: 37420 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
    */
 
 #import "common.h"
@@ -887,7 +887,7 @@ NSString * const NSFileHandleOperationException
 
 #if     defined(HAVE_GNUTLS)
 
-#if	!defined(__MINGW__)
+#if	!defined(_WIN32)
 
 @interface      GSTLSHandle : GSFileHandle
 {
@@ -1001,12 +1001,14 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 
   if (YES == [session active])
     {
+      *result =  YES;
       return YES;	/* Already connected.	*/
     }
 
   if (YES == isStandardFile)
     {
       NSLog(@"Attempt to perform ssl handshake with a standard file");
+      *result =  NO;
       return YES;
     }
 
@@ -1046,6 +1048,11 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 
   if (NO == [session handshake])
     {
+      *result = NO;
+      if (nil == session)
+        {
+          return YES;   // Unable to create session
+        }
       return NO;        // Need more.
     }
   else
@@ -1075,7 +1082,7 @@ GSTLSHandlePush(gnutls_transport_ptr_t handle, const void *buffer, size_t len)
 }
 
 @end
-#endif  /* MINGW */
+#endif  /* _WIN32 */
 
 #endif  /* HAVE_GNUTLS */
 
