@@ -1,7 +1,7 @@
 /**
    NSFileManager.m
 
-   Copyright (C) 1997-2002 Free Software Foundation, Inc.
+   Copyright (C) 1997-2015 Free Software Foundation, Inc.
 
    Author: Mircea Oancea <mircea@jupiter.elcom.pub.ro>
    Author: Ovidiu Predescu <ovidiu@net-community.com>
@@ -34,7 +34,7 @@
    Boston, MA 02111 USA.
 
    <title>NSFileManager class reference</title>
-   $Date: 2014-07-25 11:38:20 +0100 (Fri, 25 Jul 2014) $ $Revision: 38010 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
 */
 
 /* The following define is needed for Solaris get(pw/gr)(nam/uid)_r declartions
@@ -81,7 +81,7 @@
 #  include <windows.h>
 #endif
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 #include <stdio.h>
 #include <tchar.h>
 #include <wchar.h>
@@ -92,8 +92,8 @@
 
 /* determine filesystem max path length */
 
-#if defined(_POSIX_VERSION) || defined(__WIN32__)
-# if defined(__MINGW__)
+#if defined(_POSIX_VERSION) || defined(_WIN32)
+# if defined(_WIN32)
 #   include <sys/utime.h>
 # else
 #   include <utime.h>
@@ -185,7 +185,7 @@
  * Macros to handle unichar filesystem support.
  */
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 
 #define	_CHMOD(A,B)	_wchmod(A,B)
 #define	_CLOSEDIR(A)	_wclosedir(A)
@@ -371,7 +371,7 @@ static NSStringEncoding	defaultEncoding;
     {
       bundleClass = [NSBundle class];
     }
-#if defined(__MINGW__)
+#if defined(_WIN32)
   return SetCurrentDirectoryW(lpath) == TRUE ? YES : NO;
 #else
   return (chdir(lpath) == 0) ? YES : NO;
@@ -400,7 +400,7 @@ static NSStringEncoding	defaultEncoding;
   old = [self fileAttributesAtPath: path traverseLink: YES];
   lpath = [defaultManager fileSystemRepresentationWithPath: path];
 
-#ifndef __MINGW__
+#ifndef _WIN32
   if (object_getClass(attributes) == GSAttrDictionaryClass)
     {
       num = ((GSAttrDictionary*)attributes)->statbuf.st_uid;
@@ -528,7 +528,7 @@ static NSStringEncoding	defaultEncoding;
 	  ASSIGN(_lastError, str);
 	}
     }
-#endif	/* __MINGW__ */
+#endif	/* _WIN32 */
 
   num = [attributes filePosixPermissions];
   if (num != NSNotFound && num != [old filePosixPermissions])
@@ -549,7 +549,7 @@ static NSStringEncoding	defaultEncoding;
       BOOL		ok = NO;
       struct _STATB	sb;
 
-#if  defined(__WIN32__) || defined(_POSIX_VERSION)
+#if  defined(_WIN32) || defined(_POSIX_VERSION)
       struct _UTIMB ub;
 #else
       time_t ub[2];
@@ -559,7 +559,7 @@ static NSStringEncoding	defaultEncoding;
 	{
 	  ok = NO;
 	}
-#if  defined(__WIN32__)
+#if  defined(_WIN32)
       else if (sb.st_mode & _S_IFDIR)
 	{
 	  ok = YES;	// Directories don't have modification times.
@@ -567,7 +567,7 @@ static NSStringEncoding	defaultEncoding;
 #endif
       else
 	{
-#if  defined(__WIN32__) || defined(_POSIX_VERSION)
+#if  defined(_WIN32) || defined(_POSIX_VERSION)
 	  ub.actime = sb.st_atime;
 	  ub.modtime = [date timeIntervalSince1970];
 	  ok = (_UTIME(lpath, &ub) == 0);
@@ -693,6 +693,76 @@ static NSStringEncoding	defaultEncoding;
     }
 }
 
+- (NSArray*) contentsOfDirectoryAtURL: (NSURL*)url
+           includingPropertiesForKeys: (NSArray*)keys
+                              options: (NSDirectoryEnumerationOptions)mask
+                                error: (NSError **)error
+{
+  NSArray               *result;
+  NSDirectoryEnumerator *direnum;
+  NSString              *path;
+  
+  DESTROY(_lastError);
+
+  if (![[url scheme] isEqualToString: @"file"])
+    {
+      return nil;
+    }
+  path = [url path];
+  
+  direnum = [[NSDirectoryEnumerator alloc]
+		       initWithDirectoryPath: path
+                   recurseIntoSubdirectories: NO
+                              followSymlinks: NO
+                                justContents: NO
+                                         for: self];
+
+  /* we make an array of NSURLs */
+  result = nil;
+  if (nil != direnum)
+    {
+      IMP	        nxtImp;
+      NSMutableArray    *urlArray;
+      NSString          *tempPath;
+
+
+      nxtImp = [direnum methodForSelector: @selector(nextObject)];
+
+      urlArray = [NSMutableArray arrayWithCapacity:128];
+      while ((tempPath = (*nxtImp)(direnum, @selector(nextObject))) != nil)
+	{
+          NSURL         *tempURL;
+          NSString      *lastComponent;
+      
+          tempURL = [NSURL fileURLWithPath: tempPath];
+          lastComponent = [tempPath lastPathComponent];
+          
+          /* we purge files beginning with . */
+          if (!((mask & NSDirectoryEnumerationSkipsHiddenFiles)
+            && [lastComponent hasPrefix:@"."]))
+            {
+              [urlArray addObject: tempURL];
+            }
+	}
+      RELEASE(direnum);
+ 
+      if ([urlArray count] > 0)
+        {
+          result = [NSArray arrayWithArray: urlArray];
+        }
+    }
+
+  if (error != NULL)
+    {
+      if (nil == result)
+	{
+	  *error = [self _errorFrom: path to: nil];
+	}
+    }
+
+  return result;  
+}
+
 - (NSArray*) contentsOfDirectoryAtPath: (NSString*)path error: (NSError**)error
 {
   NSArray       *result;
@@ -721,7 +791,7 @@ static NSStringEncoding	defaultEncoding;
 - (BOOL) createDirectoryAtPath: (NSString *)path
    withIntermediateDirectories: (BOOL)flag
 		    attributes: (NSDictionary *)attributes
-			 error: (NSError **) error
+			 error: (NSError **)error
 {
   BOOL result = NO;
 
@@ -741,6 +811,11 @@ static NSStringEncoding	defaultEncoding;
 	      result = [self createDirectoryAtPath: dir
 		     			attributes: attributes];
 	    }
+          // an existing not created dir is equivalent to a created one
+          else
+            {
+              result = YES;
+            }
 	}
     }
   else
@@ -821,7 +896,7 @@ static NSStringEncoding	defaultEncoding;
     }
   else
     {
-#if defined(__MINGW__)
+#if defined(_WIN32)
       const _CHAR   *lpath;
           
       lpath = [self fileSystemRepresentationWithPath: path];
@@ -882,7 +957,7 @@ static NSStringEncoding	defaultEncoding;
 		 contents: (NSData*)contents
 	       attributes: (NSDictionary*)attributes
 {
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   const _CHAR *lpath = [self fileSystemRepresentationWithPath: path];
   HANDLE fh;
   DWORD	written = 0;
@@ -901,7 +976,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   fh = CreateFileW(lpath, GENERIC_WRITE, 0, 0, CREATE_ALWAYS,
     FILE_ATTRIBUTE_NORMAL, 0);
   if (fh == INVALID_HANDLE_VALUE)
@@ -950,7 +1025,7 @@ static NSStringEncoding	defaultEncoding;
 	    NSFileOwnerAccountName, NSUserName(), nil];
 	  if (![self changeFileAttributes: attributes atPath: path])
 	    {
-	      NSLog(@"Failed to change ownership of '%@' to '%@'",
+	      NSDebugLog(@"Failed to change ownership of '%@' to '%@'",
 		path, NSUserName());
 	    }
 	}
@@ -977,7 +1052,7 @@ static NSStringEncoding	defaultEncoding;
 {
   NSString *currentDir = nil;
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
   int len = GetCurrentDirectoryW(0, 0);
   if (len > 0)
     {
@@ -991,8 +1066,10 @@ static NSStringEncoding	defaultEncoding;
 
 	      // Windows may count the trailing nul ... we don't want to.
 	      if (len > 0 && lpath[len] == 0) len--;
-	      path = [NSString stringWithCharacters: lpath length: len];
-	      currentDir = path;
+	      path = [[NSString alloc] initWithCharacters: lpath length: len];
+	      // Standardise to get rid of backslashes
+	      currentDir = [path stringByStandardizingPath];
+	      RELEASE(path);
 	    }
 	  free(lpath);
 	}
@@ -1008,7 +1085,7 @@ static NSStringEncoding	defaultEncoding;
 #endif /* HAVE_GETCWD */
   currentDir = [self stringWithFileSystemRepresentation: path
 						 length: strlen(path)];
-#endif /* !MINGW */
+#endif /* !_WIN32 */
 
   return currentDir;
 }
@@ -1387,7 +1464,7 @@ static NSStringEncoding	defaultEncoding;
     }
   else
     {
-#if defined(__MINGW__)
+#if defined(_WIN32)
       DWORD res;
 
       res = GetFileAttributesW(lpath);
@@ -1412,12 +1489,12 @@ static NSStringEncoding	defaultEncoding;
 	  return NO;
 	}
       is_dir = ((statbuf.st_mode & S_IFMT) == S_IFDIR);
-#endif /* MINGW */
+#endif /* _WIN32 */
     }
 
   if (!is_dir)
     {
-#if defined(__MINGW__)
+#if defined(_WIN32)
       if (DeleteFileW(lpath) == FALSE)
 #else
       if (unlink(lpath) < 0)
@@ -1517,7 +1594,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
     {
       DWORD res;
 
@@ -1555,7 +1632,7 @@ static NSStringEncoding	defaultEncoding;
 
       return YES;
     }
-#endif /* MINGW */
+#endif /* _WIN32 */
 }
 
 /**
@@ -1572,7 +1649,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
     {
       DWORD res;
 
@@ -1609,7 +1686,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
     {
       DWORD res;
 
@@ -1651,7 +1728,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
     {
       DWORD res;
       NSString  *ext;
@@ -1710,7 +1787,7 @@ static NSStringEncoding	defaultEncoding;
       return NO;
     }
 
-#if defined(__MINGW__)
+#if defined(_WIN32)
       // TODO - handle directories
     {
       DWORD res;
@@ -1934,7 +2011,7 @@ static NSStringEncoding	defaultEncoding;
  */
 - (NSDictionary*) fileSystemAttributesAtPath: (NSString*)path
 {
-#if defined(__MINGW__)
+#if defined(_WIN32)
   unsigned long long totalsize, freesize;
   id  values[5];
   id	keys[5] = {
@@ -2037,11 +2114,11 @@ static NSStringEncoding	defaultEncoding;
 
   return [NSDictionary dictionaryWithObjects: values forKeys: keys count: 5];
 #else
-  NSLog(@"NSFileManager", @"no support for filesystem attributes");
+  GSOnceMLog(@"NSFileManager", @"no support for filesystem attributes");
   ASSIGN(_lastError, @"no support for filesystem attributes");
   return nil;
 #endif
-#endif /* MINGW */
+#endif /* _WIN32 */
 }
 
 /**
@@ -2088,7 +2165,7 @@ static NSStringEncoding	defaultEncoding;
 	}
       RELEASE(direnum);
     }
-  return [content makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(content);
 }
 
 /**
@@ -2151,7 +2228,7 @@ static NSStringEncoding	defaultEncoding;
 	}
       RELEASE(direnum);
     }
-  return [content makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(content);
 }
 
 /**
@@ -2197,7 +2274,7 @@ static NSStringEncoding	defaultEncoding;
 #endif
 }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 - (const GSNativeChar*) fileSystemRepresentationWithPath: (NSString*)path
 {
   if (path != nil && [path rangeOfString: @"/"].length > 0)
@@ -2321,7 +2398,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
         }
       else
         {
-          NSLog(@"Failed to recurse into directory '%@' - %@", path,
+          NSDebugLog(@"Failed to recurse into directory '%@' - %@", path,
             [NSError _last]);
         }
     }
@@ -2409,7 +2486,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 
       if (dirbuf)
 	{
-#if defined(__MINGW__)
+#if defined(_WIN32)
 	  /* Skip "." and ".." directory entries */
 	  if (wcscmp(dirbuf->d_name, L".") == 0
 	    || wcscmp(dirbuf->d_name, L"..") == 0)
@@ -2444,7 +2521,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	    {
 	      // Do not follow links
 #ifdef S_IFLNK
-#ifdef __MINGW__
+#ifdef _WIN32
 #warning "lstat does not support unichars"
 #else
 	      if (!_flags.isFollowing)
@@ -2472,7 +2549,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 		}
 	      if (S_IFDIR == (S_IFMT & statbuf.st_mode))
 		{
-		  _DIR*  dir_pointer;
+		  _DIR  *dir_pointer;
 
 		  dir_pointer
 		    = _OPENDIR([_mgr fileSystemRepresentationWithPath:
@@ -2488,7 +2565,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 		    }
 		  else
 		    {
-		      NSLog(@"Failed to recurse into directory '%@' - %@",
+		      NSDebugLog(@"Failed to recurse into directory '%@' - %@",
 			_currentFilePath, [NSError _last]);
 		    }
 		}
@@ -2679,7 +2756,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	    toFile: (NSString*)destination
 	   handler: (id)handler
 {
-#if defined(__MINGW__)
+#if defined(_WIN32)
   if (CopyFileW([self fileSystemRepresentationWithPath: source],
     [self fileSystemRepresentationWithPath: destination], NO))
     {
@@ -2804,10 +2881,18 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 
       if ([fileType isEqual: NSFileTypeDirectory])
 	{
-	  BOOL	dirOK;
+          NSMutableDictionary   *newAttributes;
+	  BOOL	                dirOK;
 
+          newAttributes = [attributes mutableCopy];
+          [newAttributes removeObjectForKey: NSFileOwnerAccountID];
+          [newAttributes removeObjectForKey: NSFileGroupOwnerAccountID];
+          [newAttributes removeObjectForKey: NSFileGroupOwnerAccountName];
+          [newAttributes setObject: NSUserName()
+                            forKey: NSFileOwnerAccountName];
 	  dirOK = [self createDirectoryAtPath: destinationFile
-				   attributes: attributes];
+				   attributes: newAttributes];
+          RELEASE(newAttributes);
 	  if (dirOK == NO)
 	    {
               if (![self _proceedAccordingToHandler: handler
@@ -2832,8 +2917,8 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	    {
 	      [enumerator skipDescendents];
 	      if (![self _copyPath: sourceFile
-                         toPath: destinationFile
-                         handler: handler])
+                            toPath: destinationFile
+                           handler: handler])
                 {
                   RELEASE(pool);
                   return NO;
@@ -2876,7 +2961,7 @@ static inline void gsedRelease(GSEnumeratedDirectory X)
 	  s = [NSString stringWithFormat: @"cannot copy file type '%@'",
 	    fileType];
 	  ASSIGN(_lastError, s);
-	  NSLog(@"%@: %@", sourceFile, s);
+	  NSDebugLog(@"%@: %@", sourceFile, s);
 	  continue;
 	}
       [self changeFileAttributes: attributes atPath: destinationFile];
@@ -3105,7 +3190,7 @@ static NSSet	*fileKeys = nil;
   d = (GSAttrDictionary*)NSAllocateObject(self, (l+1)*sizeof(_CHAR),
     NSDefaultMallocZone());
 
-#if defined(S_IFLNK) && !defined(__MINGW__)
+#if defined(S_IFLNK) && !defined(_WIN32)
   if (traverse == NO)
     {
       if (lstat(lpath, &d->statbuf) != 0)
@@ -3188,7 +3273,7 @@ static NSSet	*fileKeys = nil;
 {
   NSString	*group = @"UnknownGroup";
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   DWORD		returnCode = 0;
   PSID		sidOwner;
   int		result = TRUE;
@@ -3345,7 +3430,7 @@ static NSSet	*fileKeys = nil;
 {
   NSString	*owner = @"UnknownUser";
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   DWORD		returnCode = 0;
   PSID		sidOwner;
   int		result = TRUE;
@@ -3475,7 +3560,7 @@ static NSSet	*fileKeys = nil;
 
 - (NSUInteger) fileSystemNumber
 {
-#if defined(__MINGW__)
+#if defined(_WIN32)
   DWORD volumeSerialNumber = 0;
   _CHAR volumePathName[128];
   if (GetVolumePathNameW(_path,volumePathName,128))
@@ -3571,7 +3656,7 @@ static NSSet	*fileKeys = nil;
     }
   if (count >= 2)
     {
-      NSLog(@"Warning ... key '%@' not handled", key);
+      NSDebugLog(@"Warning ... key '%@' not handled", key);
     }
   return nil;
 }
