@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2014-01-26 13:50:26 +0000 (Sun, 26 Jan 2014) $ $Revision: 37637 $
+   $Date: 2015-10-08 10:13:32 +0100 (Thu, 08 Oct 2015) $ $Revision: 39042 $
 */
 
 #import "common.h"
@@ -573,6 +573,12 @@ cifframe_callback(ffi_cif *cif, void *retp, void **args, void *user)
                 imp = [[GSKVOSetter class]
                   instanceMethodForSelector: @selector(setterDouble:)];
                 break;
+#if __GNUC__ > 2 && defined(_C_BOOL)
+              case _C_BOOL:
+                imp = [[GSKVOSetter class]
+                  instanceMethodForSelector: @selector(setterChar:)];
+                break;
+#endif
               case _C_ID:
               case _C_CLASS:
               case _C_PTR:
