@@ -752,12 +752,12 @@ static NSDictionary *makeReference(unsigned ref)
 	return;
 
       case _C_INT:
-	o = [NSNumber numberWithInt: *(NSInteger*)address];
+	o = [NSNumber numberWithInt: *(int*)address];
 	[_enc setObject: o forKey: aKey];
 	return;
 
       case _C_UINT:
-	o = [NSNumber numberWithUnsignedInt: *(NSUInteger*)address];
+	o = [NSNumber numberWithUnsignedInt: *(unsigned int*)address];
 	[_enc setObject: o forKey: aKey];
 	return;
 
@@ -792,6 +792,13 @@ static NSDictionary *makeReference(unsigned ref)
 	[_enc setObject: o forKey: aKey];
 	return;
 
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:
+	o = [NSNumber numberWithInt: (NSInteger)*(_Bool*)address];
+	[_enc setObject: o forKey: aKey];
+	return;
+#endif
+
       case _C_STRUCT_B:
 	[NSException raise: NSInvalidArgumentException
 		    format: @"-[%@ %@]: this archiver cannote encode structs",
