@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSLog reference</title>
-   $Date: 2014-07-28 15:40:29 +0100 (Mon, 28 Jul 2014) $ $Revision: 38018 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
    */
 
 #import "common.h"
@@ -37,6 +37,7 @@
 #import "Foundation/NSThread.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 
+// Some older BSD systems used a non-standard range of thread priorities.
 #ifdef	HAVE_SYSLOG_H
 #include <syslog.h>
 #elif HAVE_SYS_SLOG_H
@@ -86,6 +87,8 @@ extern NSThread	*GSCurrentThread();
 int _NSLogDescriptor = 2;
 
 static NSRecursiveLock	*myLock = nil;
+static IMP              lockImp = 0;
+static IMP              unlockImp = 0;
 
 /**
  * Returns the lock used to protect the GNUstep NSLogv() implementation.
@@ -102,6 +105,8 @@ GSLogLock()
       if (myLock == nil)
 	{
 	  myLock = [NSRecursiveLock new];
+          lockImp = [myLock methodForSelector: @selector(lock)];
+          unlockImp = [myLock methodForSelector: @selector(unlock)];
 	}
       [gnustep_global_lock unlock];
     }
@@ -114,7 +119,7 @@ _NSLog_standard_printf_handler(NSString* message)
   NSData	*d;
   const char	*buf;
   unsigned	len;
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   LPCWSTR	null_terminated_buf;
 #else
 #if	defined(HAVE_SYSLOG) || defined(HAVE_SLOGF)
@@ -145,7 +150,7 @@ _NSLog_standard_printf_handler(NSString* message)
       len = [d length];
     }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   null_terminated_buf = UNISTR(message);
 
   OutputDebugStringW(null_terminated_buf);
@@ -224,7 +229,7 @@ _NSLog_standard_printf_handler(NSString* message)
 #else
   write(_NSLogDescriptor, buf, len);
 #endif
-#endif // __MINGW__
+#endif // _WIN32
 }
 
 /**
@@ -302,8 +307,8 @@ NSLog(NSString* format, ...)
  * </p>
  * <p>
  *   In GNUstep, the GSLogThread user default may be set to YES in
- *   order to instruct this function to include the internal ID of
- *   the current thread after the process ID.  This can help you
+ *   order to instruct this function to include the name (if any)
+ *   of the current thread after the process ID.  This can help you
  *   to track the behavior of a multi-threaded program.<br />
  *   Also the GSLogOffset user default may be set to YES in order
  *   to instruct this function to include the time zone offset in
@@ -326,10 +331,11 @@ NSLog(NSString* format, ...)
 void
 NSLogv(NSString* format, va_list args)
 {
-  NSString		*prefix;
-  NSString		*message;
+  NSMutableString	*prefix;
+  NSString              *message;
+  NSString              *threadName = nil;
+  NSThread              *t = nil;
   static int		pid = 0;
-  NSAutoreleasePool	*arp = [NSAutoreleasePool new];
 
   if (_NSLog_printf_handler == NULL)
     {
@@ -338,30 +344,48 @@ NSLogv(NSString* format, va_list args)
 
   if (pid == 0)
     {
-#if defined(__MINGW__)
+#if defined(_WIN32)
       pid = (int)GetCurrentProcessId();
 #else
       pid = (int)getpid();
 #endif
     }
 
+  if (GSPrivateDefaultsFlag(GSLogThread) == YES)
+    {
+      /* If no name has been set for the current thread,
+       * we log the address of the NSThread object instead.
+       */
+      t = GSCurrentThread();
+      threadName = [t name];
+    }
+
+  prefix = [[NSMutableString alloc] initWithCapacity: 1000];
+
 #ifdef	HAVE_SYSLOG
   if (GSPrivateDefaultsFlag(GSLogSyslog) == YES)
     {
-      if (GSPrivateDefaultsFlag(GSLogThread) == YES)
-	{
-	  prefix = [NSString stringWithFormat: @"[thread:%"PRIxPTR"] ",
-	    (NSUInteger)GSCurrentThread()];
-	}
+      if (nil == t)
+        {
+          [prefix appendFormat: @"[thread:%"PRIuPTR"] ",
+            GSPrivateThreadID()];
+        }
+      else if (nil == threadName)
+        {
+          [prefix appendFormat: @"[thread:%"PRIuPTR",%p] ",
+            GSPrivateThreadID(), t];
+        }
       else
-	{
-	  prefix = @"";
-	}
+        {
+          [prefix appendFormat: @"[thread:%"PRIuPTR",%@] ",
+            GSPrivateThreadID(), threadName];
+        }
     }
   else
 #endif
     {
       NSString  *fmt;
+      NSString  *cal;
 
       if (GSPrivateDefaultsFlag(GSLogOffset) == YES)
         {
@@ -371,45 +395,47 @@ NSLogv(NSString* format, va_list args)
         {
           fmt = @"%Y-%m-%d %H:%M:%S.%F";
         }
+      cal = [[NSCalendarDate calendarDate] descriptionWithCalendarFormat: fmt];
 
-      if (GSPrivateDefaultsFlag(GSLogThread) == YES)
-	{
-	  prefix = [NSString
-	    stringWithFormat: @"%@ %@[%d,%"PRIxPTR"x] ",
-	    [[NSCalendarDate calendarDate] descriptionWithCalendarFormat: fmt],
-	    [[NSProcessInfo processInfo] processName],
-	    pid, (NSUInteger)GSCurrentThread()];
-	}
+      [prefix appendString: cal];
+      [prefix appendString: @" "];
+      [prefix appendString: [[NSProcessInfo processInfo] processName]];
+      if (nil == t)
+        {
+          [prefix appendFormat: @"[%d:%"PRIuPTR"] ",
+            pid, GSPrivateThreadID()];
+        }
+      else if (nil == threadName)
+        {
+          [prefix appendFormat: @"[%d:%"PRIuPTR",%p] ",
+            pid, GSPrivateThreadID(), t];
+        }
       else
-	{
-	  prefix = [NSString
-	    stringWithFormat: @"%@ %@[%d] ",
-	    [[NSCalendarDate calendarDate] descriptionWithCalendarFormat: fmt],
-	    [[NSProcessInfo processInfo] processName],
-	    pid];
-	}
+        {
+          [prefix appendFormat: @"[%d:%"PRIuPTR",%@] ",
+            pid, GSPrivateThreadID(), threadName];
+        }
     }
 
-  /* Check if there is already a newline at the end of the format */
-  if ([format hasSuffix: @"\n"] == NO)
+  message = [[NSString alloc] initWithFormat: format arguments: args];
+  [prefix appendString: message];
+  [message release];
+  if ([prefix hasSuffix: @"\n"] ==  NO)
     {
-      format = [format stringByAppendingString: @"\n"];
+      [prefix appendString: @"\n"];
     }
-  message = [NSString stringWithFormat: format arguments: args];
 
-  prefix = [prefix stringByAppendingString: message];
-
-  if (myLock == nil)
+  if (nil == myLock)
     {
       GSLogLock();
     }
 
-  [myLock lock];
+  (*lockImp)(myLock, @selector(lock));
 
   _NSLog_printf_handler(prefix);
 
-  [myLock unlock];
+  (*unlockImp)(myLock, @selector(unlock));
 
-  [arp drain];
+  [prefix release];
 }
 
