@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSObject class reference</title>
-   $Date: 2014-07-13 08:31:33 +0100 (Sun, 13 Jul 2014) $ $Revision: 37990 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
    */
 
 /* On some versions of mingw we need to work around bad function declarations
@@ -41,6 +41,7 @@
 #import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSArray.h"
 #import "Foundation/NSException.h"
+#import "Foundation/NSHashTable.h"
 #import "Foundation/NSPortCoder.h"
 #import "Foundation/NSDistantObject.h"
 #import "Foundation/NSThread.h"
@@ -200,7 +201,7 @@ static void GSLogZombie(id o, SEL sel)
 #endif
 
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 #ifndef _WIN64
 #undef InterlockedIncrement
 #undef InterlockedDecrement
@@ -1010,13 +1011,13 @@ static id gs_weak_load(id obj)
       GC_set_warn_proc(GSGarbageCollectorLog);
 #endif
 
-#ifdef __MINGW__
+#ifdef _WIN32
       {
         // See libgnustep-base-entry.m
         extern void gnustep_base_socket_init(void);
         gnustep_base_socket_init();
       }
-#else /* __MINGW__ */
+#else /* _WIN32 */
 
 #ifdef	SIGPIPE
     /*
@@ -1058,7 +1059,7 @@ static id gs_weak_load(id obj)
       }
 #endif /* HAVE_SIGACTION */
 #endif /* SIGPIPE */
-#endif /* __MINGW__ */
+#endif /* _WIN32 */
 
       finalize_sel = @selector(finalize);
       finalize_imp = class_getMethodImplementation(self, finalize_sel);
@@ -1137,11 +1138,6 @@ static id gs_weak_load(id obj)
 	   selector: @selector(_becomeMultiThreaded:)
 	       name: NSWillBecomeMultiThreadedNotification
 	     object: nil];
-
-      /* It is also safer to set upo the user defaults system before doing
-       * other stuff which might use it.
-       */
-      [NSUserDefaults class];
     }
   return;
 }
@@ -1902,13 +1898,16 @@ static id gs_weak_load(id obj)
 - (NSUInteger) hash
 {
   /*
-   * Ideally we would shift left to lose any zero bits produced by the
-   * alignment of the object in memory ... but that depends on the
-   * processor architecture and the memory allocatiion implementation.
-   * In the absence of detailed information, pick a reasonable value
-   * assuming the object will be aligned to an eight byte boundary.
+   *  malloc() must return pointers aligned to point to any data type
    */
-  return (NSUInteger)(uintptr_t)self >> 3;
+#define MAXALIGN (__alignof__(_Complex long double))
+
+  static int shift = MAXALIGN==16 ? 4 : (MAXALIGN==8 ? 3 : 2);
+
+  /* We shift left to lose any zero bits produced by the
+   * alignment of the object in memory.
+   */
+  return (NSUInteger)((uintptr_t)self >> shift);
 }
 
 /**
@@ -2005,8 +2004,8 @@ static id gs_weak_load(id obj)
   if (!msg)
     {
       [NSException raise: NSGenericException
-		   format: @"invalid selector passed to %s",
-		     sel_getName(_cmd)];
+		   format: @"invalid selector '%s' passed to %s",
+		     sel_getName(aSelector), sel_getName(_cmd)];
       return nil;
     }
   return (*msg)(self, aSelector);
@@ -2018,7 +2017,7 @@ static id gs_weak_load(id obj)
  * The method must be one which takes one argument and returns an object.
  * <br />Raises NSInvalidArgumentException if given a null selector.
  */
-- (id) performSelector: (SEL)aSelector withObject: (id) anObject
+- (id) performSelector: (SEL)aSelector withObject: (id)anObject
 {
   IMP msg;
 
@@ -2029,7 +2028,7 @@ static id gs_weak_load(id obj)
   /* The Apple runtime API would do:
    * msg = class_getMethodImplementation(object_getClass(self), aSelector);
    * but this cannot ask self for information about any method reached by
-   * forwarding, so the returned forwarding function would ge a generic one
+   * forwarding, so the returned forwarding function would be a generic one
    * rather than one aware of hardware issues with returning structures
    * and floating points.  We therefore prefer the GNU API which is able to
    * use forwarding callbacks to get better type information.
@@ -2038,8 +2037,8 @@ static id gs_weak_load(id obj)
   if (!msg)
     {
       [NSException raise: NSGenericException
-		   format: @"invalid selector passed to %s",
-		   sel_getName(_cmd)];
+		   format: @"invalid selector '%s' passed to %s",
+                   sel_getName(aSelector), sel_getName(_cmd)];
       return nil;
     }
 
@@ -2074,7 +2073,8 @@ static id gs_weak_load(id obj)
   if (!msg)
     {
       [NSException raise: NSGenericException
-		  format: @"invalid selector passed to %s", sel_getName(_cmd)];
+		   format: @"invalid selector '%s' passed to %s",
+                   sel_getName(aSelector), sel_getName(_cmd)];
       return nil;
     }
 
@@ -2604,3 +2604,24 @@ static id gs_weak_load(id obj)
 }
 @end
 
+NSUInteger
+GSPrivateMemorySize(NSObject *self, NSHashTable *exclude)
+{
+  if (0 == NSHashGet(exclude, self))
+    {
+      NSHashInsert(exclude, self);
+      return class_getInstanceSize(object_getClass(self));
+    }
+  return 0;
+}
+
+@implementation	NSObject (MemoryFootprint)
++ (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  return 0;
+}
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  return GSPrivateMemorySize(self, exclude);
+}
+@end
