@@ -30,7 +30,7 @@
    Boston, MA 02111 USA.
 
    <title>NSPortCoder class reference</title>
-   $Date: 2013-07-02 16:46:26 +0100 (Tue, 02 Jul 2013) $ $Revision: 36799 $
+   $Date: 2015-10-08 10:13:32 +0100 (Thu, 08 Oct 2015) $ $Revision: 39042 $
    */
 
 #import "common.h"
@@ -129,6 +129,9 @@ typeToName1(char type)
       case _C_CHARPTR:	return "cstring";
       case _C_ARY_B:	return "array";
       case _C_STRUCT_B:	return "struct";
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:	return "_Bool";
+#endif
       default:
 	{
 	  static char	buf1[32];
@@ -170,6 +173,7 @@ typeToName2(char type)
       case _GSC_ULNG_LNG:	return "unsigned long long";
       case _GSC_FLT:	return "float";
       case _GSC_DBL:	return "double";
+      case _GSC_BOOL:	return "_Bool";
       case _GSC_PTR:	return "pointer";
       case _GSC_CHARPTR:	return "cstring";
       case _GSC_ARY_B:	return "array";
@@ -218,7 +222,11 @@ static char	type_map[32] = {
 #endif
   _C_FLT,
   _C_DBL,
+#if __GNUC__ > 2 && defined(_C_BOOL)
+  _C_BOOL,
+#else
   0,
+#endif
   0,
   0,
   _C_ID,
@@ -515,6 +523,9 @@ static unsigned	encodingVersion;
 #endif
       case _C_FLT:	info = _GSC_FLT; break;
       case _C_DBL:	info = _GSC_DBL; break;
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:	info = _GSC_BOOL; break;
+#endif
       default:		info = _GSC_NONE; break;
     }
 
@@ -1028,6 +1039,17 @@ static unsigned	encodingVersion;
 	  }
 	return;
 
+#if __GNUC__ != 2
+      case _GSC_BOOL:
+	if (*type != type_map[_GSC_BOOL])
+	  {
+	    [NSException raise: NSInternalInconsistencyException
+		        format: @"expected %s and got %s",
+		    typeToName1(*type), typeToName2(info)];
+	  }
+	(*_dDesImp)(_src, dDesSel, address, type, &_cursor, nil);
+	return;
+#endif
       default:
 	[NSException raise: NSInternalInconsistencyException
 		    format: @"read unknown type info - %d", info];
@@ -1200,6 +1222,9 @@ static unsigned	encodingVersion;
       case _C_ULNG_LNG:	info = _GSC_ULNG_LNG | _GSC_S_LNG_LNG;	break;
       case _C_FLT:	info = _GSC_FLT;	break;
       case _C_DBL:	info = _GSC_DBL;	break;
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:	info = _GSC_BOOL;	break;
+#endif
       default:		info = _GSC_NONE;	break;
     }
 
@@ -1798,6 +1823,13 @@ static unsigned	encodingVersion;
 	(*_eSerImp)(_dst, eSerSel, (void*)buf, @encode(double), nil);
 	return;
 
+#if __GNUC__ > 2 && defined(_C_BOOL)
+      case _C_BOOL:
+	(*_eTagImp)(_dst, eTagSel, _GSC_BOOL);
+	(*_eSerImp)(_dst, eSerSel, (void*)buf, @encode(_Bool), nil);
+	return;
+#endif
+
       case _C_VOID:
 	[NSException raise: NSInvalidArgumentException
 		    format: @"can't encode void item"];
