@@ -141,7 +141,24 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
 }
 @end
 
-
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+@interface GSBlockPredicate : NSPredicate
+{
+  GSBlockPredicateBlock _block;
+}
+
+- (instancetype) initWithBlock: (GSBlockPredicateBlock)block;
+@end
+
+
+@interface GSBoundBlockPredicate : GSBlockPredicate
+{
+  GS_GENERIC_CLASS(NSDictionary,NSString*,id)* _bindings;
+}
+- (instancetype) initWithBlock: (GSBlockPredicateBlock)block
+                      bindings: (GS_GENERIC_CLASS(NSDictionary,NSString*,id)*)bindings;
+@end
+#endif
 
 @implementation NSPredicate
 
@@ -213,7 +230,7 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
               case 'i':
                 ptr++;
                 [arr addObject: [NSNumber numberWithInt:
-                  va_arg(args, NSInteger)]];
+                  va_arg(args, int)]];
                 break;
 
               case 'o':
@@ -224,7 +241,7 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
               case 'X':
                 ptr++;
                 [arr addObject: [NSNumber numberWithUnsignedInt:
-                  va_arg(args, NSUInteger)]];
+                  va_arg(args, unsigned)]];
                 break;
 
               case 'e':
@@ -343,6 +360,14 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
   return AUTORELEASE([self copy]);  
 }
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_5, GS_API_LATEST)
+- (BOOL) evaluateWithObject: (id)object
+	  substitutionVariables: (GS_GENERIC_CLASS(NSDictionary, NSString*, id)*)variables
+{
+  return [[self predicateWithSubstitutionVariables: variables]
+		   evaluateWithObject: object];
+}
+#endif
 - (Class) classForCoder
 {
   return [NSPredicate class];
@@ -361,6 +386,12 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
   return self;
 }
 
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
++ (NSPredicate*)predicateWithBlock: (GSBlockPredicateBlock)block
+{
+  return [[[GSBlockPredicate alloc] initWithBlock: block] autorelease];
+}
+#endif
 @end
 
 @implementation GSTruePredicate
@@ -461,6 +492,7 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
   unsigned int count = [_subs count];
   NSMutableArray *esubs = [NSMutableArray arrayWithCapacity: count];
   unsigned int i;
+  NSPredicate  *p;
 
   for (i = 0; i < count; i++)
     {
@@ -468,7 +500,8 @@ extern void     GSPropertyListMake(id,NSDictionary*,BOOL,BOOL,unsigned,id*);
                             predicateWithSubstitutionVariables: variables]];
     }
 
-  return [[[self class] alloc] initWithType: _type subpredicates: esubs];
+  p = [[[self class] alloc] initWithType: _type subpredicates: esubs];
+  return AUTORELEASE(p);
 }
 
 - (Class) classForCoder
@@ -1692,7 +1725,7 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
           [result addObject: object];  // passes filter
         }
     }
-  return [result makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(result);
 }
 
 @end
@@ -1732,7 +1765,7 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
           [result addObject: object];  // passes filter
         }
     }
-  return [result makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(result);
 }
 
 @end
@@ -2589,5 +2622,82 @@ GSICUStringMatchesRegex(NSString *string, NSString *regex, NSStringCompareOption
         }
     }
 }
-
 @end
+
+
+#if OS_API_VERSION(MAC_OS_X_VERSION_10_6, GS_API_LATEST)
+
+
+@implementation GSBlockPredicate
+
+- (instancetype) initWithBlock: (GSBlockPredicateBlock)block
+{
+  if (nil == (self = [super init]))
+    {
+      return nil;
+    }
+  _block = (GSBlockPredicateBlock)[(id)block retain];
+  return self;
+}
+
+- (instancetype) predicateWithSubstitutionVariables: 
+  (GS_GENERIC_CLASS(NSDictionary,NSString*,id)*)variables
+{
+  return [[[GSBoundBlockPredicate alloc] initWithBlock: _block
+                                              bindings: variables] autorelease];
+}
+
+- (BOOL) evaluateWithObject: (id)object
+      substitutionVariables: (GS_GENERIC_CLASS(NSDictionary,
+                                               NSString*,id)*)variables
+{
+  return CALL_BLOCK(_block, object, variables);
+}
+
+- (BOOL) evaluateWithObject: (id)object
+{
+  return [self evaluateWithObject: object
+            substitutionVariables: nil];
+}
+
+- (void) dealloc
+{
+  [(id)_block release];
+  _block = NULL;
+  [super dealloc];
+}
+
+- (NSString*) predicateFormat
+{
+  return [NSString stringWithFormat: @"BLOCKPREDICATE(%p)", (void*)_block];
+}
+@end
+
+@implementation GSBoundBlockPredicate
+
+- (instancetype) initWithBlock: (GSBlockPredicateBlock)block
+                      bindings: (GS_GENERIC_CLASS(NSDictionary,
+                                                   NSString*,id)*)bindings
+{
+  if (nil == (self = [super initWithBlock: block]))
+    {
+      return nil;
+    }
+  ASSIGN(_bindings, bindings);
+  return self;
+}
+
+- (BOOL) evaluateWithObject: (id)object
+{
+  return [self evaluateWithObject: object
+            substitutionVariables: _bindings];
+}
+
+- (void) dealloc
+{
+  DESTROY(_bindings);
+  [super dealloc];
+}
+@end
+
+#endif
