@@ -26,7 +26,7 @@
    Boston, MA 02111 USA.
 
    <title>NSRunLoop class reference</title>
-   $Date: 2014-07-14 09:24:30 +0100 (Mon, 14 Jul 2014) $ $Revision: 37995 $
+   $Date: 2015-10-03 19:56:08 +0100 (Sat, 03 Oct 2015) $ $Revision: 39024 $
 */
 
 #import "common.h"
@@ -112,10 +112,12 @@ static NSDate	*theFuture = nil;
   NS_HANDLER
     {
       NSLog(@"*** NSRunLoop ignoring exception '%@' (reason '%@') "
-        @"raised during performSelector... with target %p "
-        @"and selector '%@'",
-        [localException name], [localException reason], target,
-        NSStringFromSelector([target selector]));
+        @"raised during performSelector... with target %s(%s) "
+        @"and selector '%s'",
+        [localException name], [localException reason],
+        GSClassNameFromObject(target),
+        GSObjCIsInstance(target) ? "instance" : "class",
+        sel_getName(selector));
     }
   NS_ENDHANDLER
 }
@@ -443,27 +445,30 @@ static inline BOOL timerInvalidated(NSTimer *t)
           NSAutoreleasePool	*arp = [NSAutoreleasePool new];
 	  GSRunLoopPerformer	*array[count];
 	  NSMapEnumerator	enumerator;
-	  GSRunLoopCtxt		*context;
+	  GSRunLoopCtxt		*original;
 	  void			*mode;
 	  unsigned		i;
 
           found = YES;
-	  /*
-	   * Copy the array - because we have to cancel the requests
-	   * before firing.
+
+	  /* We have to remove the performers before firing, so we copy
+	   * the pointers without releasing the objects, and then set
+	   * the performers to be empty.  The copied objects in 'array'
+	   * will be released later.
 	   */
 	  for (i = 0; i < count; i++)
 	    {
-	      array[i] = RETAIN(GSIArrayItemAtIndex(performers, i).obj);
+	      array[i] = GSIArrayItemAtIndex(performers, i).obj;
 	    }
+          performers->count = 0;
 
-	  /*
-	   * Remove the requests that we are about to fire from all modes.
+	  /* Remove the requests that we are about to fire from all modes.
 	   */
+          original = context;
 	  enumerator = NSEnumerateMapTable(_contextMap);
 	  while (NSNextMapEnumeratorPair(&enumerator, &mode, (void**)&context))
 	    {
-	      if (context != nil)
+	      if (context != nil && context != original)
 		{
 		  GSIArray	performers = context->performers;
 		  unsigned	tmpCount = GSIArrayCount(performers);
@@ -485,8 +490,7 @@ static inline BOOL timerInvalidated(NSTimer *t)
 	    }
 	  NSEndMapTableEnumeration(&enumerator);
 
-	  /*
-	   * Finally, fire the requests.
+	  /* Finally, fire the requests ands release them.
 	   */
 	  for (i = 0; i < count; i++)
 	    {
