@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSScanner class reference</title>
-   $Date: 2014-01-20 09:29:17 +0000 (Mon, 20 Jan 2014) $ $Revision: 37621 $
+   $Date: 2015-06-26 09:52:23 +0100 (Fri, 26 Jun 2015) $ $Revision: 38693 $
 */
 
 #import "common.h"
@@ -1301,14 +1301,7 @@ GSScanDouble(unichar *buf, unsigned length, double *result)
     {
       /* Mantissa too long ... ignore excess.
        */
-      if (19 == mantissaLength && (dotPos < 0 || dotPos >= mantissaLength))
-        {
-          mantissaLength--;
-        }
-      else
-        {
-          mantissaLength = 19;
-        }
+      mantissaLength = 18;
     }
   if (dotPos < 0)
     {
