@@ -22,7 +22,7 @@
    Boston, MA 02111 USA.
 
    <title>NSSerializer class reference</title>
-   $Date: 2013-08-22 16:44:54 +0100 (Thu, 22 Aug 2013) $ $Revision: 37003 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
    */
 
 #import "common.h"
@@ -648,7 +648,7 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 		}
 	      if (code != ST_MARRAY && info->mutable == NO)
 		{
-		  [a makeImmutableCopyOnFail: NO];
+                  a = GS_IMMUTABLE(a);
 		}
 	    }
 	  return a;
@@ -701,7 +701,7 @@ deserializeFromInfo(_NSDeserializerInfo* info)
 		}
 	      if (code != ST_MDICT && info->mutable == NO)
 		{
-		  [d makeImmutableCopyOnFail: NO];
+                  d = GS_IMMUTABLE(d);
 		}
 	    }
 	  return d;
