@@ -22,11 +22,12 @@
    Boston, MA 02111 USA.
 
    <title>NSSet class reference</title>
-   $Date: 2014-01-20 17:45:23 +0000 (Mon, 20 Jan 2014) $ $Revision: 37624 $
+   $Date: 2016-03-10 16:50:41 +0000 (Thu, 10 Mar 2016) $ $Revision: 39520 $
    */
 
 #import "common.h"
 #import "Foundation/NSArray.h"
+#import "Foundation/NSAutoreleasePool.h"
 #import "Foundation/NSSet.h"
 #import "Foundation/NSCoder.h"
 #import "Foundation/NSArray.h"
@@ -590,6 +591,11 @@ static Class NSMutableSet_concrete_class;
   return NO;
 }
 
+- (NSUInteger)_countForObject: (id)object
+{
+  return 1;
+}
+
 /**
  *  Return whether each set is subset of the other.
  */
@@ -602,8 +608,19 @@ static Class NSMutableSet_concrete_class;
       id	o, e = [self objectEnumerator];
 
       while ((o = [e nextObject]))
-	if (![other member: o])
-	  return NO;
+        {
+	  if (![other member: o])
+            {
+	      return NO;
+            }
+         else
+           {
+             if ([self _countForObject: o] != [other _countForObject: o])
+               {
+                 return NO;
+               }
+           }
+        }
     }
   /* xxx Recheck this. */
   return YES;
@@ -656,7 +673,7 @@ static Class NSMutableSet_concrete_class;
         {
           if ([path isEqualToString: @"@count"] == YES)
             {
-              result = [NSNumber numberWithUnsignedInt: [self count]];
+              result = [NSNumber numberWithUnsignedInteger: [self count]];
             }
           else
             {
@@ -671,7 +688,7 @@ static Class NSMutableSet_concrete_class;
 
           if ([op isEqualToString: @"@count"] == YES)
             {
-              result = [NSNumber numberWithUnsignedInt: count];
+              result = [NSNumber numberWithUnsignedInteger: count];
             }
           else if ([op isEqualToString: @"@avg"] == YES)
             {
@@ -815,7 +832,7 @@ static Class NSMutableSet_concrete_class;
                       o = [o valueForKeyPath: rem];
                       [result addObjectsFromArray: o];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -835,7 +852,7 @@ static Class NSMutableSet_concrete_class;
                       o = [o valueForKeyPath: rem];
                       [result addObject: o];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -855,7 +872,7 @@ static Class NSMutableSet_concrete_class;
                       o = [o valueForKeyPath: rem];
                       [result addObjectsFromArray: [o allObjects]];
                     }
-                  [result makeImmutableCopyOnFail: NO];
+                  result = GS_IMMUTABLE(result);
                 }
               else
                 {
@@ -929,7 +946,7 @@ static Class NSMutableSet_concrete_class;
     }
   END_FOR_IN(enumerator)
     
-  return [resultSet makeImmutableCopyOnFail: NO];
+  return GS_IMMUTABLE(resultSet);
 }
 
 /** Return a set formed by adding anObject to the receiver.
@@ -981,6 +998,30 @@ static Class NSMutableSet_concrete_class;
     [self subclassResponsibility: _cmd];
     return 0;
 }
+
+- (NSUInteger) sizeInBytesExcluding: (NSHashTable*)exclude
+{
+  NSUInteger    size = [super sizeInBytesExcluding: exclude];
+
+  if (size > 0)
+    {
+      NSUInteger        count = [self count];
+
+      size += 3 * sizeof(void*) * count;
+      if (count > 0)
+        {
+          NSEnumerator          *enumerator = [self objectEnumerator];
+          NSObject              *o;
+
+          while ((o = [enumerator nextObject]) != nil)
+            {
+              size += [o sizeInBytesExcluding: exclude];
+            }
+        }
+    }
+  return size;
+}
+
 @end
 
 
