@@ -21,7 +21,7 @@
    Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02111 USA.
 
-   $Date: 2014-06-20 15:17:17 +0100 (Fri, 20 Jun 2014) $ $Revision: 37956 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
    */
 
 /* define to get system-v functions including inet_aton()
@@ -53,7 +53,7 @@
 
 #import "GSPortPrivate.h"
 
-#ifdef __MINGW__
+#ifdef _WIN32
 #include <winsock2.h>
 #include <wininet.h>
 #else
@@ -183,15 +183,17 @@ typedef enum {
   if (e != nil)
     {
       NSDebugMLLog(@"NSSocketPortNameServer",
-	@"failed connect to gdomap on %@ - %@",
-	[[notification object] socketAddress], e);
+	@"failed connect to gdomap on %@:%@ - %@",
+	[[notification object] socketAddress],
+	[[notification object] socketService],
+        e);
       /*
        * Remove our file handle, then either retry or fail.
        */
       [self close];
       if (launchCmd == nil)
 	{
-	  launchCmd = [NSTask launchPathForTool: @"gdomap"];
+	  launchCmd = RETAIN([NSTask launchPathForTool: @"gdomap"]);
 	}
       if (state == GSPC_LOPEN && launchCmd != nil)
 	{
@@ -251,8 +253,9 @@ typedef enum {
   if (d == nil || [d length] == 0)
     {
       [self fail];
-      NSLog(@"NSSocketPortNameServer lost connection to gdomap on %@",
-	[[notification object] socketAddress]);
+      NSLog(@"NSSocketPortNameServer lost connection to gdomap on %@:%@",
+	[[notification object] socketAddress],
+	[[notification object] socketService]);
     }
   else
     {
@@ -321,8 +324,10 @@ typedef enum {
   if (e != nil)
     {
       [self fail];
-      NSLog(@"NSSocketPortNameServer failed write to gdomap on %@ - %@",
-	[[notification object] socketAddress], e);
+      NSLog(@"NSSocketPortNameServer failed write to gdomap on %@:%@ - %@",
+	[[notification object] socketAddress],
+	[[notification object] socketService],
+        e);
     }
   else
     {
