@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSTimeZone class reference</title>
-   $Date: 2014-01-11 08:09:39 +0000 (Sat, 11 Jan 2014) $ $Revision: 37572 $
+   $Date: 2016-03-15 18:22:57 +0000 (Tue, 15 Mar 2016) $ $Revision: 39547 $
  */
 
 /* Use the system time zones if available. In other cases, use an
@@ -112,7 +112,6 @@
 #ifdef HAVE_TZHEAD
 #include <tzfile.h>
 #else
-#define NOID
 #include "nstzfile.h"
 #endif
 
@@ -223,6 +222,7 @@ static NSString *tzdir = nil;
  */
 static GSPlaceholderTimeZone	*defaultPlaceholderTimeZone;
 static NSMapTable		*placeholderMap;
+static GSAbsTimeZone            *commonAbsolutes[145] = { 0 };
 
 /*
  * Temporary structure for holding time zone details.
@@ -260,7 +260,7 @@ typedef struct {
 }
 @end
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 @interface	GSWindowsTimeZone : NSTimeZone
 {
 @public
@@ -510,7 +510,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 	          data = [NSData dataWithContentsOfFile: fileName];
 		}
 	      if (nil == data)
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
                 {
                   zone = [[GSWindowsTimeZone alloc] initWithName: name data: 0];
                   DESTROY(self);
@@ -522,7 +522,7 @@ static NSString *_time_zone_path(NSString *subpath, NSString *type)
 		}
 #endif
 	    }
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 	  if (!data)
 	    zone = [[GSWindowsTimeZone alloc] initWithName: name data: data];
 	  else
@@ -704,6 +704,17 @@ static NSMapTable	*absolutes = 0;
     }
   anOffset *= sign;
 
+  if (anOffset % 900 == 0)
+    {
+      z = commonAbsolutes[anOffset/900 + 72];
+      if (z != nil)
+        {
+          IF_NO_GC(RETAIN(z));
+          DESTROY(self);
+          return z;
+        }
+    }
+
   if (zone_mutex != nil)
     {
       [zone_mutex lock];
@@ -749,6 +760,15 @@ static NSMapTable	*absolutes = 0;
       NSMapInsert(absolutes, (void*)(uintptr_t)anOffset, (void*)z);
       [zoneDictionary setObject: self forKey: (NSString*)name];
     }
+  if (anOffset % 900 == 0)
+    {
+      int       index = anOffset/900 + 72;
+
+      if (nil == commonAbsolutes[index])
+        {
+          commonAbsolutes[index] = RETAIN(self);
+        }
+    }
   if (zone_mutex != nil)
     {
       [zone_mutex unlock];
@@ -1020,8 +1040,15 @@ static NSMapTable	*absolutes = 0;
 		    }
 		}
 	    }
-	  [md makeImmutableCopyOnFail: NO];
-	  abbreviationDictionary = md;
+          if ([md makeImmutable] == YES)
+            {
+              abbreviationDictionary = md;
+            }
+          else
+            {
+              abbreviationDictionary = [md copy];
+              RELEASE(md);
+            }
 	}
       [pool drain];
     }
@@ -1070,7 +1097,7 @@ static NSMapTable	*absolutes = 0;
       path = _time_zone_path (ABBREV_MAP, nil);
       if (path != nil)
 	{
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 	  unichar	mode[3];
 
 	  mode[0] = 'r';
@@ -1167,8 +1194,14 @@ static NSMapTable	*absolutes = 0;
 	  [ma addObject: the_name];
 	}
 
-      [md makeImmutableCopyOnFail: NO];
-      abbreviationMap = RETAIN(md); 
+      if ([md makeImmutable] == YES)
+        {
+          abbreviationMap = RETAIN(md); 
+        }
+      else
+        {
+          abbreviationMap = [md copy];
+        }
       [pool drain];
     }
   if (zone_mutex != nil)
@@ -1211,9 +1244,15 @@ static NSMapTable	*absolutes = 0;
 
 	  [ma addObjectsFromArray: names];
 	}
-
-      [ma makeImmutableCopyOnFail: NO];
-      namesArray = ma;
+      if ([ma makeImmutable] == YES)
+        {
+          namesArray = ma;
+        }
+      else
+        {
+          namesArray = [ma copy];
+          RELEASE(ma);
+        }
     }
   if (zone_mutex != nil)
     {
@@ -1456,7 +1495,7 @@ static NSMapTable	*absolutes = 0;
 	    }
 	}
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
       /*
        * Try to get timezone from windows system call.
        */
@@ -1801,7 +1840,7 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
       path = _time_zone_path (REGIONS_FILE, nil);
       if (path != nil)
 	{
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 	  unichar	mode[3];
 
 	  mode[0] = 'r';
@@ -1928,9 +1967,50 @@ localZoneString, [zone name], sign, s/3600, (s/60)%60);
 + (NSTimeZone*) timeZoneForSecondsFromGMT: (NSInteger)seconds
 {
   NSTimeZone	*zone;
+  int		sign = seconds >= 0 ? 1 : -1;
+  int           extra;
 
-  zone = [[GSAbsTimeZone alloc] initWithOffset: seconds name: nil];
-  return AUTORELEASE(zone);
+  /*
+   * Round the offset to the nearest minute, (for MacOS-X compatibility)
+   * and ensure it is no more than 18 hours.
+   */
+  seconds *= sign;
+  extra = seconds % 60;
+  if (extra < 30)
+    {
+      seconds -= extra;
+    }
+  else
+    {
+      seconds += 60 - extra;
+    }
+  if (seconds > 64800)
+    {
+      return nil;
+    }
+  seconds *= sign;
+  if (seconds % 900 == 0)
+    {
+      zone = commonAbsolutes[seconds/900 + 72];
+    }
+  else
+    {
+      if (zone_mutex != nil)
+        {
+          [zone_mutex lock];
+        }
+      zone = (NSTimeZone*)NSMapGet(absolutes, (void*)(uintptr_t)seconds);
+      if (zone_mutex != nil)
+        {
+          [zone_mutex unlock];
+        }
+    }
+  if (nil == zone)
+    {
+      zone = [[GSAbsTimeZone alloc] initWithOffset: seconds name: nil];
+      zone = AUTORELEASE(zone);
+    }
+  return zone;
 }
 
 /**
@@ -2464,7 +2544,7 @@ static NSString *zoneDirs[] = {
 @end
 
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 /* Timezone information data as stored in the registry */
 typedef struct TZI_format {
 	LONG       Bias;
@@ -2495,8 +2575,9 @@ lastDayOfGregorianMonth(int month, int year)
 
 /* IMPORT from NSCalendar date */
 void
-GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
-  NSInteger*hour, NSInteger*minute, NSInteger*second, NSInteger*mil);
+GSBreakTime(NSTimeInterval when,
+  NSInteger *year, NSInteger *month, NSInteger *day,
+  NSInteger *hour, NSInteger *minute, NSInteger *second, NSInteger *mil);
 
 
 @implementation GSWindowsTimeZone
@@ -2721,7 +2802,7 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
 
 - (BOOL) isDaylightSavingTimeForDate: (NSDate*)aDate
 {
-  int year, month, day, hour, minute, second, mil;
+  NSInteger year, month, day, hour, minute, second, mil;
   int	dow;
   int daylightdate, count, maxdate;
   NSTimeInterval when;
@@ -2903,7 +2984,7 @@ GSBreakTime(NSTimeInterval when, NSInteger*year, NSInteger*month, NSInteger*day,
   return [self name];
 }
 @end
-#endif // __MINGW__
+#endif // _WIN32
 
 
 @implementation	GSTimeZone
