@@ -25,7 +25,7 @@
    Boston, MA 02111 USA.
 
    <title>NSURL class reference</title>
-   $Date: 2013-12-21 07:56:33 +0000 (Sat, 21 Dec 2013) $ $Revision: 37489 $
+   $Date: 2016-03-09 13:16:16 +0000 (Wed, 09 Mar 2016) $ $Revision: 39492 $
 */
 
 /*
@@ -1495,7 +1495,7 @@ static NSUInteger	urlAlign;
       unescape(buf, buf);
     }
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   /* On windows a file URL path may be of the form C:\xxx (ie we should
    * not insert the leading slash).
    * Also the vertical bar symbol may have been used instead of the
