@@ -36,6 +36,7 @@
 #import "GSTLS.h"
 #import "GSURLPrivate.h"
 #import "GNUstepBase/GSMime.h"
+#import "GNUstepBase/NSData+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 #import "GNUstepBase/NSURL+GNUstepBase.h"
 
@@ -46,6 +47,7 @@
 #endif
 #define	USE_ZLIB	0
 
+
 #if	USE_ZLIB
 #if	defined(HAVE_ZLIB_H)
 #include	<zlib.h>
@@ -66,6 +68,78 @@ zfree(void *opaque, void *mem)
 #endif
 #endif
 
+static void
+debugRead(id handle, int len, const unsigned char *ptr)
+{
+  int           pos;
+  uint8_t       *hex;
+  NSUInteger    hl;
+
+  hl = ((len + 2) / 3) * 4;
+  hex = malloc(hl + 1);
+  hex[hl] = '\0';
+  GSPrivateEncodeBase64(ptr, (NSUInteger)len, hex);
+
+  for (pos = 0; pos < len; pos++)
+    {
+      if (0 == ptr[pos])
+        {
+          NSData        *data;
+          char          *esc;
+
+          data = [[NSData alloc] initWithBytesNoCopy: (void*)ptr
+                                              length: len
+                                        freeWhenDone: NO];
+          esc = [data escapedRepresentation: 0];
+
+          NSLog(@"Read for %p of %d bytes (escaped) - '%s'\n<[%s]>",
+            handle, len, esc, hex); 
+          free(esc);
+          RELEASE(data);
+          free(hex);
+          return;
+        }
+    }
+  NSLog(@"Read for %p of %d bytes - '%*.*s'\n<[%s]>",
+    handle, len, len, len, ptr, hex); 
+  free(hex);
+}
+static void
+debugWrite(id handle, int len, const unsigned char *ptr)
+{
+  int           pos;
+  uint8_t       *hex;
+  NSUInteger    hl;
+
+  hl = ((len + 2) / 3) * 4;
+  hex = malloc(hl + 1);
+  hex[hl] = '\0';
+  GSPrivateEncodeBase64(ptr, (NSUInteger)len, hex);
+
+  for (pos = 0; pos < len; pos++)
+    {
+      if (0 == ptr[pos])
+        {
+          NSData        *data;
+          char          *esc;
+
+          data = [[NSData alloc] initWithBytesNoCopy: (void*)ptr
+                                              length: len
+                                        freeWhenDone: NO];
+          esc = [data escapedRepresentation: 0];
+          NSLog(@"Write for %p of %d bytes (escaped) - '%s'\n<[%s]>",
+            handle, len, esc, hex); 
+          free(esc);
+          RELEASE(data);
+          free(hex);
+          return;
+        }
+    }
+  NSLog(@"Write for %p of %d bytes - '%*.*s'\n<[%s]>",
+    handle, len, len, len, ptr, hex); 
+  free(hex);
+}
+
 @interface	GSSocketStreamPair : NSObject
 {
   NSInputStream		*ip;
@@ -278,7 +352,6 @@ static NSLock		*pairLock = nil;
   NSURLCredential		*_credential;
   NSHTTPURLResponse		*_response;
 }
-- (void) setDebug: (BOOL)flag;
 @end
 
 @interface _NSHTTPSURLProtocol : _NSHTTPURLProtocol
@@ -398,28 +471,6 @@ static NSURLProtocol	*placeholder = nil;
   return NO;
 }
 
-+ (Class) _classToHandleRequest:(NSURLRequest *)request
-{
-  Class protoClass = nil;
-  int count;
-  [regLock lock];
-
-  count = [registered count];
-  while (count-- > 0)
-    {
-      Class	proto = [registered objectAtIndex: count];
-
-      if ([proto canInitWithRequest: request] == YES)
-	{
-	  protoClass = proto;
-	  break;
-	}
-    }
-  [regLock unlock];
-  return protoClass;
-}
-
-
 + (void) setProperty: (id)value
 	      forKey: (NSString *)key
 	   inRequest: (NSMutableURLRequest *)request
@@ -544,17 +595,33 @@ static NSURLProtocol	*placeholder = nil;
   return this->request;
 }
 
-/* This method is here so that it's safe to set debug on any NSURLProtocol
- * even if the concrete subclass doesn't actually support debug logging.
- */
-- (void) setDebug: (BOOL)flag
+@end
+
+@implementation	NSURLProtocol (Private)
+
++ (Class) _classToHandleRequest:(NSURLRequest *)request
 {
-  return;
+  Class protoClass = nil;
+  int count;
+  [regLock lock];
+
+  count = [registered count];
+  while (count-- > 0)
+    {
+      Class	proto = [registered objectAtIndex: count];
+
+      if ([proto canInitWithRequest: request] == YES)
+	{
+	  protoClass = proto;
+	  break;
+	}
+    }
+  [regLock unlock];
+  return protoClass;
 }
 
 @end
 
-
 @implementation	NSURLProtocol (Subclassing)
 
 + (BOOL) canInitWithRequest: (NSURLRequest *)request
@@ -631,24 +698,13 @@ static NSURLProtocol	*placeholder = nil;
   [super dealloc];
 }
 
-- (id) init
-{
-  if (nil != (self = [super init]))
-    {
-      _debug = GSDebugSet(@"NSURLProtocol");
-    }
-  return self;
-}
-
-- (void) setDebug: (BOOL)flag
-{
-  _debug = flag;
-}
-
 - (void) startLoading
 {
   static NSDictionary *methods = nil;
 
+  _debug = GSDebugSet(@"NSURLProtocol");
+  if (YES == [this->request _debug]) _debug = YES;
+
   if (methods == nil)
     {
       methods = [[NSDictionary alloc] initWithObjectsAndKeys: 
@@ -887,8 +943,7 @@ static NSURLProtocol	*placeholder = nil;
     }
   if (_debug)
     {
-      NSLog(@"%@ read %d bytes: '%*.*s'",
-	self, readCount, readCount, readCount, buffer);
+      debugRead(self, readCount, buffer);
     }
 
   if (_parser == nil)
@@ -1508,8 +1563,7 @@ static NSURLProtocol	*placeholder = nil;
 		    {
 		      if (_debug == YES)
 		        {
-			  NSLog(@"%@ wrote %d bytes: '%*.*s'", self, written,
-			    written, written, bytes + _writeOffset);
+                          debugWrite(self, written, bytes + _writeOffset);
 			}
 		      _writeOffset += written;
 		      if (_writeOffset >= len)
@@ -1566,8 +1620,7 @@ static NSURLProtocol	*placeholder = nil;
 			    {
 			      if (_debug == YES)
 				{
-				  NSLog(@"%@ wrote %d bytes: '%*.*s'", self,
-				    written, written, written, buffer);
+                                  debugWrite(self, written, buffer);
 				}
 			      len -= written;
 			      if (len > 0)
@@ -1579,6 +1632,17 @@ static NSURLProtocol	*placeholder = nil;
 				    buffer + written length: len];
 				  _writeOffset = 0;
 				}
+			      else if (len == 0 && ![_body hasBytesAvailable])
+				{
+				  /* all _body's bytes are read and written
+                                   * so we shouldn't wait for another
+                                   * opportunity to close _body and set
+				   * the flag 'sent'.
+				   */
+				  [_body close];
+				  DESTROY(_body);
+				  sent = YES;
+				}
 			    }
                           else if ([this->output streamStatus]
                             == NSStreamStatusWriting)
