@@ -24,7 +24,7 @@
    Boston, MA 02111 USA.
 
    <title>NSUserDefaults class reference</title>
-   $Date: 2014-07-28 15:40:29 +0100 (Mon, 28 Jul 2014) $ $Revision: 38018 $
+   $Date: 2016-03-18 14:56:55 +0000 (Fri, 18 Mar 2016) $ $Revision: 39565 $
 */
 
 #import "common.h"
@@ -56,7 +56,7 @@
 #import "GNUstepBase/NSProcessInfo+GNUstepBase.h"
 #import "GNUstepBase/NSString+GNUstepBase.h"
 
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
 @class	NSUserDefaultsWin32;
 #endif
 
@@ -88,8 +88,10 @@ static NSString		*GSPrimaryDomain = @"GSPrimaryDomain";
 static NSString		*defaultsFile = @".GNUstepDefaults";
 
 static NSUserDefaults	*sharedDefaults = nil;
+static NSDictionary     *argumentsDictionary = nil;
 static NSMutableString	*processName = nil;
 static NSRecursiveLock	*classLock = nil;
+static NSLock	        *syncLock = nil;
 
 /* Flag to say whether the sharedDefaults variable has been set up by a
  * call to the +standardUserDefaults method.  If this is YES but the variable
@@ -97,10 +99,10 @@ static NSRecursiveLock	*classLock = nil;
  * have no defaults available.
  */
 static BOOL		hasSharedDefaults = NO;
-/*
- * Caching some defaults.
+
+/* Caching some default flag values.  Until the standard defaults have
+ * been loaded, these values are taken from the process arguments.
  */
-static BOOL     parsingArguments = NO;
 static BOOL	flags[GSUserDefaultMaxFlag] = { 0 };
 
 /* An instance of the GSPersistentDomain class is used to encapsulate
@@ -114,19 +116,23 @@ static BOOL	flags[GSUserDefaultMaxFlag] = { 0 };
 @interface	GSPersistentDomain : NSObject
 {
   NSString		*name;
+  NSString		*path;
   NSUserDefaults	*owner;
-  NSDate		*updated;
-@public
-  BOOL			modified;
   NSMutableDictionary	*contents;
+  NSMutableSet          *added;
+  NSMutableSet          *modified;
+  NSMutableSet          *removed;
+  BOOL                  loaded;
 }
-- (NSMutableDictionary*) contents;
+- (NSDictionary*) contents;
+- (void) empty;
 - (id) initWithName: (NSString*)n
 	      owner: (NSUserDefaults*)o;
 - (NSString*) name;
-- (void) setContents: (NSDictionary*)domain;
+- (id) objectForKey: (NSString*)aKey;
+- (BOOL) setObject: (id)anObject forKey: (NSString*)aKey;
+- (BOOL) setContents: (NSDictionary*)domain;
 - (BOOL) synchronize;
-- (NSDate*) updated;
 @end
 
 static NSString *
@@ -244,6 +250,8 @@ updateCache(NSUserDefaults *self)
 	    }
 	}
 
+      /* NB the following flags are first set up, in the +initialize method.
+       */
       flags[GSMacOSXCompatible]
 	= [self boolForKey: @"GSMacOSXCompatible"];
       flags[GSOldStyleGeometry]
@@ -429,7 +437,7 @@ newLanguages(NSArray *oldNames)
  *** Private method definitions
  *************************************************************************/
 @interface NSUserDefaults (Private)
-- (NSDictionary*) _createArgumentDictionary;
++ (void) _createArgumentDictionary: (NSArray*)args;
 - (void) _changePersistentDomain: (NSString*)domainName;
 - (NSString*) _directory;
 - (BOOL) _lockDefaultsFile: (BOOL*)wasLocked;
@@ -547,13 +555,20 @@ newLanguages(NSArray *oldNames)
 {
   DESTROY(sharedDefaults);
   DESTROY(processName);
+  DESTROY(argumentsDictionary);
   DESTROY(classLock);
+  DESTROY(syncLock);
 }
 
 + (void) initialize
 {
   if (self == [NSUserDefaults class])
     {
+      CREATE_AUTORELEASE_POOL(pool);
+      NSEnumerator      *enumerator;
+      NSArray           *args;
+      NSString          *key;
+
       nextObjectSel = @selector(nextObject);
       objectForKeySel = @selector(objectForKey:);
       addSel = @selector(addEntriesFromDictionary:);
@@ -567,8 +582,72 @@ newLanguages(NSArray *oldNames)
       NSNumberClass = [NSNumber class];
       NSMutableDictionaryClass = [NSMutableDictionary class];
       NSStringClass = [NSString class];
-      classLock = [GSLazyRecursiveLock new];
+      argumentsDictionary = [NSDictionary new];
       [self registerAtExit];
+
+      /* Initialise the defaults flags to take values from the
+       * process arguments.  These are otherwise set in updateCache()
+       * We do this early on so that the boolean argument settings can
+       * be used while parsing property list values of other args in
+       * the +_createArgumentDictionary: method.
+       */
+      args = [[NSProcessInfo processInfo] arguments];
+      enumerator = [[[NSProcessInfo processInfo] arguments] objectEnumerator];
+      [enumerator nextObject];	// Skip process name.
+      while (nil != (key = [enumerator nextObject]))
+        {
+          if ([key hasPrefix: @"-"] == YES && [key isEqual: @"-"] == NO)
+	    {
+              id        val;
+
+	      /* Anything beginning with a '-' is a defaults key and we
+               * must strip the '-' from it.
+               */
+	      key = [key substringFromIndex: 1];
+	      while (nil != (val = [enumerator nextObject]))
+                {
+                  if ([val hasPrefix: @"-"] == YES && [val isEqual: @"-"] == NO)
+                    {
+                      key = val;
+                    }
+                  else if ([key isEqualToString: @"GSMacOSXCompatible"])
+                    {
+                      flags[GSMacOSXCompatible] = [val boolValue];
+                    }
+                  else if ([key isEqualToString: @"GSOldStyleGeometry"])
+                    {
+                      flags[GSOldStyleGeometry] = [val boolValue];
+                    }
+                  else if ([key isEqualToString: @"GSLogSyslog"])
+                    {
+                      flags[GSLogSyslog] = [val boolValue];
+                    }
+                  else if ([key isEqualToString: @"GSLogThread"])
+                    {
+                      flags[GSLogThread] = [val boolValue];
+                    }
+                  else if ([key isEqualToString: @"GSLogOffset"])
+                    {
+                      flags[GSLogOffset] = [val boolValue];
+                    }
+                  else if ([key isEqual: @"NSWriteOldStylePropertyLists"])
+                    {
+                      flags[NSWriteOldStylePropertyLists] = [val boolValue];
+                    }
+	        }
+	    }
+        }
+      /* The classLock must be created after setting up the flags[] array,
+       * so once it exists we know we can used them safely.
+       */
+      classLock = [NSRecursiveLock new];
+
+      /* This lock protects locking the defaults file.
+       */
+      syncLock = [NSLock new];
+
+      [self _createArgumentDictionary: args];
+      DESTROY(pool);
     }
 }
 
@@ -733,7 +812,6 @@ newLanguages(NSArray *oldNames)
   BOOL		added_lang;
   BOOL		added_locale;
   BOOL		setup;
-  BOOL		flag;
   id		lang;
   NSArray	*nL;
   NSArray	*uL;
@@ -761,7 +839,7 @@ newLanguages(NSArray *oldNames)
        * instance locked ourselves at the point when it first becomes
        * visible to other threads.
        */
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
       {
         NSString	*path = GSDefaultsRootForUser(NSUserName());
         NSRange		r = [path rangeOfString: @":REGISTRY:"];
@@ -804,77 +882,27 @@ newLanguages(NSArray *oldNames)
 
       if (nil == defs)
 	{
+	  const unsigned        retryCount = 100;
+	  const NSTimeInterval  retryInterval = 0.1;
+	  unsigned              i;
+
+	  for (i = 0; i < retryCount; i++)
+	    {
+	      [NSThread sleepForTimeInterval: retryInterval];
+	      [classLock lock];
+	      defs = RETAIN(sharedDefaults);
+	      setup = hasSharedDefaults;
+	      [classLock unlock];
+	      if (YES == setup)
+		{
+		  NS_VALRETURN(AUTORELEASE(defs));
+		}
+              RELEASE(defs);
+	    }
 	  NSLog(@"WARNING - unable to create shared user defaults!\n");
 	  NS_VALRETURN(nil);
 	}
 
-      if (NO == [defs _readOnly]
-	&& YES == [defs _lockDefaultsFile: &flag])
-	{
-	  NSFileManager	*mgr = [NSFileManager defaultManager];
-	  NSString	*path;
-
-	  path = [defs _directory];
-	  path = [path stringByAppendingPathComponent: defaultsFile];
-	  if (YES == [mgr isReadableFileAtPath: path])
-	    {
-	      NSString	*bck = [path stringByAppendingPathExtension: @"bck"];
-
-	      if (NO == [mgr isReadableFileAtPath: bck])
-		{
-		  NSData	*data;
-		  NSDictionary	*d = nil;
-
-		  /* An old style defaults file was found,
-		   * and no backup had been made of it, so
-		   * we make a backup and convert it to a
-		   * new style collection of files.
-		   */
-		  data = [NSData dataWithContentsOfFile: path];
-		  if (nil != data)
-		    {
-		      d = [NSPropertyListSerialization
-			propertyListWithData: data
-			options: NSPropertyListImmutable
-			format: 0
-			error: 0];
-		    }
-		  if ([d isKindOfClass: [NSDictionary class]])
-		    {
-		      NSEnumerator	*e;
-		      NSString	*name;
-
-		      [mgr movePath: path toPath: bck handler: nil];
-		      e = [d keyEnumerator];
-		      while (nil != (name = [e nextObject]))
-			{
-			  NSDictionary	*domain = [d objectForKey: name];
-
-			  path = [[defs _directory]
-			    stringByAppendingPathComponent: name];
-			  path = [path
-			    stringByAppendingPathExtension: @"plist"];
-			  if ([domain isKindOfClass: [NSDictionary class]]
-			    && [domain count] > 0
-			    && NO == [mgr fileExistsAtPath: path])
-			    {
-			      writeDictionary(domain, path);
-			    }
-			}
-		    }
-		  else
-		    {
-		      fprintf(stderr, "Found unparseable file at '%s'\n",
-			[path UTF8String]);
-		    }
-		}
-	    }
-	  if (NO == flag)
-	    {
-	      [defs _unlockDefaultsFile];
-	    }
-	}
-
       /*
        * Set up search list (excluding language list, which we don't know yet)
        */
@@ -1121,7 +1149,7 @@ newLanguages(NSArray *oldNames)
     }
 
   r = [path rangeOfString: @":INTERNAL:"];
-#if	defined(__MINGW__)
+#if	defined(_WIN32)
   if (r.length == 0)
     {
       r = [path rangeOfString: @":REGISTRY:"];
@@ -1158,8 +1186,7 @@ newLanguages(NSArray *oldNames)
 
   // Create volatile defaults and add the Argument and the Registration domains
   _tempDomains = [[NSMutableDictionaryClass alloc] initWithCapacity: 10];
-  [_tempDomains setObject: [self _createArgumentDictionary]
-		   forKey: NSArgumentDomain];
+  [_tempDomains setObject: argumentsDictionary forKey: NSArgumentDomain];
   [_tempDomains
     setObject: [NSMutableDictionaryClass dictionaryWithCapacity: 10]
     forKey: NSRegistrationDomain];
@@ -1350,7 +1377,7 @@ newLanguages(NSArray *oldNames)
           NSDictionary		*td;
 
           pd = (*pImp)(_persDomains, objectForKeySel, dN);
-          if (pd != nil && (object = [pd->contents objectForKey: defaultName]))
+          if (pd != nil && (object = [pd objectForKey: defaultName]))
 	    break;
           td = (*tImp)(_tempDomains, objectForKeySel, dN);
           if (td != nil && (object = [td objectForKey: defaultName]))
@@ -1377,12 +1404,8 @@ newLanguages(NSArray *oldNames)
 
       if (nil != pd)
 	{
-	  id	obj = [pd->contents objectForKey: defaultName];
-
-	  if (nil != obj)
+          if ([pd setObject: nil forKey: defaultName])
 	    {
-	      pd->modified = YES;
-	      [pd->contents removeObjectForKey: defaultName];
 	      [self _changePersistentDomain: processName];
 	    }
 	}
@@ -1517,9 +1540,10 @@ static BOOL isPlistObject(id o)
           [_persDomains setObject: pd forKey: processName];
 	  [pd release];
 	}
-      pd->modified = YES;
-      [pd->contents setObject: value forKey: defaultName];
-      [self _changePersistentDomain: processName];
+      if ([pd setObject: value forKey: defaultName])
+        {
+          [self _changePersistentDomain: processName];
+        }
       [_lock unlock];
     }
   NS_HANDLER
@@ -1667,14 +1691,8 @@ static BOOL isPlistObject(id o)
       pd = [_persDomains objectForKey: domainName];
       if (nil != pd)
         {
-	  if (YES == [domainName isEqualToString: NSGlobalDomain])
-	    {
-	      /* Don't remove the global domain, just its contents.
-	       */
-	      [pd->contents removeAllObjects];
-	      pd->modified = YES;
-	    }
-	  else
+          [pd empty];
+	  if (NO == [domainName isEqualToString: NSGlobalDomain])
 	    {
 	      /* Remove the domain entirely.
 	       */
@@ -1771,7 +1789,8 @@ static BOOL isPlistObject(id o)
 - (BOOL) synchronize
 {
   NSDate		*saved;
-  BOOL			wasLocked;
+  BOOL			isLocked = NO;
+  BOOL			wasLocked = NO;
   BOOL			result = YES;
   BOOL			haveChange = NO;
 
@@ -1807,6 +1826,7 @@ static BOOL isPlistObject(id o)
 	      NSEnumerator		*enumerator;
 	      NSString			*domainName;
 
+              isLocked = YES;
 	      haveChange = [self _readDefaults];
 	      if (YES == haveChange)
 		{
@@ -1849,8 +1869,9 @@ static BOOL isPlistObject(id o)
 		{
 		  updateCache(self);
 		}
-	      if (NO == wasLocked)
+	      if (YES == isLocked && NO == wasLocked)
 		{
+                  isLocked = NO;
 		  [self _unlockDefaultsFile];
 		}
 	    }
@@ -1860,6 +1881,11 @@ static BOOL isPlistObject(id o)
     {
       [_lastSync release];
       _lastSync = saved;
+      if (YES == isLocked && NO == wasLocked)
+        {
+          isLocked = NO;
+          [self _unlockDefaultsFile];
+        }
       [_lock unlock];
       [localException raise];
     }
@@ -2031,7 +2057,7 @@ static BOOL isPlistObject(id o)
 	      pd = (*pImp)(_persDomains, objectForKeySel, obj);
 	      if (nil != pd)
 		{
-		  dict = pd->contents;
+		  dict = [pd contents];
 		}
 	      else
 		{
@@ -2042,10 +2068,9 @@ static BOOL isPlistObject(id o)
                   (*addImp)(dictRep, addSel, dict);
                 }
 	    }
-          [dictRep makeImmutableCopyOnFail: NO];
-          _dictionaryRep = dictRep;
+          _dictionaryRep = GS_IMMUTABLE(dictRep);
         }
-      rep = [[_dictionaryRep retain] autorelease];
+      rep = AUTORELEASE(RETAIN(_dictionaryRep));
       [_lock unlock];
     }
   NS_HANDLER
@@ -2111,9 +2136,18 @@ static BOOL isPlistObject(id o)
 BOOL
 GSPrivateDefaultsFlag(GSUserDefaultFlagType type)
 {
-  if (nil == sharedDefaults && NO == parsingArguments)
+  if (nil == classLock)
     {
-      [NSUserDefaults standardUserDefaults];
+      /* The order of +initialise of NSUserDefaults is such that our
+       * flags[] array is set up directly from the process arguments
+       * before classLock is created, so once * that variable exists
+       * this function may be used safely.
+       */
+      [NSUserDefaults class];
+      if (NO == hasSharedDefaults)
+        {
+          [NSUserDefaults standardUserDefaults];
+        }
     }
   return flags[type];
 }
@@ -2129,7 +2163,7 @@ NSDictionary *GSPrivateDefaultLocale()
   NSDictionary	        *locale = nil;
   NSUserDefaults        *defs = nil;
 
-  if (classLock == nil)
+  if (nil == classLock)
     {
       [NSUserDefaults standardUserDefaults];
     }
@@ -2156,24 +2190,16 @@ NSDictionary *GSPrivateDefaultLocale()
 
 @implementation NSUserDefaults (Private)
 
-- (NSDictionary*) _createArgumentDictionary
++ (void) _createArgumentDictionary: (NSArray*)args
 {
-  NSArray	*args;
   NSEnumerator	*enumerator;
   NSMutableDictionary *argDict = nil;
   BOOL		done;
   id		key, val;
 
-  [_lock lock];
-  if (YES == parsingArguments)
-    {
-      [_lock unlock];
-      return nil;       // Prevent recursion
-    }
-  parsingArguments = YES;
+  [classLock lock];
   NS_DURING
     {
-      args = [[NSProcessInfo processInfo] arguments];
       enumerator = [args objectEnumerator];
       argDict = [NSMutableDictionaryClass dictionaryWithCapacity: 2];
       [enumerator nextObject];	// Skip process name.
@@ -2181,40 +2207,35 @@ NSDictionary *GSPrivateDefaultLocale()
 
       while (done == NO)
         {
-          if ([key hasPrefix: @"-"] == YES && [key isEqual: @"-"] == NO)
+          /* Any value with a leading '-' may be the name of a default
+           * in the argument domain.
+           * NB. Testing on OSX shows that this includes a single '-'
+           * (where the key is an empty string), but GNUstep disallows
+           * en empty string as a key (so it can be a value).
+           */
+          if ([key hasPrefix: @"-"] == YES
+            && [key isEqual: @"-"] == NO)
 	    {
-	      NSString	*old = nil;
-
-	      /* anything beginning with a '-' is a defaults key and we
-               * must strip the '-' from it.
-               * As a special case, we leave the '- in place for '-GS...'
-               * and '--GS...' for backward compatibility.
+	      /* Strip the '-' before the defaults key, and get the
+               * corresponding value (the next argument).
                */
-	      if ([key hasPrefix: @"-GS"] == YES
-                || [key hasPrefix: @"--GS"] == YES)
-	        {
-	          old = key;
-	        }
 	      key = [key substringFromIndex: 1];
 	      val = [enumerator nextObject];
-	      if (val == nil)
-	        {            // No more args
-	          [argDict setObject: @"" forKey: key];		// arg is empty.
-	          if (old != nil)
-		    {
-		      [argDict setObject: @"" forKey: old];
-		    }
+	      if (nil == val)
+	        {
+                  /* No more arguments and no value ... arg is not set.
+                   */
 	          done = YES;
 	          continue;
 	        }
 	      else if ([val hasPrefix: @"-"] == YES
                 && [val isEqual: @"-"] == NO)
-	        {  // Yet another argument
-	          [argDict setObject: @"" forKey: key];		// arg is empty.
-	          if (old != nil)
-		    {
-		      [argDict setObject: @"" forKey: old];
-		    }
+	        {
+                  /* Value is actually an argument key ...
+                   * current key is not used (behavior matches OSX).
+                   * NB. GNUstep allows a '-' as the value for a default,
+                   * but OSX does not.
+                   */
 	          key = val;
 	          continue;
 	        }
@@ -2227,15 +2248,26 @@ NSDictionary *GSPrivateDefaultLocale()
 		     foreign environment. */
 	          NSObject *plist_val;
 
-	          NS_DURING
-		    {
-		      plist_val = [val propertyList];
-		    }
-	          NS_HANDLER
-		    {
-		      plist_val = val;
-		    }
-	          NS_ENDHANDLER
+                  NS_DURING
+                    {
+                      NSData		        *data;
+
+                      data = [val dataUsingEncoding: NSUTF8StringEncoding];
+                      plist_val = [NSPropertyListSerialization
+                        propertyListFromData: data
+                        mutabilityOption: NSPropertyListMutableContainers
+                        format: 0
+                        errorDescription: 0];
+                      if (nil == plist_val)
+                        {
+                          plist_val = val;
+                        }
+                    }
+                  NS_HANDLER
+                    {
+                      plist_val = val;
+                    }
+                  NS_ENDHANDLER
 
 	          /* Make sure we don't crash being caught adding nil to
                      a dictionary. */
@@ -2245,25 +2277,19 @@ NSDictionary *GSPrivateDefaultLocale()
 		    }
 
 	          [argDict setObject: plist_val  forKey: key];
-	          if (old != nil)
-		    {
-		      [argDict setObject: plist_val  forKey: old];
-		    }
 	        }
 	    }
           done = ((key = [enumerator nextObject]) == nil);
         }
-      parsingArguments = NO;
-      [_lock unlock];
+      ASSIGNCOPY(argumentsDictionary, argDict);
+      [classLock unlock];
     }
   NS_HANDLER
     {
-      parsingArguments = NO;
-      [_lock unlock];
+      [classLock unlock];
       [localException raise];
     }
   NS_ENDHANDLER
-  return argDict;
 }
 
 - (void) _changePersistentDomain: (NSString*)domainName
@@ -2303,52 +2329,65 @@ NSDictionary *GSPrivateDefaultLocale()
 static BOOL isLocked = NO;
 - (BOOL) _lockDefaultsFile: (BOOL*)wasLocked
 {
-  *wasLocked = isLocked;
-  if (isLocked == NO && _fileLock != nil)
+  [syncLock lock];
+  NS_DURING
     {
-      NSDate	*started = [NSDateClass date];
+      *wasLocked = isLocked;
+      if (NO == isLocked && _fileLock != nil)
+        {
+          NSDate	*started = [NSDateClass date];
 
-      while ([_fileLock tryLock] == NO)
-	{
-	  NSAutoreleasePool	*arp = [NSAutoreleasePool new];
-	  NSDate		*when;
-	  NSDate		*lockDate;
+          while ([_fileLock tryLock] == NO)
+            {
+              CREATE_AUTORELEASE_POOL(arp);
+              NSDate		*when;
+              NSDate		*lockDate;
 
-	  lockDate = [_fileLock lockDate];
-	  when = [NSDateClass dateWithTimeIntervalSinceNow: 0.1];
+              lockDate = [_fileLock lockDate];
+              when = [NSDateClass dateWithTimeIntervalSinceNow: 0.1];
 
-	  /*
-	   * In case we have tried and failed to break the lock,
-	   * we give up after a while ... 16 seconds should give
-	   * us three lock breaks if we do them at 5 second
-	   * intervals.
-	   */
-	  if ([when timeIntervalSinceDate: started] > 16.0)
-	    {
-	      NSLog(@"Failed to lock user defaults database even after "
-		@"breaking old locks!");
-	      [arp drain];
-	      return NO;
-	    }
+              /*
+               * In case we have tried and failed to break the lock,
+               * we give up after a while ... 66 seconds should give
+               * us three lock breaks if we do them at 20 second
+               * intervals.
+               */
+              if ([when timeIntervalSinceDate: started] > 66.0)
+                {
+                  fprintf(stderr, "Failed to lock user defaults database"
+                    " even after breaking old locks!\n");
+                  RELEASE(arp);
+                  break;
+                }
 
-	  /*
-	   * If lockDate is nil, we should be able to lock again ... but we
-	   * wait a little anyway ... so that in the case of a locking
-	   * problem we do an idle wait rather than a busy one.
-	   */
-	  if (lockDate != nil && [when timeIntervalSinceDate: lockDate] > 5.0)
-	    {
-	      [_fileLock breakLock];
-	    }
-	  else
-	    {
-	      [NSThread sleepUntilDate: when];
-	    }
-	  [arp drain];
-	}
-      isLocked = YES;
+              /* If lockDate is nil, we should be able to lock again ... but we
+               * wait a little anyway ... so that in the case of a locking
+               * problem we do an idle wait rather than a busy one.
+               */
+              if (lockDate != nil
+                && [when timeIntervalSinceDate: lockDate] > 20.0)
+                {
+                  NSLog(@"NSUserdefaults file lock at %@ is dated %@ ... break",
+                    _fileLock, lockDate);
+                  [_fileLock breakLock];
+                }
+              else
+                {
+                  [NSThread sleepUntilDate: when];
+                }
+              RELEASE(arp);
+            }
+          isLocked = YES;
+        }
+      [syncLock unlock];
     }
-   return YES;
+  NS_HANDLER
+    {
+      [syncLock unlock];
+      [localException raise];
+    }
+  NS_ENDHANDLER
+  return isLocked;
 }
 
 - (BOOL) _readDefaults
@@ -2418,27 +2457,32 @@ static BOOL isLocked = NO;
 
 - (void) _unlockDefaultsFile
 {
+  [syncLock lock];
   NS_DURING
     {
-      [_fileLock unlock];
+      if (YES == isLocked)
+        {  
+          [_fileLock unlock];
+        }
     }
   NS_HANDLER
     {
-      NSLog(@"Warning ... someone broke our lock (%@) ... and may have"
-        @" interfered with updating defaults data in file.",
-        lockPath(_defaultsDatabase, NO));
+      fprintf(stderr, "Warning ... someone broke our lock (%s) ... and may have"
+        " interfered with updating defaults data in file.",
+        [lockPath(_defaultsDatabase, NO) UTF8String]);
     }
   NS_ENDHANDLER
   isLocked = NO;
+  [syncLock unlock];
 }
 
 @end
 
 @implementation	GSPersistentDomain
 
-- (NSMutableDictionary*) contents
+- (NSDictionary*) contents
 {
-  if (nil == updated)
+  if (NO == loaded)
     {
       [self synchronize];
     }
@@ -2447,20 +2491,48 @@ static BOOL isLocked = NO;
 
 - (void) dealloc
 {
+  DESTROY(added);
+  DESTROY(removed);
+  DESTROY(modified);
   DESTROY(contents);
-  DESTROY(updated);
   DESTROY(name);
+  DESTROY(path);
   [super dealloc];
 }
 
+- (void) empty
+{
+  if (NO == loaded)
+    {
+      [self synchronize];
+    }
+  if ([contents count] > 0)
+    {
+      NSEnumerator      *e;
+      NSString          *k;
+
+      e = [[contents allKeys] objectEnumerator];
+      while (nil != (k = [e nextObject]))
+        {
+          [self setObject: nil forKey: k];
+        }
+      [self synchronize];
+    }
+}
+
 - (id) initWithName: (NSString*)n
 	      owner: (NSUserDefaults*)o
 {
   if (nil != (self = [super init]))
     {
-      name = [n copy];
       owner = o;	// Not retained
+      name = [n copy];
+      path = RETAIN([[[owner _directory] stringByAppendingPathComponent: name]
+        stringByAppendingPathExtension: @"plist"]);
       contents = [NSMutableDictionary new];
+      added = [NSMutableSet new];
+      removed = [NSMutableSet new];
+      modified = [NSMutableSet new];
     }
   return self;
 }
@@ -2470,113 +2542,216 @@ static BOOL isLocked = NO;
   return name;
 }
 
-- (void) setContents: (NSDictionary*)domain
+- (id) objectForKey: (NSString*)aKey
 {
+  return [contents objectForKey: aKey];
+}
+
+- (BOOL) setContents: (NSDictionary*)domain
+{
+  BOOL  changed = NO;
+
   if (NO == [contents isEqual: domain])
     {
-      NSMutableDictionary	*m = [domain mutableCopy];
+      NSEnumerator      *e;
+      NSString          *k;
 
-      if (nil == m)
+      e = [[contents allKeys] objectEnumerator];
+      while (nil != (k = [e nextObject]))
 	{
-	  m = [NSMutableDictionary new];
+	  if ([domain objectForKey: k] == nil)
+            {
+              [self setObject: nil forKey: k];
+            }
 	}
-      [contents release];
-      contents = m;
-      updated = [NSDate new];
-      modified = YES;
+      e = [domain keyEnumerator];
+      while (nil != (k = [e nextObject]))
+	{
+          [self setObject: [domain objectForKey: k] forKey: k];
+        }
+      changed = YES;
+    }
+  return changed;
+}
+
+- (BOOL) setObject: (id)anObject forKey: (NSString*)aKey
+{
+  if (nil == anObject)
+    {
+      if (nil == [contents objectForKey: aKey])
+        {
+          return NO;
+        }
+      if ([added member: aKey])
+        {
+          [added removeObject: aKey];
+        }
+      else if ([modified member: aKey])
+        {
+          [modified removeObject: aKey];
+          [removed addObject: aKey];
+        }
+      else
+        {
+          [removed addObject: aKey];
+        }
+      [contents removeObjectForKey: aKey];
+      return YES;
+    }
+  else
+    {
+      id        old = [contents objectForKey: aKey];
+
+      if ([anObject isEqual: old])
+        {
+          return NO;
+        }
+      if ([removed member: aKey])
+        {
+          [modified addObject: aKey];
+          [removed removeObject: aKey];
+        }
+      else if (nil == [modified member: aKey] && nil == [added member: aKey])
+        {
+          if (nil == old)
+            {
+              [added addObject: aKey];
+            }
+          else
+            {
+              [modified addObject: aKey];
+            }
+        }
+      [contents setObject: anObject forKey: aKey];
+      return YES;
     }
 }
 
 - (BOOL) synchronize
 {
-  BOOL  wasLocked;
-  BOOL	hadChange = NO; // Have we read a change from disk?
+  BOOL  isLocked = NO;
+  BOOL  wasLocked = NO;
+  BOOL  shouldLock = NO;
+  BOOL  defaultsChanged = NO;
+  BOOL  hasLocalChanges = NO;
 
-  if (NO == [owner _lockDefaultsFile: &wasLocked])
+  if ([removed count] || [added count] || [modified count])
     {
-      hadChange = NO;
-      wasLocked = NO;
+      hasLocalChanges = YES;
     }
-  else
+  if (YES == hasLocalChanges && NO == [owner _readOnly])
     {
-      NSString	*path;
-
-      path = [[[owner _directory] stringByAppendingPathComponent: name]
-	stringByAppendingPathExtension: @"plist"];
-
-      if (YES == modified && NO == [owner _readOnly])
-	{
-	  NSDate	*mod;
-          BOOL          result;
-
-          mod = [NSDate date];
-	  if (0 == [contents count])
-	    {
-	      /* Remove empty defaults dictionary.
-	       */
-	      result = writeDictionary(nil, path);
-	    }
-	  else
-	    {
-	      /* Write dictionary to file.
-	       */
-	      result = writeDictionary(contents, path);
-	    }
-	  if (YES == result)
-	    {
-	      ASSIGN(updated, mod);
-	      modified = NO;
-	    }
-	}
-      else
-	{
-	  NSFileManager	*mgr = [NSFileManager defaultManager];
-	  NSDate	*mod;
-	  
-	  /* If the database was modified since the last refresh
-	   * we need to read it.
-	   */
-	  mod = [[mgr fileAttributesAtPath: path traverseLink: YES]
-	    objectForKey: NSFileModificationDate];
-	  if (nil == updated
-	    || (nil != mod && [updated laterDate: mod] != updated))
-	    {
-	      ASSIGN(updated, mod);
-	      if (nil != updated)
-		{
-		  NSData	*data;
-
-		  data = [NSData dataWithContentsOfFile: path];
-		  if (nil != data)
-		    {
-		      id	o;
-
-		      o = [NSPropertyListSerialization
-			propertyListWithData: data
-			options: NSPropertyListImmutable
-			format: 0
-			error: 0];
-		      if ([o isKindOfClass: [NSDictionary class]])
-			{
-			  [contents release];
-			  contents = [o mutableCopy];
-			}
-		    }
-		}
-              hadChange = YES;
-	    }
-	}
-      if (NO == wasLocked)
-	{
-	  [owner _unlockDefaultsFile];
-	}
+      shouldLock = YES;
     }
-  return hadChange;
-}
+  if (YES == shouldLock && YES == [owner _lockDefaultsFile: &wasLocked])
+    {
+      isLocked = YES;
+    }
+  NS_DURING
+    {
+      NSFileManager	        *mgr;
+      NSMutableDictionary       *disk;
 
-- (NSDate*) updated
-{
-  return updated;
+      mgr = [NSFileManager defaultManager];
+      disk = nil;
+      if (YES == [mgr isReadableFileAtPath: path])
+        {
+          NSData	*data;
+
+          data = [NSData dataWithContentsOfFile: path];
+          if (nil != data)
+            {
+              id	o;
+
+              o = [NSPropertyListSerialization
+                propertyListWithData: data
+                options: NSPropertyListImmutable
+                format: 0
+                error: 0];
+              if ([o isKindOfClass: [NSDictionary class]])
+                {
+                  disk = AUTORELEASE([o mutableCopy]);
+                }
+            }
+        }
+      if (nil == disk)
+        {
+          disk = [NSMutableDictionary dictionary];
+        }
+      loaded = YES;
+
+      if (NO == [contents isEqual: disk])
+        {
+          defaultsChanged = YES;
+          if (YES == hasLocalChanges)
+            {
+              NSEnumerator  *e;
+              NSString      *k;
+
+              e = [removed objectEnumerator];
+              while (nil != (k = [e nextObject]))
+                {
+                  [disk removeObjectForKey: k];
+                }
+              e = [added objectEnumerator];
+              while (nil != (k = [e nextObject]))
+                {
+                  [disk setObject: [contents objectForKey: k] forKey: k];
+                }
+              e = [modified objectEnumerator];
+              while (nil != (k = [e nextObject]))
+                {
+                  [disk setObject: [contents objectForKey: k] forKey: k];
+                }
+            }
+          ASSIGN(contents, disk);
+        }
+      if (YES == hasLocalChanges)
+        {
+          BOOL  written = NO;
+
+          if (NO == [owner _readOnly])
+            {
+              if (YES == isLocked)
+                {
+                  if (0 == [contents count])
+                    {
+                      /* Remove empty defaults dictionary.
+                       */
+                      written = writeDictionary(nil, path);
+                    }
+                  else
+                    {
+                      /* Write dictionary to file.
+                       */
+                      written = writeDictionary(contents, path);
+                    }
+                }
+            }
+          if (YES == written)
+            {
+              [added removeAllObjects];
+              [removed removeAllObjects];
+              [modified removeAllObjects];
+            }
+        }
+      if (YES == isLocked && NO == wasLocked)
+        {
+          isLocked = NO;
+          [owner _unlockDefaultsFile];
+        }
+    }
+  NS_HANDLER
+    {
+      fprintf(stderr, "problem synchronising defaults domain '%s': %s\n",
+        [name UTF8String], [[localException description] UTF8String]);
+      if (YES == isLocked && NO == wasLocked)
+        {
+          [owner _unlockDefaultsFile];
+        }
+    }
+  NS_ENDHANDLER
+  return defaultsChanged;
 }
 
 @end
