@@ -22,6 +22,10 @@
    Boston, MA 02111 USA.
    */
 
+/* mingw wants winsock2.h before windows.h */
+#include <windows.h>
+#include <winsock2.h>
+
 #include "common.h"
 
 #import "Foundation/NSObject.h"
@@ -43,7 +47,6 @@
 #import "../GSPrivate.h"
 #import "../GSNetwork.h"
 
-#include <winsock2.h>
 #include <fcntl.h>
 #include <sys/file.h>
 #include <sys/types.h>
@@ -67,9 +70,9 @@
 #define	NETBUF_SIZE	4096
 #define	READ_SIZE	NETBUF_SIZE*10
 
-static GSFileHandle*	fh_stdin = nil;
-static GSFileHandle*	fh_stdout = nil;
-static GSFileHandle*	fh_stderr = nil;
+static GSFileHandle     *fh_stdin = nil;
+static GSFileHandle     *fh_stdout = nil;
+static GSFileHandle     *fh_stderr = nil;
 
 // Key to info dictionary for operation mode.
 static NSString*	NotificationKey = @"NSFileHandleNotificationKey";
@@ -236,6 +239,24 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 
 - (void) dealloc
 {
+  if (self == fh_stdin)
+    {
+      RETAIN(self);
+      [NSException raise: NSGenericException
+                  format: @"Attempt to deallocate standard input handle"];
+    }
+  if (self == fh_stdout)
+    {
+      RETAIN(self);
+      [NSException raise: NSGenericException
+                  format: @"Attempt to deallocate standard output handle"];
+    }
+  if (self == fh_stderr)
+    {
+      RETAIN(self);
+      [NSException raise: NSGenericException
+                  format: @"Attempt to deallocate standard error handle"];
+    }
   RELEASE(address);
   RELEASE(service);
   RELEASE(protocol);
@@ -279,7 +300,10 @@ getAddr(NSString* name, NSString* svc, NSString* pcl, struct sockaddr_in *sin)
 	      WSACloseEvent(event);
 	      event = WSA_INVALID_EVENT;
             }
-	  close(descriptor);
+	  else
+	    {
+	      close(descriptor);
+	    }
 	  descriptor = -1;
 	}
     }
@@ -1019,18 +1043,16 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   if (fh_stderr != nil)
     {
-      RETAIN(fh_stderr);
-      DESTROY(self);
+      ASSIGN(self, fh_stderr);
     }
   else
     {
       self = [self initWithFileDescriptor: 2 closeOnDealloc: NO];
-      fh_stderr = self;
-    }
-  self = fh_stderr;
-  if (self)
-    {
-      readOK = NO;
+      ASSIGN(fh_stderr, self);
+      if (self)
+        {
+          readOK = NO;
+        }
     }
   return self;
 }
@@ -1039,18 +1061,16 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   if (fh_stdin != nil)
     {
-      RETAIN(fh_stdin);
-      DESTROY(self);
+      ASSIGN(self, fh_stdin);
     }
   else
     {
       self = [self initWithFileDescriptor: 0 closeOnDealloc: NO];
-      fh_stdin = self;
-    }
-  self = fh_stdin;
-  if (self)
-    {
-      writeOK = NO;
+      ASSIGN(fh_stdin, self);
+      if (self)
+        {
+          writeOK = NO;
+        }
     }
   return self;
 }
@@ -1059,18 +1079,16 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 {
   if (fh_stdout != nil)
     {
-      RETAIN(fh_stdout);
-      DESTROY(self);
+      ASSIGN(fh_stdout, self);
     }
   else
     {
       self = [self initWithFileDescriptor: 1 closeOnDealloc: NO];
-      fh_stdout = self;
-    }
-  self = fh_stdout;
-  if (self)
-    {
-      readOK = NO;
+      ASSIGN(fh_stdout, self);
+      if (self)
+        {
+          readOK = NO;
+        }
     }
   return self;
 }
@@ -1280,13 +1298,13 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
   int			len;
 
   [self checkRead];
-  if (isNonBlocking == YES)
-    {
-      [self setNonBlocking: NO];
-    }
   d = [NSMutableData dataWithCapacity: 0];
   if (isStandardFile)
     {
+      if (isNonBlocking == YES)
+	{
+	  [self setNonBlocking: NO];
+	}
       while ((len = [self read: buf length: sizeof(buf)]) > 0)
         {
 	  [d appendBytes: buf length: len];
@@ -1294,7 +1312,41 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
     }
   else
     {
+      if (isNonBlocking == NO)
+	{
+	  [self setNonBlocking: YES];
+	}
       len = [self read: buf length: sizeof(buf)];
+
+      if (len <= 0)
+	{
+          if (WSAGetLastError()== WSAEINTR
+	    || WSAGetLastError()== WSAEWOULDBLOCK)
+	    {
+	      /*
+	       * Read would have blocked ... so try to get a single character
+	       * in non-blocking mode (to ensure we wait until data arrives)
+	       * and then try again.
+	       * This ensures that we block for *some* data as we should.
+	       */
+	      [self setNonBlocking: NO];
+	      len = [self read: buf length: 1];
+	      [self setNonBlocking: YES];
+	      if (len == 1)
+		{
+		  len = [self read: &buf[1] length: sizeof(buf) - 1];
+		  if (len <= 0)
+		    {
+		      len = 1;
+		    }
+		  else
+		    {
+		      len = len + 1;
+		    }
+		}
+	    }
+	}
+
       if (len > 0)
 	{
 	  [d appendBytes: buf length: len];
@@ -2315,6 +2367,10 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
 
 - (void) setNonBlocking: (BOOL)flag
 {
+  if (flag == isNonBlocking)
+    {
+      return;
+    }
   if (descriptor < 0)
     {
       return;
@@ -2335,7 +2391,7 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
         {                        // Not a file and not a socket, must be a pipe
           DWORD mode;
 
-          if (flag)
+          if (YES == flag)
             mode = PIPE_NOWAIT;
           else
             mode = PIPE_WAIT;
@@ -2346,34 +2402,39 @@ NSString * const GSSOCKSRecvAddr = @"GSSOCKSRecvAddr";
             }
           else
             {
-	      NSLog(@"unable to set pipe non-blocking mode - %d",
-		GetLastError());
+	      NSLog(@"unable to set pipe non-blocking mode to %s - %d",
+		(YES  == flag ? "YES" : "NO"), GetLastError());
             }
           return;
         }
 
-      if (flag)
+      if (YES == flag)
 	{
+	  WSAEventSelect((SOCKET)_get_osfhandle(descriptor), event,
+	    FD_ALL_EVENTS);
 	  dummy = 1;
 	  if (ioctlsocket((SOCKET)_get_osfhandle(descriptor), FIONBIO, &dummy)
 	    == SOCKET_ERROR)
 	    {
-	      NSLog(@"unable to set non-blocking mode - %@",
+	      NSLog(@"unable to set non-blocking mode to YES - %@",
 		[NSError _last]);
 	    }
 	  else
-	    isNonBlocking = flag;
+	    {
+	      isNonBlocking = flag;
+	    }
 	}
       else
 	{
+	  WSAEventSelect((SOCKET)_get_osfhandle(descriptor), event, 0);
 	  dummy = 0;
 	  if (ioctlsocket((SOCKET)_get_osfhandle(descriptor), FIONBIO, &dummy)
 	    == SOCKET_ERROR)
 	    {
-	      NSLog(@"unable to set blocking mode - %@", [NSError _last]);
+	      NSLog(@"unable to set blocking mode to NO - %@",
+		[NSError _last]);
 	    }
-	  else
-	    isNonBlocking = flag;
+          isNonBlocking = flag;
 	}
     }
 }
